/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.Layout;
import net.sf.dc.console.components.PictureField;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.resources.DCLabels;
import net.sf.ext.util.Resources;
import net.sf.ext.util.SwingUtil;
import net.sf.ext.util.Utilities;

/**
 * Displays all events
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class LogForm extends JFrame {

    private JTextArea text;
    private PictureField logo;
    private JLabel labelVersion;
    private JLabel labelProductName;
    private JLabel labelLastUpd;

    private JPanel panelLogging;
    private JPanel panelPicture;
    private JPanel panelProductInfo;

    /**
     * Initializes and builds the frame
     */
    public LogForm() {
        buildPanel();
        this.addMouseListener(new ActiveFrame());
    }

    /**
     * Creates the panel
     */
    private void buildPanel() {
        panelLogging        = new JPanel(false);
        panelPicture        = new JPanel(false);
        panelProductInfo    = new JPanel(false);

        // ************************************************************************
        // PICTURE PANEL
        // Set layout style
        panelPicture.setLayout(Layout.getGBL());

        // Initialize components
        logo = ComponentFactory.getPictureField(false);
        try {
            File file = new File("./icons_system/logo.jpg");
            logo.setValue(file.toURL());
        } catch (Exception exp) {
            new MessageBox(exp.toString(), MessageBox._ERROR);
        }

        // Build the panel
        panelPicture.add(logo,  Layout.getGBC( 0, 0, 1, 1, 10.0, 10.0
                               ,GridBagConstraints.NORTH, GridBagConstraints.BOTH,
                                new Insets(0, 5, 0, 5), 0, 0));


        // ************************************************************************
        // SEARCH OPTION PANEL
        // Set layout style
        panelPicture.setLayout(Layout.getGBL());

        // Initialize components
        text = ComponentFactory.getTextArea();
        text.setMargin(new Insets(5,5,5,5));
        text.setEditable(false);
        text.setDoubleBuffered(true);
        text.setWrapStyleWord(true);
        text.setLineWrap(true);
        text.setBackground(ComponentFactory.getTableColor());
        
        JScrollPane scroll = new JScrollPane(text);
        scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        scroll.setPreferredSize(new Dimension(510, 300));

        // Build the panel
        panelLogging.add(   scroll,  Layout.getGBC( 0, 0, 1, 1, 10.0, 10.0
                                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                     new Insets( 0, 5, 0, 5), 0, 0));

        // ************************************************************************
        // PRODUCT INFORMATION PANEL
        // Set layout style
        panelProductInfo.setLayout(Layout.getGBL());

        // Initialize components
        labelProductName    = ComponentFactory.getLabel(Resources.getText(DCLabels.lblDataCrowFullName), 200);
        labelVersion        = ComponentFactory.getLabel(Resources.getText(DCLabels.lblVersion), 200);
        labelLastUpd        = ComponentFactory.getLabel(Resources.getText(DCLabels.lblDataCrowDate), 200);

        labelProductName.setPreferredSize(new Dimension(515, 16));
        labelVersion.setPreferredSize(new Dimension(515, 16));
        labelLastUpd.setPreferredSize(new Dimension(515, 16));

        // Build the panel
        panelProductInfo.add(labelProductName, Layout.getGBC( 0, 0, 1, 1, 0.0, 0.0
                                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                               new Insets( 0, 5, 0, 0), 0, 0));
        panelProductInfo.add(labelVersion,     Layout.getGBC( 0, 1, 1, 1, 0.0, 0.0
                                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                               new Insets( 0, 5, 0, 0), 0, 0));
        panelProductInfo.add(labelLastUpd,     Layout.getGBC( 0, 2, 1, 1, 0.0, 0.0
                                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                               new Insets( 0, 5, 0, 0), 0, 0));


        // ************************************************************************
        // MAIN PANEL
        // Set layout style
        this.getContentPane().setLayout(Layout.getGBL());

        // Initialize components
        setTitle(Resources.getText(DCLabels.lblLogEvent));
        setResizable(false);

        Image image = IconLibrary._icoLog.getImage();
        this.setIconImage(image);
        image.flush();

        // Build the panel
        this.getContentPane().add(   panelPicture,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                                    new Insets( 0, 0, 0, 0), 0, 0));
        this.getContentPane().add(   panelLogging,  Layout.getGBC( 0, 1, 1, 1, 30.0, 30.0
                                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                                    new Insets( 0, 0, 0, 0), 0, 0));
        this.getContentPane().add(panelProductInfo, Layout.getGBC( 0, 2, 1, 1, 0.0, 0.0
                                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                    new Insets( 0, 0, 5, 0), 0, 0));
        
        pack();

        Dimension location = Utilities.getCenteredWindowLocation(getSize());
        setLocation(location.width, location.height);

    }

    /**
     * Moves this frame to the front
     */
    public void logToFront(boolean show) {
        this.setVisible(show);

        if (show) {
            this.toFront();
        }
    }

    /**
     * Writes an event to the screen. New events are written to the top
     *
     * @param message   event String
     */
    public void writeToScreen(String message) {
        text.insert(message, 0);
    }
    
    /**
     * Forces a refresh of the content of the Software categories combobox
     */
    public class ActiveFrame implements MouseListener {
        public void mouseReleased(MouseEvent e) {}
        public void mouseEntered(MouseEvent e) {
            SwingUtil.setRootFrame((JFrame) e.getSource());
        }
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}        
    }     
}
