/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ToolTipManager;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.Layout;
import net.sf.dc.console.software.SoftwareCategoryForm;
import net.sf.dc.core.objects.DCCollection;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.Requestor;
import net.sf.dc.core.objects.RequestorCollection;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCMessages;
import net.sf.ext.util.Resources;

/**
 * 
 * Base for all insert panels (for objects like Software, Music and Movie)
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class TableInsertPanel extends TablePanel {
    
    private JPanel panelInput  = new JPanel(false);
    private JPanel panelStatus = new JPanel(false);

    /**
     * Initializes the panel. Should be called by the child by calling
     * super(module). The popup menu and the action panel are automatically
     * created. 
     * 
     * @param module the module indentifier
     */
    public TableInsertPanel (int module) {
        super(module);        
        
        boolean imdb = getModule().isImdbUpdateAllowed();
        boolean amazon = getModule().isAmazonUpdateAllowed();
        boolean open = this instanceof SoftwareCategoryForm.CatagoryInsertPanel ?
                       false : true;

        panelTableActions = new TableActionsPanel(true);
        panelTableActions.addSaveActionListener(new SaveNewValuesAction());
            
        popupMenu = new PopupMenu(imdb, amazon, false, true, open, false);
        popupMenu.addAmazonActionListener(new AmazonUpdateAction());
        popupMenu.addIMDBActionListener(new ImdbUpdateAction());
        popupMenu.addSaveActionListener(new SaveSelectedValuesAction());
        
        buildPanel();
        
        for (Enumeration enum = table.getColumnModel().getColumns();
             enum.hasMoreElements(); ) {
            
            vColumns.addElement(enum.nextElement());
        }
    }

    /**
     * Saves the selected values only
     */
    public void saveSelectedValues(boolean silent) {
        cancelEdit();
        int[] rows = table.getSelectedRows();
        DCCollection collection = new DCCollection();
        if (rows != null && rows.length > 0) {
            DCObject o = getModule().getDataCrowObject();
            for (int i = 0; i < rows.length; i++) {
                Object[] row = getValuesAtRow(rows[i], true);
                o.setValues(row);
                o.setSilence(silent);
                o.setValue(DCObject._ID, "");
                o.setUIRowNumber(rows[i]);
                collection.addObject(o, new Integer(rows[i]));
            }
        }
        
        RequestorCollection requestors = new RequestorCollection();

        collection.saveNew(requestors);
        setStatus(Resources.getText(DCMessages.msgSaveHasBeenExecuted));        
    }

    /**
     * Saves all the new values
     */
    public void saveNewValues() {
        cancelEdit();
        
        if (tableModel.getRowCount() > 1000) {
        	QuestionBox qstBox = new QuestionBox("It is strongly advised to select only a maximum of about 1000 records to save at once. " +
                                                 "Shall I select the first 1000 rows and save them ?");
            qstBox.dispose();
            if (QuestionBox.result.booleanValue()) {
            	table.setRowSelectionInterval(0, 1000);
                table.setColumnSelectionInterval(0, table.getColumnCount() -1);
                saveSelectedValues(false);
                return;
            } 
        } 

        DCCollection collection = new DCCollection();
        
        for (int i = 0; i < table.getRowCount(); i++) {
            Object[] row = getValuesAtRow(i, true);
            DCObject o = getModule().getDataCrowObject();
            o.setValues(row);
            o.setUIRowNumber(i);
            o.setValue(DCObject._ID, "");
            collection.addObject(o, new Integer(i));
        }
        Requestor requestor = new Requestor(getClass(), 
                                            "setStatus", 
                                            new String[] {Resources.getText(DCMessages.msgDataSaved)});
        RequestorCollection requestors = new RequestorCollection();
        requestors.addRequestor(requestor);      
        collection.saveNew(requestors);
        cache.clear();
        collection = null;
        setStatus(Resources.getText(DCMessages.msgSaveHasBeenExecuted));        
    }

    /**
     * Creates the panel
     */
    private void buildPanel() {
        // ************************************************************************
        // INPUT PANEL
        // Set layout style
        panelInput.setLayout(Layout.getGBL());
 
        // Initialize components
        table = ComponentFactory.getTableForObject(getModule().getDataCrowObject(), 
                                                   tableModel);
        table.addMouseListener(tableMouseListener);

        JScrollPane scrollResults = new JScrollPane(table);
        scrollResults.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

        // Build the panel
        panelInput.add( scrollResults,      Layout.getGBC( 0, 0, 7, 1, 20.0, 20.0
                       ,GridBagConstraints.CENTER, GridBagConstraints.BOTH,
                        new Insets(5, 5, 5, 5), 0, 0));


        // ************************************************************************
        // STATUS PANEL
        // Set layout style
        panelStatus.setLayout(Layout.getGBL());

        // Initialize components
        labelStatus = ComponentFactory.getLabel(Resources.getText(DCLabels.lblVersion));

        // Build the panel
        panelStatus.add(    labelStatus,       Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
                            new Insets(0,  0, 0, 0), 0, 0));


        // ************************************************************************
        // MAIN PANEL
        // Set layout style
        setLayout(Layout.getGBL());

        // Initialize components
        panelInput.setBorder(ComponentFactory.getTitleBorder(
                             Resources.getText(DCLabels.lblInsert)));
        panelStatus.setBorder(ComponentFactory.getTitleBorder(
                              Resources.getText(DCLabels.lblStatus)));

        // Fill the main panel
        add(panelInput,     Layout.getGBC( 0, 0, 8, 1, 20.0, 20.0
                           ,GridBagConstraints.NORTH, GridBagConstraints.BOTH,
                            new Insets( 5, 5, 5, 5), 0, 0));
        add(panelTableActions,Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                            new Insets( 5, 5, 5, 5), 0, 0));
                            
        add(panelStatus,    Layout.getGBC( 0, 2, 8, 1, 1.0, 1.0
                           ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                            new Insets(5, 5, 5, 5), 0, 0));

        ToolTipManager.sharedInstance().registerComponent(table);
    }
    
    /**
     * Saves all selected values 
     * @see TableInsertPanel#saveSelectedValues(boolean)
     */
    private class SaveSelectedValuesAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            cancelEdit();
            saveSelectedValues(false);
        }
    }  

    /**
     * Saves all new values (insert action)
     * @see TableInsertPanel#saveNewValues()
     */
    private class SaveNewValuesAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            saveNewValues();
        }
    }    
    
    /**
     * Starts the Amazon update job
     */
    private class AmazonUpdateAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startAmazonUpdate();
        }
    }

    /**
     * Starts the IMDB update job
     */
    private class ImdbUpdateAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startIMDBUpdate();
        }
    }      
}
