/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;

import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.ToolTipManager;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.table.DefaultTableCellRenderer;

import net.sf.dc.core.Repository;
import net.sf.ext.jflexsett.settings.Settings;

/**
 * Table renderer. Applies the right selection fore and background.
 * Enables the odd/even colors
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC1.00
 */
public class DCTableCellRenderer extends DefaultTableCellRenderer {

    private static boolean disabled = false;
    private Border focusBorder = UIManager.getBorder("Table.focusCellHighlightBorder");
    
    public Component getTableCellRendererComponent(
                                            JTable table, 
                                            Object value, 
                                            boolean isSelected, 
                                            boolean hasFocus, 
                                            int row,
                                            int column) {

        JComponent c = (JComponent) super.getTableCellRendererComponent(  
                                                            table,  
                                                            value,
                                                            isSelected,
                                                            hasFocus,
                                                            row,
                                                            column);
        
        if (Settings.getValueAsBoolean(Repository.settings.stShowTableTooltip)) {
            if (disabled) {
                ToolTipManager.sharedInstance().registerComponent(table);
                ToolTipManager.sharedInstance().registerComponent(table.getTableHeader());
                disabled = false;
            } 
            
            ToolTipManager.sharedInstance().setDismissDelay(2000);
            
            String tiptext = value.toString();

            int length = tiptext.length();
            if (length > 175) {
            	tiptext = tiptext.substring(0, 175) + "..";
            }

            if (length > 0) {
                c.setToolTipText(tiptext);
                setEnabled(table == null || table.isEnabled());
            } else {
                ToolTipManager.sharedInstance().setDismissDelay(0);
            }
        } else {
            if (!disabled) { 
                disabled = true;
            	ToolTipManager.sharedInstance().unregisterComponent(table);
            	ToolTipManager.sharedInstance().unregisterComponent(table.getTableHeader());
            }
        }
        
        Color colorOddRow = Settings.getValueAsColor(Repository.settings.stOddRowColor);
        Color colorEvenRow = Settings.getValueAsColor(Repository.settings.stEvenRowColor);
        
        setForeground(new Color(0,0,0));
        if (!isSelected) {
            if ((row % 2) == 0) {
                setBackground(colorEvenRow);
            } else {
                setBackground(colorOddRow);
            }
        } else {
            setBackground(Color.LIGHT_GRAY);
            setForeground(Color.WHITE);
        }
        
        if (hasFocus){
            setForeground(Color.WHITE);
            setBorder(focusBorder);
        } else {
            setBorder(noFocusBorder);
        }
        return c;
    }

    // As long as you don't have any HTML text, this override is ok.
    protected void firePropertyChange(String propertyName, Object oldValue, Object newValue) {}

    // This override is only appropriate if this will never contain any
    // children AND the Graphics is not clobbered during painting.
    public void paint(Graphics g) {
    	ui.update(g, this);
    }

    public boolean isOpaque() { 
    	return true;
    }

    // This is generally ok for non-Composite components (like Labels)
    public void invalidate() {}

    // Can be ignored, we don't exist in the containment hierarchy.
    public void repaint() {}    
}