/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Dimension;
import java.awt.Graphics;

import javax.swing.CellRendererPane;
import javax.swing.JComponent;
import javax.swing.JTextArea;
import javax.swing.JToolTip;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicToolTipUI;

/**
 * UI implementation of a multiple line tooltip
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC1.00
 */
public class MultiLineToolTipUI extends BasicToolTipUI {

    private final static MultiLineToolTipUI sharedInstance = 
                         new MultiLineToolTipUI();
    private CellRendererPane rendererPane;
    private static final JTextArea textArea = 
                         ComponentFactory.getTextArea();
    
    
    public MultiLineToolTipUI() {
        super();
    }

    public static ComponentUI createUI(JComponent c) {
        return sharedInstance;
    }
    
    public void installUI(JComponent c) {
        super.installUI(c);
        //tip = (JToolTip)c;
        rendererPane = new CellRendererPane();
        c.add(rendererPane);
    }
    
    public void uninstallUI(JComponent c) {
        super.uninstallUI(c);
        
        c.remove(rendererPane);
        rendererPane = null;
    }
    
    public void paint(Graphics g, JComponent c) {
        Dimension size = c.getSize();
        textArea.setBackground(c.getBackground());
        rendererPane.paintComponent(g, textArea, c, 1, 1,
                size.width - 1, size.height - 1, true);
    }
    
    public Dimension getPreferredSize(JComponent c) {
        String tipText = ((JToolTip)c).getTipText();
        if (tipText == null) {
            return new Dimension(0,0);
        }
        textArea.setText(tipText);
        rendererPane.removeAll();
        rendererPane.add(textArea );
        textArea.setWrapStyleWord(true);
        int width = ((MultiLineToolTip)c).getFixedWidth();
        int columns = ((MultiLineToolTip)c).getColumns();
        
        if (columns > 0) {
            textArea.setColumns(columns);
            textArea.setSize(0,0);
            textArea.setLineWrap(true);
            textArea.setSize( textArea.getPreferredSize() );
        } else if (width > 0) {
            textArea.setLineWrap(true);
            Dimension d = textArea.getPreferredSize();
            d.width = width;
            d.height++;
            textArea.setSize(d);
        } else {
            textArea.setLineWrap(false);
        }
        
        Dimension dim = textArea.getPreferredSize();
        
        dim.height += 1;
        dim.width += 1;
        return dim;
    }
    
    public Dimension getMinimumSize(JComponent c) {
        return getPreferredSize(c);
    }
    
    public Dimension getMaximumSize(JComponent c) {
        return getPreferredSize(c);
    }
}
