/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.da;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import net.sf.dc.console.MessageBox;
import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.FileField;
import net.sf.dc.console.components.Layout;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.messages.Messages;
import net.sf.dc.messages.MessagesWriter;
import net.sf.dc.processes.ImportCDProcess;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCMessages;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.Resources;
import net.sf.ext.util.Utilities;

/**
 * Via this dialog CD content can be imported. The import type is
 * determined by the specified module. The imported data is transfered
 * to the insert table. Here, the user can edit the values and commits
 * the data to the database 
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC1.00
 */
public class ImportCDDialog extends JFrame {

    private final JButton buttonCDExport;
        
    private final JLabel labelCDName;
    private final JTextField textCDName;

    private final JLabel labelLocation;
    private final FileField fieldLocation;

    private final JCheckBox checkVerify;
    
    private final DCModule module;

    public ImportCDDialog(DCModule module) {
        this.module = module;
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
            Settings.setValue(Repository.settings.stImportCDDialogSize, getSize());
            }
        });
        
        Image image = IconLibrary._icoImport.getImage();
        setIconImage(image);
        image.flush();  
        setTitle(Resources.getText(DCLabels.lblFileImport));      
        
        JPanel panelExportCD = new JPanel(false);
        JPanel panelExportActions = new JPanel(false);
        
        // ************************************************************************
        // EXPORT CD PANEL
        // Set layout style
        panelExportCD.setLayout(Layout.getGBL());

        // Initialize components
        labelLocation = ComponentFactory.getLabel(Resources.getText(DCLabels.lblCDLocation));
        fieldLocation = new FileField(false, true);
        checkVerify = ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblAskPerDir));
        labelCDName = ComponentFactory.getLabel(Resources.getText(DCLabels.lblLocationName));
        textCDName = ComponentFactory.getTextField();

        Color color = Settings.getValueAsColor(Repository.settings.stColor);
        checkVerify.setBackground(color);

        // Build the panel
        panelExportCD.add(  labelLocation,  Layout.getGBC( 0, 0, 2, 1, 1.0, 1.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets(0, 5, 0, 5), 0, 0));
        panelExportCD.add(  fieldLocation,  Layout.getGBC( 1, 0, 3, 1, 1.0, 1.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets(0, 5, 0, 5), 0, 0));
        panelExportCD.add(  labelCDName,    Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets(0, 5, 0, 5), 0, 0));
        panelExportCD.add(  textCDName,     Layout.getGBC( 1, 1, 2, 1, 20.0, 20.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets(0, 5, 0, 5), 0, 0));
        panelExportCD.add(  checkVerify,    Layout.getGBC( 0, 2, 3, 1, 1.0, 1.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets(5, 5, 0, 5), 0, 0));
                                            
        // ************************************************************************
        // EXPORT FILE ACTION PANEL
        // Set layout style
        panelExportActions.setLayout(Layout.getGBL());

        // Initilialize components
        buttonCDExport = ComponentFactory.getButton(Resources.getText(DCLabels.lblStartImport));
        buttonCDExport.addActionListener(new ExportCDButtonAction());

        // Build the panel
        panelExportActions.add(buttonCDExport,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                               ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                                new Insets( 5, 5, 5, 5), 0, 0));
                    
        // ************************************************************************
        // MAIN PANEL

        panelExportCD.setBorder(ComponentFactory.getTitleBorder(""));
                    
        this.getContentPane().setLayout(Layout.getGBL());
        this.getContentPane().add(
                 panelExportCD,      Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 5, 5, 5, 5), 0, 0));
        this.getContentPane().add(
                 panelExportActions, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 0, 0, 0), 0, 0));                 

        getContentPane().setBackground(color);
        panelExportCD.setBackground(color);
        panelExportActions.setBackground(color);
        
        this.pack();
        setSize(Settings.getValueAsDimension(Repository.settings.stImportCDDialogSize));
        
        Dimension location = Utilities.getCenteredWindowLocation(getSize());
        setLocation(location.width, location.height);
        
        this.setResizable(false);
        this.setVisible(true);
    }

    /**
     * Start the import of the CD Content. The specified module is 
     * used to get the actual import job. The import is run in a 
     * seperate thread
     */
    protected void startImport() {
        boolean bVerify = checkVerify.isSelected();
        
        String sourceDir = "";
        if (fieldLocation.getFile() != null) {
            sourceDir = fieldLocation.getFileAsString();
        }
        
        if (sourceDir == null || sourceDir.equals("")) {
            new MessageBox(Resources.getText(DCMessages.msgSelectDirectory), 
                           MessageBox._WARNING);
            return;
        }
        
        String cdName = textCDName.getText().trim();
        if (cdName.equals("")) {
            new MessageBox(Resources.getText(DCMessages.msgEnterCDName), 
                           MessageBox._WARNING);
            return;
        }
        
        Messages.setMessage(Resources.getText(DCMessages.msgStartImportJob, sourceDir), 
                            Messages._STANDARD);
        MessagesWriter._useLogFile = false;
        ImportCDProcess importCD = module.getCDImportProcess(sourceDir, 
                                                       bVerify,
                                                       cdName);
        if (importCD != null) {
            Thread importDataThread = new Thread(importCD, "ImportCD-Prio5");
            importDataThread.setPriority(1);
            importDataThread.setDaemon(true);
            importDataThread.start();
            MessagesWriter._windowShow = true;
            this.dispose();
        }
    }   
    
    private class ExportCDButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startImport();
        }
    }
}
