/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.note;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.RandomAccessFile;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;

import net.sf.dc.console.DataCrowPanel;
import net.sf.dc.console.MessageBox;
import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.Layout;
import net.sf.dc.core.Repository;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCMessages;
import net.sf.ext.util.Resources;

/**
 * Panel for writing notes and comments. The notes are written to
 * a plain textfile.
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class NotePanel extends DataCrowPanel {

    private JPanel                  panelNote;
    private JPanel                  panelActions;
    private JButton                 buttonSave;
    private JButton                 buttonClear;
    private JButton                 buttonLoad;
    private JPanel                  panelStatus;
    private JTextArea             	textNote;

    private File                    file = new File(Repository.files.flCommentFile);
    private RandomAccessFile        access;

    /**
     * Retrieves the saved note (as a textfile) and display it
     */
    public void loadNote() {
        try {
            access = new RandomAccessFile(file, "rw");
            long filePointer = 0;
            long fileLength = file.length();

            StringBuffer sb = new StringBuffer("");
            while (filePointer < fileLength) {
                sb.append(access.readLine().trim() + '\n');
                filePointer = access.getFilePointer();
            }

            textNote.setText(sb.toString());
        } catch (Exception exp) {}
    }

    /**
     * Creates this panel
     */
    public NotePanel() {
        super();
        buildPanel();
        loadNote();
    }

    /**
     * Saves the notes to a text file
     */
    public void saveNote() {
        boolean success = false;
        try {
            access = new RandomAccessFile(file, "rw");
            textNote.getText();
            access.setLength(0l);
            access.writeBytes(textNote.getText());
            access.close();
            success = true;
        } catch (Exception exp) {
            Messages.setMessage(exp);
        } finally {
            if (success) {
                setStatus(Resources.getText(DCMessages.msgSaveHasBeenExecuted));
                new MessageBox(Resources.getText(DCMessages.msgDataSaved), 
                               MessageBox._INFORMATION);
            }
        }
    }

    /**
     * Builds the panel
     */
    private void buildPanel() {
        panelNote	= new JPanel(false);
        panelActions= new JPanel(false);
        panelStatus = new JPanel(false);

        // ************************************************************************
        // STATUS PANEL
        // Set layout style
        panelStatus.setLayout(Layout.getGBL());

        // Initialize components
        labelStatus = ComponentFactory.getLabel(Resources.getText(DCLabels.lblVersion));

        // Build the panel
        panelStatus.add(    labelStatus,       Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                            new Insets(0, 0, 0, 0), 0, 0));

        // ************************************************************************
        // NOTES PANEL
        // Set layout style
        panelNote.setLayout(Layout.getGBL());

        // Initialize components
        textNote = ComponentFactory.getTextArea();
        textNote.setBackground(ComponentFactory.getTableColor());

        JScrollPane scrollIn = new JScrollPane(textNote);
        scrollIn.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollIn.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        // Build the panel
        panelNote.add(    scrollIn,      Layout.getGBC( 0, 0, 1, 1, 80.0, 80.0
                                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                         new Insets(5, 5, 5, 5), 0, 0));

        // ************************************************************************
        // ACTIONS PANEL
        // Set layout style

        // Initialize components
        buttonSave  = ComponentFactory.getButton(Resources.getText(DCLabels.lblSave));
        buttonClear = ComponentFactory.getButton(Resources.getText(DCLabels.lblClear));
        buttonLoad  = ComponentFactory.getButton(Resources.getText(DCLabels.lblLoad));

        buttonLoad.setMnemonic(KeyEvent.VK_L);
        buttonClear.setMnemonic(KeyEvent.VK_C);
        buttonSave.setMnemonic(KeyEvent.VK_S);

        buttonSave.addActionListener(new SaveButtonAction());
        buttonClear.addActionListener(new ClearButtonAction());
        buttonLoad.addActionListener(new LoadButtonAction());

        // Build the panel
        panelActions.add(   buttonSave, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                            new Insets(0,  5, 0, 0), 0, 0));
        panelActions.add(      buttonLoad, Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                            new Insets(0,  5, 0, 0), 0, 0));
        panelActions.add(   buttonClear,Layout.getGBC( 2, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                            new Insets(0,  5, 0, 0), 0, 0));


        // ************************************************************************
        // MAIN PANEL
        // Set layout style
        setLayout(Layout.getGBL());

        // Initialize components
        panelStatus.setBorder(ComponentFactory.getTitleBorder(
                              Resources.getText(DCLabels.lblStatus)));
        panelNote.setBorder(ComponentFactory.getTitleBorder(
                            Resources.getText(DCLabels.lblTabNote)));

        // Build the panel
        add( panelNote,             Layout.getGBC( 0, 0, 7, 1, 80.0, 80.0
                                   ,GridBagConstraints.NORTH, GridBagConstraints.BOTH,
                                    new Insets(5, 5, 5, 5), 0, 0));
        add( panelActions,          Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                                   ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                                    new Insets(5, 5, 5, 5), 0, 0));
        add( panelStatus,           Layout.getGBC( 0, 2, 7, 1, 1.0, 1.0
                                   ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                                    new Insets(5, 5, 5, 5), 0, 0));
    }


    private class SaveButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            saveNote();
        }
    }

    private class ClearButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            textNote.setText("");
        }
    }

    private class LoadButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            loadNote();
        }
    }
}
