/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.software;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Vector;

import net.sf.dc.console.MessageBox;
import net.sf.dc.console.SearchForm;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.software.Software;
import net.sf.dc.http.AmazonSoftwareQuery;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DCMessages;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.Resources;
import net.sf.ext.util.SwingUtil;

/**
 * Searches Amazon.com for information about programs and games
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.75
 */
public class AmazonSoftwareSearchForm extends SearchForm {
    
    /**
     * Instantiates this form and starts the search
     * 
     * @param rowNumber the rownumber of the to be updated record
     * @param title the title for this form
     * @param name the name on which to search
     * @param sender the requestor to which feedback should be given 
     */
    public AmazonSoftwareSearchForm(int rowNumber, 
                                    String title, 
                                    String name, 
                                    Object sender) {
                          
        super(sender, new int[] {Software._A_NAME, 
                                 Software._B_DESCRIPTION, 
                                 Software._C_WEBPAGE,
                                 Software._G_PICTUREFRONT,
								 Software._J_SCREENSHOTONE,
								 Software._K_SCREENSHOTTWO,
								 Software._L_SCREENSHOTTHREE},
              DCModules._SOFTWARE);
                                 
        super.setTitle(title);
        SwingUtil.setRootFrame(this);        
    
        isUpdateRunning = true;
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                Settings.setValue(Repository.settings.stImdbSearchFormViewSize,
                                  getSize());
                running = false;
            }
        });        
    
        this.rowNumber = rowNumber;
    
        textSearch.setText(name);
        startSearch();
    
        setBackgrounds(Settings.getValueAsColor(Repository.settings.stColor));
        setVisible(true);
    }

    /**
     * Starts the search thread
     * @see SearchJob
     */    
    protected void startSearch() {
        String name = textSearch.getText();
        
        if (name == null || name.equals("")) {
            return;   
        }
        threadSearch = new Thread(new SearchJob(name), "SearchJob");
        threadSearch.setDaemon(true);
        threadSearch.setPriority(1);
        threadSearch.start();        
    }

    /**
     * Simple search thread. Uses a query object to contact the
     * internet and puts the results in the table of this form
     */    
    private class SearchJob extends Thread {
    
        final String name;
    
        public SearchJob(String name) {
            this.name = name;
        }
    
        public void run() {
            try {
                running = true;
                buttonSearch.setEnabled(false);
                buttonStop.setEnabled(true);
                buttonStopAll.setEnabled(true);                
                searchAmazon(name);
            } catch (Exception exp) {
                exp.printStackTrace();
                Messages.setMessage(exp);
                new MessageBox(Resources.getText(DCMessages.msgProcessCrash), 
                               MessageBox._ERROR);
            }
        }      

        private void searchAmazon(String name) {
            try {
                AmazonSoftwareQuery amazonQuery = new AmazonSoftwareQuery(
                									name, 
                									checkBroadband.isSelected());
                Vector v = amazonQuery.search();
    
                progress.setMaximum(v.size());
                progress.setValue(0);
                
                int i = 1;
                Software software = null;
                while (v.size() > 0) {
                    software = amazonQuery.parseInformation(v.remove(0).toString());
                    if (software != null) {
                        setData(software);
                    }                    
                    progress.setValue(i++);    
                } 
                
            } catch (Exception exp) {
                new MessageBox(exp.getMessage(), MessageBox._ERROR);   
            } finally {
                buttonSearch.setEnabled(true);
                buttonStop.setEnabled(false);
                buttonStopAll.setEnabled(false);
            }
        }
    }
    
    public void setBackgrounds(Color color) {
        this.setBackground(color);
        setCompBackColors(this.getComponents(), color);
    }        

    private void setCompBackColors(Component[] comps, Color color) {
        try {
            for (int i = 0; i < comps.length; i++) {
                comps[i].setBackground(color);
            }
        } catch (java.lang.Exception exp) {
            Messages.setMessage(exp);
        }
    }    
}
