/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.software;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;
import javax.swing.JTabbedPane;

import net.sf.dc.console.TableInsertPanel;
import net.sf.dc.console.TableSearchPanel;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.IconCache;
import net.sf.dc.core.objects.cache.CacheManager;
import net.sf.dc.resources.DCLabels;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.Resources;
import net.sf.ext.util.SwingUtil;
import net.sf.ext.util.Utilities;


/**
 * A form consisting out of a search software category tab and
 * an insert tab for software categories. Basically this is a 
 * small implementation of the MainFrame..
 * 
 * With this form, software categories are maintained. When
 * this form is closed, the Software Category Cache is refreshed.
 * 
 * @see net.sf.dc.core.objects.cache.CacheManager
 * @see net.sf.dc.core.objects.software.SoftwareCategory
 */
public class SoftwareCategoryForm extends JFrame {
    private JTabbedPane tabbedPane = new JTabbedPane();

    public final CatagorySearchPanel searchCat;
    public final CatagoryInsertPanel insertCat;

    public SoftwareCategoryForm() {
        SwingUtil.setRootFrame(this);
        
        this.setTitle(Resources.getText(DCLabels.lblSoftCatagories));
        Image image = IconLibrary._icoCategory.getImage();
        this.setIconImage(image);
        image.flush();
        
        searchCat = new CatagorySearchPanel();
        insertCat = new CatagoryInsertPanel();
        
        tabbedPane.addTab(Resources.getText(DCLabels.lblExistingValues), 
                          IconLibrary._icoExisting, 
                          searchCat);
        tabbedPane.addTab(Resources.getText(DCLabels.lblNewValues), 
                          IconLibrary._icoNew, 
                          insertCat);        
        getContentPane().add(tabbedPane);
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                Settings.setValue(Repository.settings.stSoftwareCatFormSize, getSize());
                SwingUtil.setRootFrame(null);
                
                // update the cache as values might have been changed
                CacheManager.refresh(CacheManager._SOFTWARECATEGORY);
                IconCache.refresh();
                dispose();
            }
        });        
        
        addMouseListener(new ActiveFrame());
        
        this.setBackgrounds(Settings.getValueAsColor(Repository.settings.stColor));
        pack();
        
        this.setSize(Settings.getValueAsDimension(
                     Repository.settings.stSoftwareCatFormSize));
        
        Dimension location = Utilities.getCenteredWindowLocation(getSize());
        setLocation(location.width, location.height);
    }

    public CatagorySearchPanel getSearchPanel() {
        return searchCat;
    }

    public CatagoryInsertPanel getInsertPanel() {
        return insertCat;
    }

    public void setBackgrounds(Color color) {
        searchCat.setBackground(color);
        insertCat.setBackground(color);
    }

    public class CatagorySearchPanel extends TableSearchPanel {
        public CatagorySearchPanel () {
            super(DCModules._SOFTWARECAT);
        }
    }

    public class CatagoryInsertPanel extends TableInsertPanel {
        public CatagoryInsertPanel() {
            super(DCModules._SOFTWARECAT);
        }
    }
    
    /**
     * Forces a refresh of the content of the Software categories combobox
     */
    public class ActiveFrame implements MouseListener {
        public void mouseReleased(MouseEvent e) {}
        public void mouseEntered(MouseEvent e) {
            SwingUtil.setRootFrame((JFrame) e.getSource());
        }
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}        
    }     
}
