/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/
package net.sf.dc.core.modules;

import javax.swing.ImageIcon;
import javax.swing.JButton;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.movie.InsertMoviePanel;
import net.sf.dc.console.movie.SearchMoviePanel;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DCFieldDefinition;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.processes.ImportCDProcess;
import net.sf.dc.processes.ImportMovieCD;
import net.sf.dc.resources.DCLabels;
import net.sf.ext.util.Resources;

/**
 * The Movie module
 * 
 * @see net.sf.dc.core.modules.DCModules
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC1.00
 */
public class MovieModule extends DCModule {
    
    /**
     * Initializes this module
     */
    public MovieModule() {
        super(DCModules._MOVIE,
              Resources.getText(DCLabels.lblModuleMovie),
              true,
              false,
              true);
    }

    public ImageIcon getIcon() {
        return IconLibrary._icoMovieModule;
    }
    
    /**
     * Returns the specific menu items for this module.
     * For this module, there are no additional menu items,
     * so an empty array is returned (never return null !)
     */
    public JButton[] getMenuItems() {
        return new JButton[0];
    }
    
    /**
     * Return the object managed by this module 
     * @see Movie
     */
    public DCObject getDataCrowObject() {
        return new Movie();
    }
        
    /**
     * Create the insert and search panel
     */
    protected void initializeUI() {
        insertPanel = new InsertMoviePanel();
        searchPanel = new SearchMoviePanel();
    }
        
    public String getHiddenFieldsSettingsKey() {
        return Repository.settings.stHiddenFieldsMovie;
    }

    public String getRequiredFieldsSettingsKey() {
        return Repository.settings.stMovieRequiredFields;
    }
    
    /**
     * Returns the movie CD import process
     * 
     * @param sourceDir directory which is used as source for the
     *        import
     * @param confirmEachDir indicates whether for each directory
     *        confirmation should be asked if the underlying data
     *        of this directory should be read or not.
     * @param nameOfCD cd name (used as the location name)
     */    
    public ImportCDProcess getCDImportProcess(String sourceDir,
                                        boolean confirmEachDir,
                                        String nameOfCD) {

        return new ImportMovieCD(sourceDir,
                                 confirmEachDir,
                                 nameOfCD);
    }
    
    /**
     * Initalizes all the fields
     */
    protected void initializeFields() {
        fields = new DCFieldDefinition[25];
        fields[Movie._ID] = new DCFieldDefinition(Resources.getText(DCLabels.lblID),
                                        true,
                                        15,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "ID");
        fields[Movie._A_TITLE] = new DCFieldDefinition(Resources.getText(DCLabels.lblTitle),
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Title");
        fields[Movie._B_DESCRIPTION] = new DCFieldDefinition(Resources.getText(DCLabels.lblDescription),
                                        false,
                                        8000,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Description");
        fields[Movie._C_WEBPAGE] = new DCFieldDefinition(Resources.getText(DCLabels.lblWebPage),
                                        false,
                                        255,
                                        ComponentFactory._URLFIELD,
                                        Repository.datatypes._STRING,
                                        "Webpage");                                       
        fields[Movie._D_GENRE] = new DCFieldDefinition(Resources.getText(DCLabels.lblGenre),
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Genre");                                     
        fields[Movie._E_ACTORS] = new DCFieldDefinition(Resources.getText(DCLabels.lblActors),
                                        false,
                                        255,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Actors");   
        fields[Movie._F_DIRECTOR] = new DCFieldDefinition(Resources.getText(DCLabels.lblDirector),
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Director");     
        fields[Movie._G_COUNTRY] = new DCFieldDefinition(Resources.getText(DCLabels.lblCountry),
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Country");    
        fields[Movie._H_YEAR] = new DCFieldDefinition(Resources.getText(DCLabels.lblYear),
                                        false,
                                        4,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Year");     
        fields[Movie._I_PLAYLENGTH] = new DCFieldDefinition(Resources.getText(DCLabels.lblPlayLength),
                                        false,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "PlayLength"); 
        fields[Movie._J_FILENAME] = new DCFieldDefinition(Resources.getText(DCLabels.lblFileName),
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Filename");  
        fields[Movie._K_VIDEOCODEC] = new DCFieldDefinition(Resources.getText(DCLabels.lblVideoCodec),
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "VideoCodec");   
        fields[Movie._L_AUDIOCODEC] = new DCFieldDefinition(Resources.getText(DCLabels.lblAudioCodec),
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "AudioCodec");  
        fields[Movie._M_WIDTH] = new DCFieldDefinition(Resources.getText(DCLabels.lblWidth),
                                        false,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Width");                                         
        fields[Movie._N_HEIGHT] = new DCFieldDefinition(Resources.getText(DCLabels.lblHeight),
                                        false,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Height");                                         
        fields[Movie._O_FPS] = new DCFieldDefinition(Resources.getText(DCLabels.lblFPS),
                                        false,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Fps");                                         
        fields[Movie._P_FRAMES] = new DCFieldDefinition(Resources.getText(DCLabels.lblFrames),
                                        false,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Frames");                                         
        fields[Movie._Q_AUDIOBITRATE] = new DCFieldDefinition(Resources.getText(DCLabels.lblAudioBitRate),
                                        false,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "AudioBitRate");                                         
        fields[Movie._R_AUDIOSAMPLINGRATE] = new DCFieldDefinition(Resources.getText(DCLabels.lblAudioSampleRate),
                                        false,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "AudioSampleRate");                                         
        fields[Movie._S_AUDIOCHANNEL] = new DCFieldDefinition(Resources.getText(DCLabels.lblAudioChannel),
                                        false,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "AudioChannels");                                         
        fields[Movie._T_VIDEOBITRATE] = new DCFieldDefinition(Resources.getText(DCLabels.lblVideoBitRate),
                                        false,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "VideoBitRate");                                         
        fields[Movie._U_PICTUREFRONT] = new DCFieldDefinition(Resources.getText(DCLabels.lblPictureFront),
                                        true,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "PictureFront");                                         
        fields[Movie._V_PICTUREBACK] = new DCFieldDefinition(Resources.getText(DCLabels.lblPictureBack),
                                        true,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "PictureBack");                                         
        fields[Movie._W_PICTURECD] = new DCFieldDefinition(Resources.getText(DCLabels.lblPictureCD),
                                        true,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "PictureCD");                                         
        fields[Movie._X_LOCATION] = new DCFieldDefinition(Resources.getText(DCLabels.lblLocation),
                                        false,
                                        4000,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Location");                                         
    }
}
