/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import javax.swing.ImageIcon;
import javax.swing.JButton;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.music.InsertMusicPanel;
import net.sf.dc.console.music.SearchMusicPanel;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DCFieldDefinition;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.music.Music;
import net.sf.dc.processes.ImportCDProcess;
import net.sf.dc.processes.ImportMusicCD;
import net.sf.dc.resources.DCLabels;
import net.sf.ext.util.Resources;

/**
 * The Music module
 * 
 * @see net.sf.dc.core.modules.DCModules
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC1.00
 */
public class MusicModule extends DCModule {
    
    /**
     * Initializes this module
     */
    public MusicModule() {
        super(DCModules._MUSIC,
              Resources.getText(DCLabels.lblModuleMusic),
              false,
              false,
              true);
    }
    
    public ImageIcon getIcon() {
        return IconLibrary._icoMusicModule;
    }
    
    /**
     * Returns the specific menu items for this module.
     * For this module, there are no additional menu items,
     * so an empty array is returned (never return null !)
     */    
    public JButton[] getMenuItems() {
        return new JButton[0];
    }
    
    /**
     * Return the object managed by this module 
     * @see Music
     */    
    public DCObject getDataCrowObject() {
        return new Music();
    }
        
    /**
     * Create the insert and search panel
     */    
    protected void initializeUI() {
        insertPanel = new InsertMusicPanel();
        searchPanel = new SearchMusicPanel();
    }
        
    public String getHiddenFieldsSettingsKey() {
        return Repository.settings.stHiddenFieldsMusic;
    }

    public String getRequiredFieldsSettingsKey() {
        return Repository.settings.stMusicRequiredFields;
    }
    
    /**
     * Returns the music CD import process
     * 
     * @param sourceDir directory which is used as source for the
     *        import
     * @param confirmEachDir indicates whether for each directory
     *        confirmation should be asked if the underlying data
     *        of this directory should be read or not.
     * @param nameOfCD cd name (used as the location name)
     * 
     * @return
     */     
    public ImportCDProcess getCDImportProcess(String sourceDir,
                                        	  boolean confirmEachDir,
											  String nameOfCD) {

        return new ImportMusicCD(sourceDir,
                                 confirmEachDir,
                                 nameOfCD);
    }
    
    /**
     * Initalizes all the fields
     */    
    protected void initializeFields() {
        fields = new DCFieldDefinition[12];
        fields[Music._ID] = new DCFieldDefinition(Resources.getText(DCLabels.lblID),
                                        true,
                                        15,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "ID");
        fields[Music._A_FILENAME]	= 	new DCFieldDefinition(Resources.getText(DCLabels.lblFileName),
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Filename");
        fields[Music._B_TITLE] = 	    new DCFieldDefinition(Resources.getText(DCLabels.lblTitle),
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Title");
        fields[Music._C_ARTIST] 	 =  new DCFieldDefinition(Resources.getText(DCLabels.lblArtist),
                                        false,
                                        255,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Artist");                                       
        fields[Music._D_ALBUM] = 	new DCFieldDefinition(Resources.getText(DCLabels.lblAlbum),
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Album");                                     
        fields[Music._E_YEAR] 	      = new DCFieldDefinition(Resources.getText(DCLabels.lblYear),
                                        false,
                                        4,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Year");   
        fields[Music._F_GENRE] 	      = new DCFieldDefinition(Resources.getText(DCLabels.lblGenre),
                                        false,
                                        255,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Genre");     
        fields[Music._G_PLAYLENGTH]   = new DCFieldDefinition(Resources.getText(DCLabels.lblPlayLength),
                                        false,
                                        20,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Length");    
        fields[Music._H_QUALITY] 	  = new DCFieldDefinition(Resources.getText(DCLabels.lblQuality),
                                        false,
                                        4,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Quality");
        fields[Music._I_ENCODING] 	  = new DCFieldDefinition(Resources.getText(DCLabels.lblEncoding),
                						false,
										255,
										ComponentFactory._SHORTTEXTFIELD,
									    Repository.datatypes._STRING,
                						"Encoding");
        fields[Music._J_LYRIC] 	  	  = new DCFieldDefinition(Resources.getText(DCLabels.lblLyric),
										false,
										8000,
										ComponentFactory._LONGTEXTFIELD,
									    Repository.datatypes._STRING,
										"Lyric");        
        fields[Music._K_LOCATION]      = new DCFieldDefinition(Resources.getText(DCLabels.lblLocation),
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Location");   
    }
}
