/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import javax.swing.ImageIcon;
import javax.swing.JButton;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.software.SoftwareCategoryForm;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DCFieldDefinition;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.software.SoftwareCategory;
import net.sf.dc.processes.ImportCDProcess;
import net.sf.dc.resources.DCLabels;
import net.sf.ext.util.Resources;

/**
 * The Software category module
 * 
 * @see net.sf.dc.core.modules.DCModules
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC1.00
 */
public class SoftwareCategoryModule extends DCModule {
    
    private SoftwareCategoryForm form;
    
    /**
     * Initializes this module
     */
    public SoftwareCategoryModule() {
        super(DCModules._SOFTWARECAT,
              Resources.getText(DCLabels.lblModuleSoftwareCategory),
              false,
              false,
              false);
    }

    public ImageIcon getIcon() {
        return null;
    }
    
    /**
     * Create the insert and search panel
     */    
    protected void initializeUI() {
        form = new SoftwareCategoryForm();
        insertPanel = form.insertCat;
        searchPanel = form.searchCat;
    }

    /**
     * Returns the form which holds the UI items for this Module.
     * In this, the Software Category module is an odd one. None
     * of the other modules have a specialized form.
     * 
     * The form consists of the insert and search panels
     */
    public SoftwareCategoryForm getForm() {
        return form;    
    }

    /**
     * Returns the specific menu items for this module.
     * For this module, there are no additional menu items,
     * so an empty array is returned (never return null !)
     */    
    public JButton[] getMenuItems() {
        return new JButton[0];
    }

    /**
     * Return the object managed by this module 
     * @see SoftwareCategory
     */ 
    public DCObject getDataCrowObject() {
        return new SoftwareCategory();
    }
    
    public String getHiddenFieldsSettingsKey() {
        return Repository.settings.stHiddenFieldsSoftwareCat;
    }

    public String getRequiredFieldsSettingsKey() {
        return Repository.settings.stSoftwareCatRequiredFields;
    }
    
    /**
     * Returns null (no CD import process present for Software
     * Categories) 
     * 
     * @param sourceDir directory which is used as source for the
     *        import
     * @param confirmEachDir indicates whether for each directory
     *        confirmation should be asked if the underlying data
     *        of this directory should be read or not.
     * @param nameOfCD cd name (used as the location name)
     * 
     * @return null
     */ 
    public ImportCDProcess getCDImportProcess(String sourceDir,
                                        boolean confirmEachDir,
                                        String nameOfCD) {
        return null;
    }  
    
    /**
     * Initalizes all the fields
     */    
    protected void initializeFields() {
        fields = new DCFieldDefinition[3];
        fields[SoftwareCategory._ID] = new DCFieldDefinition(Resources.getText(DCLabels.lblID),
                                        true,
                                        15,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "ID");        
        fields[SoftwareCategory._A_NAME] = new DCFieldDefinition(Resources.getText(DCLabels.lblName),
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Name");
        fields[SoftwareCategory._B_ICON] = new DCFieldDefinition(Resources.getText(DCLabels.lblIcon),
                                        true,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "Icon");        
    }    
}
