/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.software.InsertSoftwarePanel;
import net.sf.dc.console.software.SearchSoftwarePanel;
import net.sf.dc.console.software.SoftwareCategoryForm;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DCFieldDefinition;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.software.Software;
import net.sf.dc.processes.ImportCDProcess;
import net.sf.dc.processes.ImportSoftwareCD;
import net.sf.dc.resources.DCLabels;
import net.sf.ext.util.Resources;

/**
 * The Software module
 * 
 * @see net.sf.dc.core.modules.DCModules
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC1.00
 */
public class SoftwareModule extends DCModule {
    
    /**
     * Initializes this module
     */    
    public SoftwareModule() {
        super(DCModules._SOFTWARE,
              Resources.getText(DCLabels.lblModuleSoftware),
              false,
              true,
              true);
    }
    
    public ImageIcon getIcon() {
        return IconLibrary._icoSoftwareModule;
    }    
    
    /**
     * Returns the specific menu items for this module.
     * For this module, there is one additional menu item,
     * namely the Software Category Form
     */    
    public JButton[] getMenuItems() {
        JButton buttonCategory = ComponentFactory.getMenuButton(
                                    IconLibrary._icoCategory,
                                    Resources.getText(DCLabels.lblCatagoryForm));
        buttonCategory.addActionListener(new CatagoryFormMenuAction());
        return new JButton[] {buttonCategory};
    }    
    
    /**
     * Create the insert and search panel
     */    
    protected void initializeUI() {
        insertPanel = new InsertSoftwarePanel();
        searchPanel = new SearchSoftwarePanel();    
    }
    
    public String getHiddenFieldsSettingsKey() {
        return Repository.settings.stHiddenFieldsSoftware;
    }

    public String getRequiredFieldsSettingsKey() {
        return Repository.settings.stSoftwareRequiredFields;
    }
    
    /**
     * Return the object managed by this module 
     * @see Software
     */    
    public DCObject getDataCrowObject() {
        return new Software();
    }
    
    /**
     * Returns the movie CD import process
     * 
     * @param sourceDir directory which is used as source for the
     *        import
     * @param confirmEachDir indicates whether for each directory
     *        confirmation should be asked if the underlying data
     *        of this directory should be read or not.
     * @param nameOfCD cd name (used as the location name)
     */      
    public ImportCDProcess getCDImportProcess(String sourceDir,
                                      boolean confirmEachDir,
                                      String nameOfCD) {

        return new ImportSoftwareCD(sourceDir,
                                    confirmEachDir,
                                    nameOfCD);
    }
    
    /**
     * Initalizes all the fields
     */    
    protected void initializeFields() {
        fields = new DCFieldDefinition[14];
        fields[Software._ID] = new DCFieldDefinition(Resources.getText(DCLabels.lblID),
                                        true,
                                        15,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "ID");        
        fields[Software._A_NAME] = new DCFieldDefinition(Resources.getText(DCLabels.lblName),
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Name");        
        fields[Software._B_DESCRIPTION] = new DCFieldDefinition(Resources.getText(DCLabels.lblDescription),
                                        false,
                                        8000,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Description");        
        fields[Software._C_WEBPAGE] = new DCFieldDefinition(Resources.getText(DCLabels.lblWebPage),
                                        false,
                                        255,
                                        ComponentFactory._URLFIELD,
                                        Repository.datatypes._STRING,
                                        "Webpage");        
        fields[Software._D_FILENAME] = new DCFieldDefinition(Resources.getText(DCLabels.lblFileName),
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Filename");        
        fields[Software._E_TYPE] = new DCFieldDefinition(Resources.getText(DCLabels.lblType),
                                        false,
                                        255,
                                        ComponentFactory._SOFTWARECATEGORYCOMBO,
                                        Repository.datatypes._REFERENCE,
                                        "Type");        
        fields[Software._F_MULTI] = new DCFieldDefinition(Resources.getText(DCLabels.lblMulti),
                                        false,
                                        4,
                                        ComponentFactory._YESNOCOMBO,
                                        Repository.datatypes._STRING,
                                        "Multiplayer");        
        fields[Software._G_PICTUREFRONT] = new DCFieldDefinition(Resources.getText(DCLabels.lblPictureFront),
                                        true,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "PictureFront");        
        fields[Software._H_PICTUREBACK] = new DCFieldDefinition(Resources.getText(DCLabels.lblPictureBack),
                                        true,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "PictureBack");        
        fields[Software._I_PICTURECD] = new DCFieldDefinition(Resources.getText(DCLabels.lblPictureCD),
                                        true,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "PictureCD");        
        fields[Software._J_SCREENSHOTONE] = new DCFieldDefinition(Resources.getText(DCLabels.lblScreenshotOne),
                                        true,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "ScreenshotOne");        
        fields[Software._K_SCREENSHOTTWO] = new DCFieldDefinition(Resources.getText(DCLabels.lblScreenshotTwo),
                                        true,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "ScreenshotTwo");        
        fields[Software._L_SCREENSHOTTHREE] = new DCFieldDefinition(Resources.getText(DCLabels.lblScreenshotThree),
                                        true,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "ScreenshotThree");        
        fields[Software._M_LOCATION] = new DCFieldDefinition(Resources.getText(DCLabels.lblLocation),
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Location");         
    }
    
    private class CatagoryFormMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            SoftwareCategoryModule softCatMod = 
                (SoftwareCategoryModule) DCModules.getModule(DCModules._SOFTWARECAT);
            SoftwareCategoryForm categoryForm = softCatMod.getForm(); 
            categoryForm.setVisible(true);
        }
    }
}


