/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects;

import java.io.File;
import java.util.Enumeration;
import java.util.Vector;

import net.sf.dc.console.QuestionBox;
import net.sf.dc.messages.Messages;
import net.sf.dc.messages.MessagesWriter;
import net.sf.dc.resources.DCMessages;
import net.sf.ext.util.Converter;
import net.sf.ext.util.Resources;

/**
 * Holder of all the drives on the system. An array of drives,
 * A: to Z: is tested if they exist on the system and passed on
 * another array which holds all available drives.
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class DriveReader {

    QuestionBox questionBox;

    /**
     * Retrieves all files and directories from the given locations
     *
     * @param sDrive starting location for reading data
     * @param bConfirmation ask for confirmation for eah directory
     * @param bLeveledSearch read subdirectories
     * @param bReturnDirectories allow directory names in the result
     * @param vExtensions extensions to filter on
     */
    public Vector getInformation(String sDrive,
                                 boolean bConfirmation,
                                 boolean bLeveledSearch,
                                 boolean bReturnDirectories,
                                 Vector vExtensions) throws Exception {

        Vector result = new Vector();
        File directory = new File(sDrive + File.separator);
        Object oTrue = new Boolean(true);

        if (!directory.exists()) {
            return null;
        }

        boolean bCriterium = false;
        if (vExtensions != null && vExtensions.size() > 0) {
            bCriterium = true;
        }

        Vector vUnhandeld = Converter.stringArrayToVector(directory.list());
        File tempFile = null;

        while (vUnhandeld.size() > 0) {
            String sCurrent = sDrive + File.separator + vUnhandeld.firstElement().toString();
            tempFile = new File(sCurrent);
            if (tempFile.isDirectory()) {
                if (bConfirmation) {
                    questionBox = new QuestionBox(Resources.getText(DCMessages.msgImport, sCurrent),
                                                  MessagesWriter.logForm);
                }
                
                if ((bConfirmation && QuestionBox.result.equals(oTrue)) ||
                    !bConfirmation) {
                        
					QuestionBox.result = null;
                    Messages.setMessage(
                        Resources.getText(DCMessages.msgReadingFrom, sCurrent), 
                        Messages._STANDARD);

                    String[] list = tempFile.list();
                    if (list != null) {
                        for (int j = 0; j < list.length; j++) {
                            String sTempCurrent = sCurrent + File.separator + list[j];
                            if (!vUnhandeld.contains(sTempCurrent)) {
                                tempFile = new File(sTempCurrent);
                                if (tempFile.isDirectory()) {
                                    vUnhandeld.addElement(sTempCurrent.substring(sDrive.length() + 1, 
                                                                                 sTempCurrent.length()));
                                } else {
                                    if (bCriterium && criteriumApplied(sTempCurrent, vExtensions)) {
                                        result.addElement(sTempCurrent);
                                    } else if (!bCriterium) {
                                        result.addElement(sTempCurrent);
                                    }
                                }
                            }
                        }
                    }
                }  else {
					QuestionBox.result = null;
                    
                    if (bReturnDirectories) {
                        Messages.setMessage(
                            Resources.getText(DCMessages.msgRegDirName, sCurrent), 
                            Messages._STANDARD);
                        result.addElement(sCurrent);
                    }
                }
            } else {
                if (bCriterium && criteriumApplied(sCurrent, vExtensions)) {
                    result.addElement(sCurrent);
                } else if (!bCriterium) {
                    result.addElement(sCurrent);
                }
            }

            vUnhandeld.removeElementAt(0);

            if (!bLeveledSearch) {
                return result;
            }
        }
        result.trimToSize();
        return result;
    }

    /**
     * Checks if the file name has one of the provided extensions
     * @param filename filename to check on
     * @param vExtensions the extension for which is checked
     */
    private static boolean criteriumApplied(String filename, 
                                            Vector vExtensions) {
        int last = filename.lastIndexOf(".");
        boolean equals = false;
        if (last + 1 < filename.length()) {
            filename = filename.substring(last + 1, filename.length());
            
            for (Enumeration enum = vExtensions.elements(); enum.hasMoreElements(); ) {
                String criterium = enum.nextElement().toString().toLowerCase();
                equals = criterium.equals(filename.toLowerCase());
                
                if (equals) {
                    return equals;
                }
            }
        }
        return equals;
    }
}
