/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects;

import java.util.Hashtable;

import javax.swing.ImageIcon;

import net.sf.dc.core.objects.cache.CacheManager;

/**
 * Caches small icons retrieved from the database
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC1.00
 */
public class IconCache {
    
    // key = object ID / value = image icon
    private final static Hashtable icons = new Hashtable();
    
    /**
     * Refreshes the cached icons for the Software Categories
     * The icons are loaded from the database and then stored
     * in a hashtable. Key = object ID / Value = image icon
     */
    public static void refresh() {
        icons.clear();
        
        DCObject[] objects = CacheManager.getObjects(
                              CacheManager._SOFTWARECATEGORY);

        for (int i = 0; i < objects.length; i++) {
            ImageIcon icon = objects[i].getIcon();
            if (icon != null) {
            	icons.put(objects[i].getID(), icon);
            }
        }
    }
    
    /**
     * Return the icon for the object with the given ID
     * @param sID Object ID
     */
    public static ImageIcon getIconForId(String sID) {
        if (icons.containsKey(sID) && sID.length() > 0) {
        	return (ImageIcon) icons.get(sID);
        } else {
        	return null;
        }
    }
}
