/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.db;

/**
 * Holder of the query options used for a query. the options are:
 * 'Order by field', 'Select with precision' (instead of %like%) and 
 * 'All conditions must apply'
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class QueryOptions {

    private String sOrderByField;
    private boolean bPreciseSelect;
    private boolean bComplyToAllConditions;

    /**
     * Creates a new query options object
     * @param sOrderByField field name on which the query should order the results
     * @param bComplyToAllConditions specifies if all conditions must apply
     * @param bPreciseSelect specifies if the search should be done precise
     *        ('=' instead of '%like%')
     */
    public QueryOptions(String sOrderByField, boolean bComplyToAllConditions, boolean bPreciseSelect) {
        this.sOrderByField = sOrderByField;
        this.bComplyToAllConditions = bComplyToAllConditions;
        this.bPreciseSelect = bPreciseSelect;
    }

    /**
     * Returns the field name on which the query should order the results
     */
    public String getOrderByField() {
        return sOrderByField;
    }

     /**
     * Specifies if all conditions must apply
     */
    public boolean getComplyAllConditions() {
        return bComplyToAllConditions;
    }

    /**
     * Specifies whether the selection should be precise 
     * ('=' instead of '%like%')
     */
    public boolean getPreciseSelect() {
        return bPreciseSelect;
    }
}
