/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.http;

import java.net.URL;
import java.util.Vector;

import net.sf.dc.console.MessageBox;
import net.sf.dc.core.objects.software.Software;
import net.sf.dc.messages.Messages;
import net.sf.ext.util.StringUtils;

/**
 * Searches for software descriptions and screenshots on the Amazon site
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.80
 */
public class AmazonSoftwareQuery extends HttpQuery {

    private String searchString = "";
    private boolean broadband = true;

    public AmazonSoftwareQuery(String searchString, boolean broadband) {
        this.searchString = searchString;
        this.broadband = broadband;
    }

    /**
     * Searches on the Amazon.com website on the catagories software and
     * videogames. Returns a Vector of Strings. Each String describes 
     * a found item on the Amazon website
     */
    public Vector search() {
        Vector v = new Vector(0);
        
        try {
            searchString = StringUtils.replaceAll(searchString, " ", "%20");
            URL url = new URL("http://xml.amazon.com/onca/xml3?t=webservices-20&" +
                              "dev-t=test&KeywordSearch=" + searchString +
                              "&mode=videogames&type=heavy&page=1&f=xml");
            String sWebPage1 = HttpConnection.getString(url);
            
            url =     new URL("http://xml.amazon.com/onca/xml3?t=webservices-20&" +
                              "dev-t=test&KeywordSearch=" + searchString +
                              "&mode=software&type=heavy&page=1&f=xml");
            String sWebPage2 = HttpConnection.getString(url);
            String sWebPage = sWebPage1 + sWebPage2;
            v = split(sWebPage);
        } catch (Exception e) {
            new MessageBox(e.toString(), MessageBox._ERROR);
        }
        return v;
    }

    /**
     * Parses information from a raw html or xml string 
     * @param rawResult the raw input
     */
    public Software parseInformation(String rawResult) {
        Software software = new Software();
        
        int indexBegin = 0;
        int indexEnd = 0;

        String productName = "";
        indexBegin = rawResult.indexOf("<ProductName>");
        if (indexBegin != -1) {
            indexBegin = indexBegin + 13;
            indexEnd = rawResult.indexOf("</ProductName>");
            productName = clean(rawResult.substring(indexBegin, indexEnd));
        }
        
        String amazonUrl = "";
        indexBegin = rawResult.indexOf("url=");
        if (indexBegin > -1) {
            indexBegin = indexBegin + 5;
            indexEnd = rawResult.indexOf(">") - 1;
            amazonUrl = rawResult.substring(indexBegin, indexEnd);
        }        

        String sID = "";
        if (amazonUrl.indexOf("ASIN/") > 0) {
        	sID = amazonUrl.substring(amazonUrl.indexOf("ASIN/") + 5, amazonUrl.length());
        	sID = sID.substring(0, sID.indexOf("/"));
        }
        
        indexBegin = rawResult.indexOf("<ImageUrlLarge>");
        if (indexBegin != -1) {
            try {
            	String quality = broadband ? "LZZZZZZZ.jpg" : "MZZZZZZZ.jpg";
                
                URL url = new URL("http://images.amazon.com/images/P/" + 
                                  sID + ".01." + quality);
                software.setValue(Software._G_PICTUREFRONT, HttpConnection.getImageAsBase64(url));
                
            	url =   new URL("http://images.amazon.com/images/P/" +  
                                 sID + ".01.PT01." + quality);
                software.setValue(Software._J_SCREENSHOTONE, HttpConnection.getImageAsBase64(url));
                

                url =   new URL("http://images.amazon.com/images/P/" +  
                		        sID + ".01.PT02." + quality);
                software.setValue(Software._K_SCREENSHOTTWO, HttpConnection.getImageAsBase64(url));

                url =   new URL("http://images.amazon.com/images/P/" +  
                        sID + ".01.PT03." + quality);
                software.setValue(Software._L_SCREENSHOTTHREE, HttpConnection.getImageAsBase64(url));
            } catch (Exception exp) {
                Messages.setMessage(exp);
            }
        }        
        
        software.setValue(Software._A_NAME, clean(productName));
        software.setValue(Software._B_DESCRIPTION, getDescription(rawResult));
        software.setValue(Software._C_WEBPAGE, amazonUrl); 
        return software;
    }

    /**
     * Creates a description out of several pieces of information
     * @param rawResult the raw input string
     */
    private String getDescription(String rawResult) {
        int indexBegin = 0;
        int indexEnd = 0;

        String description = "";
        indexBegin = rawResult.indexOf("<ProductDescription>");
        if (indexBegin != -1) {
            indexBegin = indexBegin + 20;
            indexEnd = rawResult.indexOf("</ProductDescription>");
            description = rawResult.substring(indexBegin, indexEnd);

            indexBegin = description.indexOf("&lt;/I>");
            if (indexBegin > -1) {

            }
            description = "\n" + "\n" + "Description: " + "\n" + description;
        }

        String manufacturer = "";
        indexBegin = rawResult.indexOf("<Manufacturer>");
        if (indexBegin > -1) {
            indexBegin = indexBegin + 14;
            indexEnd = rawResult.indexOf("</Manufacturer>");
            manufacturer = rawResult.substring(indexBegin, indexEnd);
            manufacturer = "Manufactured by: " + manufacturer;
        }

        String platforms = "";
        indexBegin = rawResult.indexOf("<Platforms>");
        if (indexBegin > -1) {
            indexBegin = indexBegin + 11;
            indexEnd = rawResult.indexOf("</Platforms>");
            platforms = rawResult.substring(indexBegin, indexEnd);

            String temp = "";
            indexBegin = platforms.indexOf("<Platform>");
            while(indexBegin > -1) {
                indexBegin = indexBegin + 10;
                indexEnd = platforms.indexOf("</Platform>");

                temp = temp + platforms.substring(indexBegin, indexEnd) + " ";

                indexEnd = indexEnd + 11;
                platforms = platforms.substring(indexEnd, platforms.length());
                indexBegin = platforms.indexOf("<Platform>");
            }
            platforms = temp;
            platforms = "\n" + "\n" + "Supported platforms: " + "\n" + platforms;
        }

        String features = "";
        indexBegin = rawResult.indexOf("<Features>");
        if (indexBegin > -1) {
            indexBegin = indexBegin + 10;
            indexEnd = rawResult.indexOf("</Features>");
            features = rawResult.substring(indexBegin, indexEnd);

            String temp = "";
            indexBegin = features.indexOf("<Feature>");
            while(indexBegin > -1) {
                indexBegin = indexBegin + 9;
                indexEnd = features.indexOf("</Feature>");

                temp = temp + features.substring(indexBegin, indexEnd) + ". ";

                indexEnd = indexEnd + 10;
                features = features.substring(indexEnd, features.length());
                indexBegin = features.indexOf("<Feature>");
            }
            features = temp;
            features = "\n" + "\n" + "Features: " + "\n" + features;
        }

        String releaseDate = "";
        indexBegin = rawResult.indexOf("<ReleaseDate>");
        if (indexBegin != -1) {
            indexBegin = indexBegin + 13;
            indexEnd = rawResult.indexOf("</ReleaseDate>");
            releaseDate = rawResult.substring(indexBegin, indexEnd);
            releaseDate = "\n" + "Released on: " + releaseDate;
        }

        description = manufacturer +
                      releaseDate +
                      description +
                      features +
                      platforms;

        return clean(description);
    }

    /**
     * Splits one search result in to pieces. Every piece (String)
     * is a presentation of one record (raw data)
     * @param rawResult
     */
    private Vector split(String rawResult) {
        Vector v = new Vector(0);

        int index = rawResult.indexOf("<Details");
        while (index > -1) {
            int end = rawResult.indexOf("</Details>") + 10;
            if (end != -1) {
                String part = rawResult.substring(index, end);
                v.add(part);
                rawResult = rawResult.substring(end, rawResult.length());
            }
            index = rawResult.indexOf("<Details");
        }
        return v;
    }
}
