/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes;

import net.sf.dc.console.movie.InsertMoviePanel;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.core.objects.movie.MovieFile;
import net.sf.dc.messages.Messages;
import net.sf.ext.mediamanager.FilePropertiesMovie;

/**
 * Extract information from movie files (ogg, avi, mpeg, divx, xvid)
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class ImportMovieCD  extends ImportCDProcess {

    /**
     * Initializes (but does  not run) the Movie CD import process
     * @param sourceDir directory which is used as source for the
     *        import
     * @param confirmEachDir indicates whether for each directory
     *        confirmation should be asked if the underlying data
     *        of this directory should be read or not.
     * @param nameOfCD cd name (used as the location name)
     */      
    public ImportMovieCD( String sourceDir,
                           boolean confirmEachDir,
                           String cdName) {
        super(sourceDir, confirmEachDir, cdName);
    }

    /**
     * The UI class which will be updated with the results of the 
     * import 
     */    
    protected Class getUITargetClass() {
        return InsertMoviePanel.class;
    }

    /**
     * Parses the source file for information. 
     */    
    protected DCObject parseInformation(String sFile) {
        Movie movie = new Movie();
        try {
            FilePropertiesMovie movieFile1 = new FilePropertiesMovie(sFile);
            Integer playlength = new Integer(movieFile1.getDuration() / 60);
            
            if (playlength.intValue() == 0 && movieFile1.getVideoWidth() <= 0) {
                return null;
            }
            
            movie.setValue(Movie._A_TITLE, movieFile1.getVideoName());
            
            movie.setValue(Movie._I_PLAYLENGTH, playlength); 
            movie.setValue(Movie._J_FILENAME, sFile.substring(2, sFile.length()));
            movie.setValue(Movie._K_VIDEOCODEC, movieFile1.getVideoCodec());
            movie.setValue(Movie._L_AUDIOCODEC, movieFile1.getAudioCodec());
            movie.setValue(Movie._M_WIDTH, new Integer(movieFile1.getVideoWidth()));
            movie.setValue(Movie._N_HEIGHT, new Integer(movieFile1.getVideoHeight()));
            movie.setValue(Movie._O_FPS, new Integer(movieFile1.getVideoFps()));
            movie.setValue(Movie._P_FRAMES, new Integer(movieFile1.getDuration() * movieFile1.getVideoFps()));
            movie.setValue(Movie._Q_AUDIOBITRATE, new Integer(movieFile1.getAudioBitRate()));
            movie.setValue(Movie._R_AUDIOSAMPLINGRATE, new Integer(movieFile1.getAudioSampleRate()));
            movie.setValue(Movie._S_AUDIOCHANNEL, new Integer(movieFile1.getAudioChannels()));
            
            int bitrate = movieFile1.getVideoBitRate();
            if (bitrate <= 0) {
                try {
                    MovieFile movieFile2 = new MovieFile(sFile);
                    bitrate = new Double(movieFile2.getVideoBitrate()).intValue();
                } catch (Exception ignore) {}
            }
            
            movie.setValue(Movie._T_VIDEOBITRATE, new Integer(bitrate));
            movie.setValue(Movie._X_LOCATION, cdName);
            return movie;
        } catch (Exception exp) {
            Messages.setMessage(exp);
            return null;
        }
    }
}
