/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes;
import net.sf.dc.console.software.InsertSoftwarePanel;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.software.Software;
import net.sf.dc.messages.Messages;

/**
 * Imports data from a CD (or harddisk) location into the insert table
 * of the owner module
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class ImportSoftwareCD extends ImportCDProcess {

    /**
     * Initializes (but does  not run) the Movie CD import process
     * @param sourceDir directory which is used as source for the
     *        import
     * @param confirmEachDir indicates whether for each directory
     *        confirmation should be asked if the underlying data
     *        of this directory should be read or not.
     * @param nameOfCD cd name (used as the location name)
     */      
    public ImportSoftwareCD(String sourceDir,
                            boolean confirmEachDir,
                            String cdName) {
        super(sourceDir, confirmEachDir, cdName);
    }
   
    /**
     * The UI class which will be updated with the results of the 
     * import 
     */    
    protected Class getUITargetClass() {
        return InsertSoftwarePanel.class;
    }   

    /**
     * Parses the source file for information. 
     */ 
    protected DCObject parseInformation(String sFile) {
        Software software = new Software();
        try {
            String sFilename = sFile.substring(2, sFile.length());
            software.setValue(Software._D_FILENAME, sFilename);
            software.setValue(Software._M_LOCATION, cdName);
            return software;
        } catch (Exception exp) {
            Messages.setMessage(exp);
            return null;
        }        
    }
}
