/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   * 
 *                           sub-project: JFlexSett                           *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.settings;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.io.File;

import javax.swing.JTextField;

import net.sf.dc.console.components.ColorField;
import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.core.objects.music.Music;
import net.sf.dc.core.objects.software.Software;
import net.sf.dc.core.objects.software.SoftwareCategory;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCTooltips;
import net.sf.ext.jflexsett.settings.Setting;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.jflexsett.settings.SettingsFile;
import net.sf.ext.jflexsett.settings.SettingsGroup;
import net.sf.ext.util.Resources;

/**
 * Groups are created here, as well as the settings.
 * A similar class like this needs to be implemented in the
 * project which uses jFlexSett.
 * 
 * All settings and their values (and the settings groups) can
 * be found by referencing to the Settings class. 
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class SettingInitializator {

    public static final String _General = 
                                Resources.getText(DCLabels.lblGroupGeneral);
    public static final String _RequiredSoftFields = 
                                Resources.getText(DCLabels.lblGroupSoftRequired);
    public static final String _RequiredMusicFields = 
                                Resources.getText(DCLabels.lblGroupMusicRequired);
    public static final String _RequiredMovieFields = 
                                Resources.getText(DCLabels.lblGroupMovieRequired);
    public static final String _BackGroundColor = 
                                Resources.getText(DCLabels.lblColor);
    public static final String _TableColors =
                                Resources.getText(DCLabels.lblTableColorSettings);
    public static final String _OddRowColor = 
                                Resources.getText(DCLabels.lblOddRowColor);
    public static final String _EvenRowColor = 
                                Resources.getText(DCLabels.lblEvenRowColor);
    public static final String _HeaderColor =  
                                Resources.getText(DCLabels.lblTableHeaderColor);
    public static final String _HTTP =
                                Resources.getText(DCLabels.lblHttp);
    public static final String _Font =
                                Resources.getText(DCLabels.lblFont);
                                
    
    /**
     * Starts the initialization of the Settings and starts the UI
     */
    public SettingInitializator() {
        createGroups();
        createSettings();

        Settings.setSettingsFile(new File(Repository.files.flIniFile));
        SettingsFile.load();
    }

    /**
     * Creates the settings group (with their hierarchy)
     */
    public void createGroups() {
        SettingsGroup generalGroup = new SettingsGroup(_General);
        SettingsGroup tableColorGroup = new SettingsGroup(_TableColors);
        SettingsGroup backGround = new SettingsGroup(_BackGroundColor);
        SettingsGroup font = new SettingsGroup(_Font);
        SettingsGroup http = new SettingsGroup(_HTTP);        
        
        generalGroup.addChild(new SettingsGroup(_RequiredSoftFields));
        generalGroup.addChild(new SettingsGroup(_RequiredMusicFields));
        generalGroup.addChild(new SettingsGroup(_RequiredMovieFields));
        
        tableColorGroup.addChild(new SettingsGroup(_OddRowColor));
        tableColorGroup.addChild(new SettingsGroup(_EvenRowColor));
        tableColorGroup.addChild(new SettingsGroup(_HeaderColor));
        
        Settings.addGroup(_General, generalGroup);
        Settings.addGroup(_BackGroundColor, backGround);
        Settings.addGroup(_HTTP, http);
        Settings.addGroup(_TableColors, tableColorGroup);
        Settings.addGroup(_Font, font);
    }
    
    /**
     * Creates the settings and adds them to the specified group.
     * If a self-created component needs to be added, make sure this
     * components implements the ISettingsComponent class!
     * Otherwise the component is not supported.. The getValue and
     * setValue must correspond to the DataType of the setting.
     */
    public void createSettings() {
        Settings.addSetting(_BackGroundColor,                           // Group key
                    new Setting(Repository.datatypes._COLOR,            // Data type
                                Repository.settings.stColor,            // Key
                                new Color(223, 224, 224),               // Default value
                                new ColorField(true, 
                                    Repository.settings.stColor),       // UI Component
                                "",                                     // Help text
                                Resources.getText(DCLabels.lblColor),   // Label text
                                false,                                  // show label ?
                                true));                                 // Add to panel ?

        Settings.addSetting(_OddRowColor, 
                    new Setting(Repository.datatypes._COLOR, 
                    		    Repository.settings.stOddRowColor,
								ComponentFactory.getOddRowColor(), 
                                new ColorField(false,
                                    Repository.settings.stOddRowColor),
							    "", 
                                Resources.getText(DCLabels.lblOddRowColor), 
                                false, 
                                true));

        Settings.addSetting(_EvenRowColor, 
                    new Setting(Repository.datatypes._COLOR,
                    		    Repository.settings.stEvenRowColor, 
                                ComponentFactory.getEvenRowColor(), 
                                new ColorField(false,
                                    Repository.settings.stEvenRowColor),
                                "",
								Resources.getText(DCLabels.lblEvenRowColor), 
                                false, 
                                true));
        
        Settings.addSetting(_HeaderColor, 
                    new Setting(Repository.datatypes._COLOR, 
                                Repository.settings.stTableHeaderColor,
                                new Color(220, 220, 220), 
                                new ColorField(false,
                                    Repository.settings.stTableHeaderColor),
                                "", 
                                Resources.getText(DCLabels.lblTableHeaderColor), 
                                false, 
                                true));         
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.settings.stCheckUniqueness,
                                new Boolean(true),
                                ComponentFactory._CHECKBOX,
                                Resources.getText(DCTooltips.tpUniqueness),
                                Resources.getText(DCLabels.lblUniqueness),
                                false,
                                true));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.settings.stCheckRequiredFields,
                                new Boolean(true),
                                ComponentFactory._CHECKBOX,
                                Resources.getText(DCTooltips.tpRequiredFields),
                                Resources.getText(DCLabels.lblRequired),
                                false,
                                true));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stConnectionString,
                                "dc",
                                ComponentFactory._SHORTTEXTFIELD,
                                Resources.getText(DCTooltips.tpConnectionString),
                                Resources.getText(DCLabels.lblConnectionString),
                                true,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stDatabaseDriver,
                                "org.hsqldb.jdbcDriver",
                                ComponentFactory._SHORTTEXTFIELD,
                                Resources.getText(DCTooltips.tpDatabaseDriver),
                                Resources.getText(DCLabels.lblDatabaseDriver),
                                true,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.settings.stShowTableTooltip,
                                new Boolean(true),
                                ComponentFactory._CHECKBOX,
                                Resources.getText(DCTooltips.tpShowTableTooltips),
                                Resources.getText(DCLabels.lblShowTableTooltips),
                                false,
                                true));
        Settings.addSetting(_General,
                    new Setting(  Repository.datatypes._INTEGER,
                                Repository.settings.stMaximumSearchRes,
                                new Integer(10000),
                                ComponentFactory._NUMBERFIELD,
                                Resources.getText(DCTooltips.tpSearchRes),
                                Resources.getText(DCLabels.lblSearchResults),
                                true,
                                true));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._INTEGER,
                                Repository.settings.stModule,
                                new Integer(0),
                                ComponentFactory._SHORTTEXTFIELD,
                                "",
                                "",
                                true,
                                false));
        Settings.addSetting(_Font,
                    new Setting(Repository.datatypes._FONT,
                    		    Repository.settings.stSystemFontNormal,
								new Font("Arial Narrow", Font.PLAIN, 11),
								ComponentFactory._FONTSELECTOR,
								Resources.getText(DCTooltips.tpFont),
								Resources.getText(DCLabels.lblFontNormal),
								true,
								true));
        Settings.addSetting(_Font,
                    new Setting(Repository.datatypes._FONT,
                    		    Repository.settings.stSystemFontBold,
								new Font("Arial Narrow", Font.PLAIN, 11),
								ComponentFactory._FONTSELECTOR,
								Resources.getText(DCTooltips.tpFont),
								Resources.getText(DCLabels.lblFontBold),
								true,
								true));        
         Settings.addSetting(_HTTP,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stODB,
                                "www.imdb.com",
                                ComponentFactory._SHORTTEXTFIELD,
                                Resources.getText(DCTooltips.tpODB),
                                Resources.getText(DCLabels.lblODB),
                                true,
                                false));
        Settings.addSetting(_HTTP,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stHTTPUserAgent,
                                "Mozilla/4.04 (Windows; I; Win95)",
                                ComponentFactory._SHORTTEXTFIELD,
                                Resources.getText(DCTooltips.tpHTTPUserAgent),
                                Resources.getText(DCLabels.lblHTTPUserAgent),
                                true,
                                true));
        Settings.addSetting(_HTTP,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stProxyServerName,
                                "",
                                ComponentFactory._SHORTTEXTFIELD,
                                Resources.getText(DCTooltips.tpProxyServerName),
                                Resources.getText(DCLabels.lblProxyServerName),
                                true,
                                true));
        Settings.addSetting(_HTTP,
                    new Setting(Repository.datatypes._INTEGER,
                                Repository.settings.stProxyServerPort,
                                new Integer(0),
                                ComponentFactory._NUMBERFIELD,
                                Resources.getText(DCTooltips.tpProxyServerPort),
                                Resources.getText(DCLabels.lblProxyServerPort),
                                true,
                                true));
        Settings.addSetting(_RequiredMusicFields,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stMusicRequiredFields,
                                new int[] {Music._B_TITLE,
                                           Music._K_LOCATION},
                                ComponentFactory._MUSICREQFIELDTABLE,
                                "",
                                "",
                                false,
                                true));
        Settings.addSetting(_RequiredMovieFields,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stMovieRequiredFields,
                                new int[] {Movie._A_TITLE,
                                           Movie._X_LOCATION},
								ComponentFactory._MOVIEREQFIELDTABLE,
                                "",
                                "",
                                false,
                                true));
        Settings.addSetting(_RequiredSoftFields,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stSoftwareRequiredFields,
                                new int[] {Software._D_FILENAME,
                                           Software._A_NAME,
                                           Software._M_LOCATION},
                                ComponentFactory._SOFTWAREREQFIELDTABLE,
                                "",
                                "",
                                false,
                                true));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stSoftwareCatRequiredFields,
                                new int[] {SoftwareCategory._A_NAME},
                                ComponentFactory.getComboBox(),
                                "",
                                "",
                                false,
                                false));
        Settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stSoftwareCatFormSize,
                            new Dimension(450, 550),
                            ComponentFactory._SHORTTEXTFIELD,
                            "",
                            "",
                            false,
                            false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stSoftwareRecordFormViewSize,
                                new Dimension(450, 550),
                                ComponentFactory._SHORTTEXTFIELD,
                                "",
                                "",
                                false,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stMusicRecordFormViewSize,
                                new Dimension(450, 550),
                                ComponentFactory._SHORTTEXTFIELD,
                                "",
                                "",
                                false,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stMovieRecordFormViewSize,
                                new Dimension(450, 550),
                                ComponentFactory._SHORTTEXTFIELD,
                                "",
                                "",
                                false,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stRecordFormViewSize,
                                new Dimension(450, 550),
                                ComponentFactory._SHORTTEXTFIELD,
                                "",
                                "",
                                false,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stSoftwareCatRecordFormViewSize,
                                new Dimension(407, 165),
                                ComponentFactory._SHORTTEXTFIELD,
                                "",
                                "",
                                false,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stMusicOrganizerViewSize,
                                new Dimension(300, 400),
                                ComponentFactory._SHORTTEXTFIELD,
                                "",
                                "",
                                false,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stUpdateAllDialogSize,
                                new Dimension(600, 300),
                                ComponentFactory._SHORTTEXTFIELD,
                                "",
                                "",
                                false,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stMainViewSize,
                                new Dimension(865, 593),
                                ComponentFactory._SHORTTEXTFIELD,
                                "",
                                "",
                                false,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stSettingsViewSize,
                                new Dimension(800, 450),
                                ComponentFactory._SHORTTEXTFIELD,
                                "",
                                "",
                                false,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stFieldSelectorSize,
                                new Dimension(500, 300),
                                ComponentFactory._SHORTTEXTFIELD,
                                "",
                                "",
                                false,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stTextViewerSize,
                                new Dimension(500, 700),
                                ComponentFactory._SHORTTEXTFIELD,
                                "",
                                "",
                                false,
                                false));
         Settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stImdbSearchFormViewSize,
                                new Dimension(400, 400),
                                ComponentFactory._SHORTTEXTFIELD,
                                "",
                                "",
                                false,
                                false));
        Settings.addSetting(_General,
                   new Setting(Repository.datatypes._DIMENSION,
                               Repository.settings.stImportCDDialogSize,
                               new Dimension(400, 200),
                               ComponentFactory._SHORTTEXTFIELD,
                               "",
                               "",
                               false,
                               false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.settings.stODBOverwriteValues,
                                new Boolean(false),
                                ComponentFactory._CHECKBOX,
                                "",
                                "",
                                false,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.settings.stBroadband,
                                new Boolean(true),
                                ComponentFactory._CHECKBOX,
                                "",
                                "",
                                false,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.settings.stCloseServerQuestion,
                                new Boolean(true),
                                ComponentFactory._CHECKBOX,
                                Resources.getText(DCTooltips.tpCloseServer),
                                Resources.getText(DCLabels.lblCloseServer),
                                false,
                                true));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stHiddenFieldsSoftware,
                                new int[] {0},
                                new JTextField(),
                                "",
                                "",
                                false,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stHiddenFieldsMovie,
                                new int[] {0},
                                new JTextField(),
                                "",
                                "",
                                false,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stHiddenFieldsMusic,
                                new int[] {0},
                                new JTextField(),
                                "",
                                "",
                                false,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stHiddenFieldsSoftwareCat,
                                new int[] {0},
                                new JTextField(),
                                "",
                                "",
                                false,
                                false));
        Settings.addSetting(_General,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stLastDirectoryUsed,
                                "",
                                new JTextField(),
                                "",
                                "",
                                false,
                                false));
    }
}
