/*
 *  ********************************************************************   **
 *  Copyright notice                                                       **
 *  **																	   **
 *  (c) 2003 Entagged Developpement Team				                   **
 *  http://www.sourceforge.net/projects/entagged                           **
 *  **																	   **
 *  All rights reserved                                                    **
 *  **																	   **
 *  This script is part of the Entagged project. The Entagged 			   **
 *  project is free software; you can redistribute it and/or modify        **
 *  it under the terms of the GNU General Public License as published by   **
 *  the Free Software Foundation; either version 2 of the License, or      **
 *  (at your option) any later version.                                    **
 *  **																	   **
 *  The GNU General Public License can be found at                         **
 *  http://www.gnu.org/copyleft/gpl.html.                                  **
 *  **																	   **
 *  This copyright notice MUST APPEAR in all copies of the file!           **
 *  ********************************************************************
 */
package net.sf.ext.entagged.libraries.audioformats;

import java.io.File;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.ImageIcon;

import net.sf.ext.entagged.libraries.audioformats.interfaces.AudioFormatInterface;
import net.sf.ext.entagged.libraries.audioformats.interfaces.EncodingInfoReader;
import net.sf.ext.entagged.libraries.audioformats.interfaces.Tag;
import net.sf.ext.entagged.libraries.audioformats.interfaces.TagReaderInterface;
import net.sf.ext.entagged.libraries.audioformats.interfaces.TagTypeInterface;
import net.sf.ext.entagged.libraries.audioformats.interfaces.TagWriterInterface;
import net.sf.ext.entagged.libraries.exceptions.RequestedClassNotFoundException;
import net.sf.ext.entagged.libraries.utils.Utils;

public class AudioFormatManager {
	
	private static Hashtable ext2AudioFormat = new Hashtable();
    
	private static TagTypeInterface[] availableTagTypes;
	private static String[] availableExtensions;
	
    static {
        registerAudioTypes();
        createAvailableTagTypes();
        createAvailableExtensions();
    }
    
	public static EncodingInfoReader getEncodingInfoReader( File f )  throws RequestedClassNotFoundException {
			String ext = Utils.getExtension(f);
			return getAudioFormat(ext).getEncodingInfoReader();
	}
	
	private static AudioFormatInterface getAudioFormat(String ext) throws RequestedClassNotFoundException {
		Object o = ext2AudioFormat.get(ext);
		if(o!= null)
			return (AudioFormatInterface) o;
		else
			throw new RequestedClassNotFoundException( "No support for format -" + ext + "- found:\n");
	}
	
	public static TagTypeInterface getDefaultTagType( File f) throws RequestedClassNotFoundException {
		String ext = Utils.getExtension(f);
		return getAudioFormat(ext).getDefaultTagType();
	}
	
	public static TagTypeInterface[] getAllTagTypes( File f ) throws RequestedClassNotFoundException {
		String ext = Utils.getExtension(f);
		return getAudioFormat(ext).getSupportedTagTypes();
	}

	public static Tag getTag(File f, String tagType)  throws RequestedClassNotFoundException {
		String ext = Utils.getExtension(f);
		return getAudioFormat(ext).getTagType(tagType).getEmptyTag();
	}
	
	public static Tag getTag(File f)  throws RequestedClassNotFoundException {
		String ext = Utils.getExtension(f);
		return getAudioFormat(ext).getDefaultTagType().getEmptyTag();
	}
	
	public static TagReaderInterface[] getTagReaders( File f )  throws RequestedClassNotFoundException {
		String ext = Utils.getExtension(f);
		return getAudioFormat(ext).getTagReaders();
	}
	
	public static TagWriterInterface getTagWriter( File f )  throws RequestedClassNotFoundException {
		String ext = Utils.getExtension(f);
		return getAudioFormat(ext).getDefaultTagType().getTagWriter();
	}
	
	public static TagWriterInterface getTagWriter( File f, String tagType )  throws RequestedClassNotFoundException {
		String ext = Utils.getExtension(f);
		return getAudioFormat(ext).getTagType(tagType).getTagWriter();
	}
	
	public static ImageIcon getIcon(File f, String tagType) throws RequestedClassNotFoundException {
		String ext = Utils.getExtension(f);
		return getAudioFormat(ext).getTagType(tagType).getIcon();
	}
	
	public static ImageIcon getIcon(File f) throws RequestedClassNotFoundException {
		String ext = Utils.getExtension(f);
		return getAudioFormat(ext).getDefaultTagType().getIcon();
	}
	
	public static TagTypeInterface[] getAvailableTagTypes() {
		return availableTagTypes;
	}
	
	public static String[] getAvailableExtensions() {
		return availableExtensions;
	}
	
	private static void createAvailableExtensions() {
		Enumeration e = ext2AudioFormat.keys();
		Vector ext = new Vector();
		while(e.hasMoreElements()) {
			String extension = (String) e.nextElement();
			ext.add(extension);
		}
		
		availableExtensions = new String[ext.size()];
			for(int i = 0; i<availableExtensions.length; i++)
				availableExtensions[i] = (String)ext.elementAt(i);
	}
		
	private static void createAvailableTagTypes() {
		Enumeration e = ext2AudioFormat.keys();
		
		Vector tagTypes = new Vector();
		while(e.hasMoreElements()) {
			String extension = (String) e.nextElement();
			AudioFormatInterface af = (AudioFormatInterface) ext2AudioFormat.get(extension);
			TagTypeInterface[] tt = af.getSupportedTagTypes();
			for(int i=0; i<tt.length; i++) {
				if(!tagTypes.contains(tt[i])) {
					tagTypes.add(tt[i]);
				}
			}
		}
		
        availableTagTypes = new TagTypeInterface[tagTypes.size()];
        for(int i = 0; i<availableTagTypes.length; i++) {
			availableTagTypes[i] = (TagTypeInterface)tagTypes.elementAt(i);
        }
	}
	
	private static void registerAudioTypes() {
        ext2AudioFormat.put("flac", new net.sf.ext.entagged.plugins.audioformats.flac.AudioFormat());
        ext2AudioFormat.put("ogg", new net.sf.ext.entagged.plugins.audioformats.ogg.AudioFormat());
        ext2AudioFormat.put("wav", new net.sf.ext.entagged.plugins.audioformats.wav.AudioFormat());
    }
}
