/*
 *  ********************************************************************   **
 *  Copyright notice                                                       **
 *  **																	   **
 *  (c) 2003 Entagged Developpement Team				                   **
 *  http://www.sourceforge.net/projects/entagged                           **
 *  **																	   **
 *  All rights reserved                                                    **
 *  **																	   **
 *  This script is part of the Entagged project. The Entagged 			   **
 *  project is free software; you can redistribute it and/or modify        **
 *  it under the terms of the GNU General Public License as published by   **
 *  the Free Software Foundation; either version 2 of the License, or      **
 *  (at your option) any later version.                                    **
 *  **																	   **
 *  The GNU General Public License can be found at                         **
 *  http://www.gnu.org/copyleft/gpl.html.                                  **
 *  **																	   **
 *  This copyright notice MUST APPEAR in all copies of the file!           **
 *  ********************************************************************
 */
package net.sf.ext.entagged.libraries.audioformats;

import java.io.File;
import java.util.Vector;

import net.sf.ext.entagged.libraries.audioformats.interfaces.EncodingInfoReader;
import net.sf.ext.entagged.libraries.audioformats.interfaces.Tag;
import net.sf.ext.entagged.libraries.audioformats.interfaces.TagReaderInterface;
import net.sf.ext.entagged.libraries.exceptions.CannotReadException;
import net.sf.ext.entagged.libraries.exceptions.RequestedClassNotFoundException;
import net.sf.ext.entagged.libraries.exceptions.WarningException;

/**
 *  $Id: TagReader.java,v 1.1 2004/09/17 07:15:36 rwaals Exp $
 *
 * @author     Raphael Slinckx (KiKiDonK)
 * @version
 */
public class TagReader {

	public TaggedFile read( File f ) throws CannotReadException {
		EncodingInfo infos = null;
		EncodingInfoReader infoReader = null;
		
		try {
			infoReader = AudioFormatManager.getEncodingInfoReader(f);
		} catch(RequestedClassNotFoundException e) {
			return null;
		}
		
		try{
			infos = infoReader.read(f);
		} catch (CannotReadException e) {
			throw new CannotReadException( e.getMessage());
		}
        
		TaggedFile tf = new TaggedFile(f.getPath(), infos);
		TagReaderInterface[] tagReaders = null;
		try {
			tagReaders = AudioFormatManager.getTagReaders( f );
		} catch ( RequestedClassNotFoundException e ) {
			return tf;
		}

		for (int i = 0; i < tagReaders.length; i++) {
			Tag t = tagReaders[i].read(f);
			tf.addTag(t);
		}

		return tf;
	}


	public TaggedFile[] read(File[] list) throws WarningException{
		Vector v = new Vector();
		Vector errors = new Vector();
		for (int i = 0; i < list.length; i++) {
			try {
				TaggedFile tf = read( list[i] );
				if (tf != null) {
					v.add(tf);
                }
			} catch ( CannotReadException e ) {
				errors.add(e.getMessage());
			}
        }

		TaggedFile[] ret = new TaggedFile[v.size()];
		for (int i = 0; i < v.size(); i++) {
			ret[i] = (TaggedFile) v.elementAt( i );
        }
		
		if (!errors.isEmpty()) {
			String error = "Some recoverable errors occured (files skipped):\n";
            for (int i = 0; i<errors.size(); i++) {
				error+=errors.elementAt(i) + "\n";
            }
			error += "--- end of errors ---";
			throw new WarningException(error, ret);
		} else {
			return ret;
        }
	}
}

