/*
 *  ********************************************************************   **
 *  Copyright notice                                                       **
 *  **																	   **
 *  (c) 2003 Entagged Developpement Team				                   **
 *  http://www.sourceforge.net/projects/entagged                           **
 *  **																	   **
 *  All rights reserved                                                    **
 *  **																	   **
 *  This script is part of the Entagged project. The Entagged 			   **
 *  project is free software; you can redistribute it and/or modify        **
 *  it under the terms of the GNU General Public License as published by   **
 *  the Free Software Foundation; either version 2 of the License, or      **
 *  (at your option) any later version.                                    **
 *  **																	   **
 *  The GNU General Public License can be found at                         **
 *  http://www.gnu.org/copyleft/gpl.html.                                  **
 *  **																	   **
 *  This copyright notice MUST APPEAR in all copies of the file!           **
 *  ********************************************************************
 */
package net.sf.ext.entagged.libraries.audioformats;


import java.io.File;
import java.util.Vector;

import net.sf.ext.entagged.libraries.audioformats.interfaces.Tag;


/**
 *  $Id: TaggedFile.java,v 1.1 2004/09/17 07:15:36 rwaals Exp $
 *
 * @author     Raphael Slinckx (KiKiDonK)
 * @version    16 d�cembre 2003
 */
public class TaggedFile extends File {

	private EncodingInfo encodingInfos;

	private Vector tags;


	public TaggedFile( String file, EncodingInfo infos ) {
		super( file );
		this.tags = new Vector();
		this.encodingInfos = infos;
	}


	public TaggedFile( String file, Vector tags, EncodingInfo infos ) {
		super( file );
		this.tags = tags;
		removeNullTags();
		this.encodingInfos = infos;
	}
	
	public TaggedFile( TaggedFile oldTaggedFile, String newPath ) {
		super( newPath );
		this.tags = oldTaggedFile.getTagsAsVector();
		this.encodingInfos =oldTaggedFile.getEncodingInfos();
	}

	private void removeNullTags() {
		for(int i = 0; i< tags.size(); i++)
			if(tags.elementAt( i )==null)
				tags.removeElementAt(i);
	}

	public EncodingInfo getEncodingInfos() {
		return this.encodingInfos;
	}
	
	public boolean hasTag(String tagType) {
		for (int i = 0; i < tags.size(); i++) {
			Tag t = (Tag) tags.elementAt( i );
			if (t.getTagType().equals(tagType) && !t.isEmpty()) {
				return true;
            }
		}
		return false;
	}
			
	public Tag getTag( String tagType ) {
		if(tagType.equals(DefaultTagType.getInstance().getTagType())) {
			if(getTags().length != 0) {
				return getTags()[0];
            } else {
				return null;
            }
        }
        
		for (int i = 0; i < tags.size(); i++ ) {
			if (((Tag) tags.elementAt( i ) ).getTagType().equals(tagType)) {
				return (Tag) tags.elementAt( i );
            }
		}
		return null;
	}


	public Tag[] getTags() {
		Tag[] tagsArray = new Tag[tags.size()];

		for ( int i = 0; i < tagsArray.length; i++ )
			tagsArray[i] = (Tag) tags.elementAt( i );
		return tagsArray;
	}
	

	public Vector getTagsAsVector() {
		return tags;
	}


	public void addTag( Tag tag ) {
		if(tag != null)
			this.tags.add( tag );
	}
	
	public void updateTag( Tag tag) {
		for (int i = 0; i < tags.size(); i++) {
			Tag t = (Tag) tags.elementAt(i);
			if (t.getTagType().equals(tag.getTagType()) && !t.isEmpty()) {
				tags.removeElementAt(i);
			}
		}
		addTag( tag );
	}
	
	public void updateTags( Tag[] tags) {
		for(int i = 0; i<tags.length; i++)
			updateTag(tags[i]);
	}
	
	public void addTags( Tag[] tags ) {
		for(int i = 0; i<tags.length; i++)
			addTag(tags[i]);
	}

	public String toString() {
		String output = "-----------------------------\n";

		output += "**Fichier: " + this.getPath() + " **\n";
		//output += "Timing: " + this.timeLength + "\t" + (int)( this.timeLength / 60 ) + ":" + ( this.timeLength - 60 * (int)( this.timeLength / 60 ) ) + "\n";
		output += this.encodingInfos.toString();
		for ( int i = 0; i < tags.size(); i++ )
			output += tags.elementAt( i ).toString();
		return output;
	}
}

