/*
 *  ********************************************************************   **
 *  Copyright notice                                                       **
 *  **																	   **
 *  (c) 2003 Entagged Developpement Team				                   **
 *  http://www.sourceforge.net/projects/entagged                           **
 *  **																	   **
 *  All rights reserved                                                    **
 *  **																	   **
 *  This script is part of the Entagged project. The Entagged 			   **
 *  project is free software; you can redistribute it and/or modify        **
 *  it under the terms of the GNU General Public License as published by   **
 *  the Free Software Foundation; either version 2 of the License, or      **
 *  (at your option) any later version.                                    **
 *  **																	   **
 *  The GNU General Public License can be found at                         **
 *  http://www.gnu.org/copyleft/gpl.html.                                  **
 *  **																	   **
 *  This copyright notice MUST APPEAR in all copies of the file!           **
 *  ********************************************************************
 */
package net.sf.ext.entagged.libraries.audioformats;

import java.io.File;
import java.util.Hashtable;
import java.util.Vector;

import net.sf.ext.entagged.libraries.exceptions.CannotReadException;
import net.sf.ext.entagged.libraries.exceptions.WarningException;

public class TaggedFileBuffer {
	
	private static Hashtable ht = new Hashtable(0, 0.5f);
	private static Vector directories = new Vector(5);
	private static TagReader tr = new TagReader();
	
	public static void addDirectory(File dir) throws WarningException{
		TaggedFile[] tfs;
		boolean warning = false;
		String warningMessage = null;
		if(!hasDirectory(dir)) {
			try {
				File[] fl = dir.listFiles(new TagTypeFileFilter());
				tfs = tr.read( fl  );
			} catch (WarningException e) {
				tfs = (TaggedFile[]) e.getObject();
				warning = true;
				warningMessage = e.getMessage();
			}
				
			put(tfs);
			
			if (tfs.length != 0) {
				directories.add(dir);
            }
			
            if (warning) {
				throw new WarningException(warningMessage);
            }
		}
	}
	
	public static TagReader getTagReader() {
		return tr;
	}
	
	public static boolean hasDirectory(File dir) {
		return directories.contains(dir);
	}
		
	private static void put(TaggedFile tf) {
		ht.put(tf, tf);
	}
	
	private static void put(TaggedFile[] tf) {
		for(int i = 0; i<tf.length; i++)
			ht.put(tf[i], tf[i]);
	}
	
	public static TaggedFile get(File f) {
		TaggedFile tf = (TaggedFile) ht.get(f);
				
		if (tf == null) {
			try {
				tf = tr.read(f);
				put(tf);
			} catch(CannotReadException e) {
				return null;
			}
		}
		return tf;
	}
	
	public static void empty() {
		ht.clear();
		directories.clear();
	}
}
