/*
 *  ********************************************************************   **
 *  Copyright notice                                                       **
 *  **																	   **
 *  (c) 2003 Entagged Developpement Team				                   **
 *  http://www.sourceforge.net/projects/entagged                           **
 *  **																	   **
 *  All rights reserved                                                    **
 *  **																	   **
 *  This script is part of the Entagged project. The Entagged 			   **
 *  project is free software; you can redistribute it and/or modify        **
 *  it under the terms of the GNU General Public License as published by   **
 *  the Free Software Foundation; either version 2 of the License, or      **
 *  (at your option) any later version.                                    **
 *  **																	   **
 *  The GNU General Public License can be found at                         **
 *  http://www.gnu.org/copyleft/gpl.html.                                  **
 *  **																	   **
 *  This copyright notice MUST APPEAR in all copies of the file!           **
 *  ********************************************************************
 */
package net.sf.ext.entagged.libraries.utils;

import java.io.File;
import java.text.DecimalFormat;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.JTable;
import javax.swing.table.TableColumn;


/**
 *  This Class is a regrouping of all functions that could not fit in another place or function of general use $Id: Utils.java,v 1.1 2004/09/17 07:15:48 rwaals Exp $
 *
 * @author     Rapha?l Slinckx (KiKiDonK)
 * @version    v0.03
 */
public class Utils {
	/**  Path to the resources files */
	public final static String RESOURCES_PATH = "../resources/";
	/**  Path to audio formats plugins */
	public final static File FORMATS_DIRECTORY = new File( "../plugins/audioformats" );
	/**  Path to Skin LAF Themes */
	public final static File THEMEFOLDER = new File("../resources/themes/");
	/**  The Software name that use the freedb server */
	public final static String softwareName = "Entagged";
	/**  The Software version that use the freedb server */
	public final static String softwareVersion = "v0.06";
	/**  The client e-mail address domain (freedb)*/
	public static final String clientDomain = "sourceforge.net";
	/**  The client e-mail address (freedb)*/
	public static final String clientLogin = "entagged";
	//--------------

	/**  String displayed to indicate that the id3v1 Genre is empty */
	private final static String EMPTY_STRING = "- Empty -";
	
	/**  Array containing the 126 genres defined for the ID3v1.1 plus winamp specials  */
	public final static String[] GENRES = {"Blues", "Classic Rock", "Country", "Dance", "Disco", "Funk", "Grunge", "Hip-Hop", "Jazz",
			"Metal", "New Age", "Oldies", "Other", "Pop", "R&B", "Rap", "Reggae", "Rock", "Techno", "Industrial", "Alternative",
			"Ska", "Death Metal", "Pranks", "Soundtrack", "Euro-Techno", "Ambient", "Trip-Hop", "Vocal", "Jazz+Funk", "Fusion",
			"Trance", "Classical", "Instrumental", "Acid", "House", "Game", "Sound Clip", "Gospel", "Noise", "AlternRock",
			"Bass", "Soul", "Punk", "Space", "Meditative", "Instrumental Pop", "Instrumental Rock", "Ethnic", "Gothic",
			"Darkwave", "Techno-Industrial", "Electronic", "Pop-Folk", "Eurodance", "Dream", "Southern Rock", "Comedy",
			"Cult", "Gangsta", "Top 40", "Christian Rap", "Pop/Funk", "Jungle", "Native American", "Cabaret", "New Wave",
			"Psychadelic", "Rave", "Showtunes", "Trailer", "Lo-Fi", "Tribal", "Acid Punk", "Acid Jazz", "Polka", "Retro",
			"Musical", "Rock & Roll", "Hard Rock", "Folk", "Folk-Rock", "National Folk", "Swing", "Fast Fusion", "Bebob", "Latin", "Revival",
			"Celtic", "Bluegrass", "Avantgarde", "Gothic Rock", "Progressive Rock", "Psychedelic Rock", "Symphonic Rock", "Slow Rock",
			"Big Band", "Chorus", "Easy Listening", "Acoustic", "Humour", "Speech", "Chanson", "Opera", "Chamber Music", "Sonata",
			"Symphony", "Booty Bass", "Primus", "Porn Groove", "Satire", "Slow Jam", "Club", "Tango", "Samba", "Folklore", "Ballad",
			"Power Ballad", "Rhythmic Soul", "Freestyle", "Duet", "Punk Rock", "Drum Solo", "A capella", "Euro-House", "Dance Hall", EMPTY_STRING};
			
	
	/**  String used, when the users select multiples files in the GUI, to indicate that a field contains information that aren't equal  */
	public final static String VARIES = "<var>";

	/**  Resolution of the screen */
	public static double screenWidth, screenHeight;


	/**  The langage resources filenames */
	private final static String[] langageFilenames = {"english.lang", "french.lang"};
	/**  The corresponding langage Names in english */
	private final static String[] langageNames = {"English", "French"};

	public static String getFormattedLength( int length ) {
		if ( length != -1 ) {
			int minutes = length / 60;
			int sec = ( length - minutes * 60 );
			DecimalFormat fmt = new DecimalFormat( "00" );

			return fmt.format( minutes ) + ":" + fmt.format( sec );
		}
		else
			return "timeformat error";
	}

	/**
	 *  converts a Vector containing Object of File type , to an Array of File
	 *
	 * @param  v  a vector containing String objects only
	 * @return    a File[] from the vector v
	 */
	public static File[] toFileArray( Vector v ) {
		File[] listeFichiers = new File[v.size()];

		for ( int i = 0; i < v.size(); i++ )
			listeFichiers[i] = ( (File) v.elementAt( i ) );
		return listeFichiers;
	}


	/**
	 *  converts a Vector containing Object of String type , to an Array of String
	 *
	 * @param  v  a vector containing String objects only
	 * @return    a String[] from the vector v
	 */
	public static String[] toStringArray( Vector v ) {
		String[] stringArray = new String[v.size()];

		for ( int i = 0; i < v.size(); i++ )
			stringArray[i] = ( (String) v.elementAt( i ) );
		return stringArray;
	}


	/**
	 *  Returns the name of the given langage file
	 *
	 * @param  langageFilename  Description of the Parameter
	 * @return                  the name of the langage in English
	 */
	public static String translateLangageFilename( String langageFilename ) {
		Hashtable translateTable = new Hashtable( 10, 1.0f );

		for ( int i = 0; i < langageFilenames.length; i++ )
			translateTable.put( langageFilenames[i], langageNames[i] );
		return (String) translateTable.get( langageFilename );
	}


	/**
	 *  Returns the Filename of the given langage name
	 *
	 * @param  langageName  the langage name
	 * @return              the langage filename
	 */
	public static String translateLangageName( String langageName ) {
		Hashtable translateTable = new Hashtable( 10, 1.0f );

		for ( int i = 0; i < langageFilenames.length; i++ )
			translateTable.put( langageNames[i], langageFilenames[i] );
		return (String) translateTable.get( langageName );
	}


	/**
	 *  Converts the Java signed bytes into more useful short - unsigned bytes
	 *
	 * @param  b  a byte array to be converted
	 * @return    a short array containing the unsigned bytes
	 */
	public static int[] unsignedBytes( byte[] b ) {
		int[] output = new int[b.length];

		for ( int i = 0; i < b.length; i++ )
			output[i] = b[i] & 0xFF;
		return output;
	}


	public static int unsignedBytes( byte b ) {
		return b & 0xFF;
	}
	
	public static String getExtension(File f) {
		String name = f.getName().toLowerCase();
		int i = name.lastIndexOf( "." );

		if(i == -1)
			return "";
		
		return name.substring( i + 1 );
	}
	
	public static String getRawExtension(File f) {
		String name = f.getName();
		int i = name.lastIndexOf( "." );

		if(i == -1)
			return "";
		
		return name.substring( i + 1 );
	}
	
	public static Vector getColumnsInModelOrder(JTable table) {
		Vector result = new Vector();
        Enumeration enum = table.getColumnModel().getColumns();
        for (; enum.hasMoreElements(); )
            result.add(enum.nextElement());
    
        // Sort the columns based on the model index
        Collections.sort(result, new Comparator() {
            public int compare(Object a, Object b) {
                TableColumn c1 = (TableColumn)a;
                TableColumn c2 = (TableColumn)b;
                if (c1.getModelIndex() < c2.getModelIndex())
                    return -1;
				else if (c1.getModelIndex() == c2.getModelIndex())
                    return 0;
				else
                    return 1;
            }
        });
        return result;
    }
	
	public static int readInteger( byte b1, byte b2, byte b3, byte b4 ) {
		int value = 0;
		
		value += b1 << 24;
		value += b2 << 16;
		value += b3 << 8;
		value += b4;

		return value;
	}
	
	public static int readInteger( byte b1, byte b2, byte b3 ) {
		int value = 0;
		
		value += b1 << 16;
		value += b2 << 8;
		value += b3;

		return value;
	}
	
	public static int readInteger( byte b1, byte b2 ) {
		int value = 0;
		
		value += b1 << 8;
		value += b2;

		return value;
	}
}

