/*
 *  ********************************************************************   **
 *  Copyright notice                                                       **
 *  **																	   **
 *  (c) 2003 Entagged Developpement Team				                   **
 *  http://www.sourceforge.net/projects/entagged                           **
 *  **																	   **
 *  All rights reserved                                                    **
 *  **																	   **
 *  This script is part of the Entagged project. The Entagged 			   **
 *  project is free software; you can redistribute it and/or modify        **
 *  it under the terms of the GNU General Public License as published by   **
 *  the Free Software Foundation; either version 2 of the License, or      **
 *  (at your option) any later version.                                    **
 *  **																	   **
 *  The GNU General Public License can be found at                         **
 *  http://www.gnu.org/copyleft/gpl.html.                                  **
 *  **																	   **
 *  This copyright notice MUST APPEAR in all copies of the file!           **
 *  ********************************************************************
 */
package net.sf.ext.entagged.plugins.audioformats.flac;

import java.io.File;

import net.sf.ext.entagged.libraries.audioformats.interfaces.AudioFormatInterface;
import net.sf.ext.entagged.libraries.audioformats.interfaces.EncodingInfoReader;
import net.sf.ext.entagged.libraries.audioformats.interfaces.TagReaderInterface;
import net.sf.ext.entagged.libraries.audioformats.interfaces.TagTypeInterface;
import net.sf.ext.entagged.plugins.audioformats.flac.inforeader.FLACEncodingInfoReader;

public class AudioFormat implements AudioFormatInterface{

	private static AudioFormatInterface flac = new AudioFormat();
	private static File rootFile;
		
	public TagTypeInterface[] getSupportedTagTypes() {
		return new TagTypeInterface[] {FLACTagType.getInstance()};
	}
	
	public TagReaderInterface[] getTagReaders() {
		return new TagReaderInterface[]{FLACTagType.getInstance().getTagReader()};
	}
	
	public TagTypeInterface getTagType(String tagType) {
		if(tagType.equals(FLACTag.TAGTYPE))
			return FLACTagType.getInstance();
		else return null;
	}
	
	public String getName() {
		return "FLAC Tags";
	}
	
	public String[] getSupportedExtensions() {
		return new String[]{"flac"};
	}
	
	public EncodingInfoReader getEncodingInfoReader() {
		return new FLACEncodingInfoReader();
	}

	public TagTypeInterface getDefaultTagType() {
		return FLACTagType.getInstance();
	}
	
	public void setRootFile(File f) {
		rootFile = f;
	}
	
	public File getRootFile() {
		return rootFile;
	}
	
	
	public static AudioFormatInterface getInstance() {
		return flac;
	}
	
	public String toString() {
		return getName();
	}
	
	public String getAuthor() {
		return "Raphael Slinckx (KiKiDonK)";
	}
	
	public String getDescription() {
		String output = "This plugin is used to handle FLAC compressed files\n"+
		"It can:\n\n"+
		"\tRead All possible Blocks, and stores the blocks that can't be displayed\n\tfor a future save of the tag, so original\n\tinformation don't get lost\n\n"+
		"\tWrite New Tags, from scratch or using existing ones, it can also handle\n\tpaddings, for a faster tagging, if one is found.\n\n"+
		"Please not that I haven't tested this very har for the moment, please report any bugs !";
		
		return output;
	}
}
