/*
 *  ********************************************************************   **
 *  Copyright notice                                                       **
 *  **																	   **
 *  (c) 2003 Entagged Developpement Team				                   **
 *  http://www.sourceforge.net/projects/entagged                           **
 *  **																	   **
 *  All rights reserved                                                    **
 *  **																	   **
 *  This script is part of the Entagged project. The Entagged 			   **
 *  project is free software; you can redistribute it and/or modify        **
 *  it under the terms of the GNU General Public License as published by   **
 *  the Free Software Foundation; either version 2 of the License, or      **
 *  (at your option) any later version.                                    **
 *  **																	   **
 *  The GNU General Public License can be found at                         **
 *  http://www.gnu.org/copyleft/gpl.html.                                  **
 *  **																	   **
 *  This copyright notice MUST APPEAR in all copies of the file!           **
 *  ********************************************************************
 */
package net.sf.ext.entagged.plugins.audioformats.flac;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import java.util.Vector;

import net.sf.ext.entagged.libraries.audioformats.interfaces.Tag;
import net.sf.ext.entagged.libraries.audioformats.interfaces.TagWriterInterface;
import net.sf.ext.entagged.libraries.exceptions.CannotWriteException;
import net.sf.ext.entagged.libraries.exceptions.InvalidFormatException;
import net.sf.ext.entagged.plugins.audioformats.flac.inforeader.MetadataBlock;
import net.sf.ext.entagged.plugins.audioformats.flac.inforeader.MetadataBlockData;
import net.sf.ext.entagged.plugins.audioformats.flac.inforeader.MetadataBlockDataApplication;
import net.sf.ext.entagged.plugins.audioformats.flac.inforeader.MetadataBlockDataCueSheet;
import net.sf.ext.entagged.plugins.audioformats.flac.inforeader.MetadataBlockDataPadding;
import net.sf.ext.entagged.plugins.audioformats.flac.inforeader.MetadataBlockDataSeekTable;
import net.sf.ext.entagged.plugins.audioformats.flac.inforeader.MetadataBlockHeader;


public class FLACWriter implements TagWriterInterface {
	
	private Vector metadataBlockPadding, metadataBlockApplication, metadataBlockSeekTable, metadataBlockCueSheet;
	private MappedByteBuffer mbb = null;

	public void delete(File f) {}


	public void write( File f, Tag tag )  throws CannotWriteException{
		if ( f.canWrite() ) {
			metadataBlockPadding = new Vector(1);
			metadataBlockApplication = new Vector(1);
			metadataBlockSeekTable = new Vector(1);
			metadataBlockCueSheet = new Vector(1);
			writeFLAC( f, tag );
			metadataBlockPadding.removeAllElements();
			metadataBlockApplication.removeAllElements();
			metadataBlockSeekTable.removeAllElements();
			metadataBlockCueSheet.removeAllElements();
		}
		else
			throw new CannotWriteException("Error: Could not write in the file -"+f+"- .check that the file is not read-only.");
	}


	private void writeFLAC( File f, Tag tag ) throws CannotWriteException{
		try {
			RandomAccessFile raf = new RandomAccessFile( f, "rw" );

			mbb = raf.getChannel().map( FileChannel.MapMode.READ_WRITE, 0, raf.getChannel().size() );
			raf.close();
		} catch ( IOException e ) {
			throw new CannotWriteException("Error: Could not access the file -"+f+"- :\n"+e);
		}
        
        if (mbb == null) {
            return;
        }

		byte[] b = new byte[4];
		mbb.get(b);
		
		String flac = new String(b);
		if(!flac.equals("fLaC")) {
			throw new CannotWriteException("Error: File -"+f+"- does not appear to be a FLAC file");
        }
		
		boolean isLastBlock = false;
		while(!isLastBlock) {
			b = new byte[4];
			mbb.get(b);
			MetadataBlockHeader mbh = new MetadataBlockHeader(b);
		
			//System.out.println("BlockType: "+mbh.getBlockTypeString());
			switch(mbh.getBlockType()) {
				case MetadataBlockHeader.VORBIS_COMMENT : handlePadding(mbh); break;
				case MetadataBlockHeader.PADDING : handlePadding(mbh); break;
				case MetadataBlockHeader.APPLICATION : handleApplication(mbh); break;
				case MetadataBlockHeader.SEEKTABLE : handleSeekTable(mbh); break;
				case MetadataBlockHeader.CUESHEET : handleCueSheet(mbh); break;
				default : skipBlock(mbh); break;
			}

			isLastBlock = mbh.isLastBlock();
		}
		
		int availableRoom =  computeAvailableRoom();
		int neededRoom = computeNeededRoom(tag);
		//System.out.println("Av.:"+availableRoom+"|Needed:"+neededRoom);
		mbb.rewind();
		
		if(availableRoom>=neededRoom) {
			//OVERWRITE EXISTING TAG
			mbb.position(42);
		
			for(int i = 0; i<metadataBlockApplication.size(); i++) {
				mbb.put( ((MetadataBlock) metadataBlockApplication.elementAt(i)).getHeader().getBytes());
				mbb.put( ((MetadataBlock) metadataBlockApplication.elementAt(i)).getData().getBytes());
			}

			for(int i = 0; i<metadataBlockSeekTable.size(); i++) {
				mbb.put( ((MetadataBlock) metadataBlockSeekTable.elementAt(i)).getHeader().getBytes());
				mbb.put( ((MetadataBlock) metadataBlockSeekTable.elementAt(i)).getData().getBytes());
			}
		
			for(int i = 0; i<metadataBlockCueSheet.size(); i++) {
				mbb.put( ((MetadataBlock) metadataBlockCueSheet.elementAt(i)).getHeader().getBytes());
				mbb.put( ((MetadataBlock) metadataBlockCueSheet.elementAt(i)).getData().getBytes());
			}
		
			try{
				mbb.put(tag.getByteBufferRepresentation( availableRoom-neededRoom ));
				mbb.force();
			}catch(InvalidFormatException e) {
				//e.printStackTrace();
				throw new CannotWriteException(e.getMessage());
			}
		} else {
			try{
				//create new tag with padding (we remove the header and keep the audio data)
				RandomAccessFile raf = new RandomAccessFile( f, "rw" );
				FileChannel fc = raf.getChannel();
				b = new byte[42];
				raf.readFully(b);
				raf.seek(availableRoom+42);
	
				File tempFLAC = File.createTempFile( "entagged", "temp.flac", f.getParentFile() );
				RandomAccessFile tempRAF = new RandomAccessFile( tempFLAC, "rw" );
				FileChannel tempFC = tempRAF.getChannel();
				
				tempRAF.write( b );
				
				for(int i = 0; i<metadataBlockApplication.size(); i++) {
					tempRAF.write( ((MetadataBlock) metadataBlockApplication.elementAt(i)).getHeader().getBytes());
					tempRAF.write( ((MetadataBlock) metadataBlockApplication.elementAt(i)).getData().getBytes());
				}
	
				for(int i = 0; i<metadataBlockSeekTable.size(); i++) {
					tempRAF.write( ((MetadataBlock) metadataBlockSeekTable.elementAt(i)).getHeader().getBytes());
					tempRAF.write( ((MetadataBlock) metadataBlockSeekTable.elementAt(i)).getData().getBytes());
				}
			
				for(int i = 0; i<metadataBlockCueSheet.size(); i++) {
					tempRAF.write( ((MetadataBlock) metadataBlockCueSheet.elementAt(i)).getHeader().getBytes());
					tempRAF.write( ((MetadataBlock) metadataBlockCueSheet.elementAt(i)).getData().getBytes());
				}
				
				tempRAF.write(tag.getByteBufferRepresentation( FLACTag.DEFAULT_PADDING ).array());
				
				tempFC.transferFrom( fc, tempFC.position(), fc.size() );
				
				tempRAF.close();
				raf.close();
	
				f.delete();
				tempFLAC.renameTo( f );
			} catch ( IOException e ) {
				//e.printStackTrace();
				throw new CannotWriteException("Error: Could not access file -"+f+"- . :\n"+e);
			} catch (InvalidFormatException e) {
				//e.printStackTrace();
				throw new CannotWriteException("Error: Cannot create the tag content, no tag written in -"+f+"- . :\n"+e);
			}
		}
	}
	
	private int computeAvailableRoom() {
		int length = 0;
		
		for(int i = 0; i<metadataBlockApplication.size(); i++)
			length += ((MetadataBlock) metadataBlockApplication.elementAt(i)).getLength();
		
		for(int i = 0; i<metadataBlockSeekTable.size(); i++)
			length += ((MetadataBlock) metadataBlockSeekTable.elementAt(i)).getLength();
		
		for(int i = 0; i<metadataBlockCueSheet.size(); i++)
			length += ((MetadataBlock) metadataBlockCueSheet.elementAt(i)).getLength();
		
		for(int i = 0; i<metadataBlockPadding.size(); i++)
			length += ((MetadataBlock) metadataBlockPadding.elementAt(i)).getLength();
		
		return length;
	}
	
	private int computeNeededRoom(Tag tag) {
		int length = 0;
		
		for(int i = 0; i<metadataBlockApplication.size(); i++)
			length += ((MetadataBlock) metadataBlockApplication.elementAt(i)).getLength();
		
		for(int i = 0; i<metadataBlockSeekTable.size(); i++)
			length += ((MetadataBlock) metadataBlockSeekTable.elementAt(i)).getLength();
		
		for(int i = 0; i<metadataBlockCueSheet.size(); i++)
			length += ((MetadataBlock) metadataBlockCueSheet.elementAt(i)).getLength();
		
		length += tag.getTagLength();
		return length;
	}
		
	
	private void skipBlock(MetadataBlockHeader mbh) {
		mbb.position(mbb.position()+mbh.getDataLength());
	}
	
	
	private void handlePadding(MetadataBlockHeader mbh) {
		mbb.position(mbb.position()+mbh.getDataLength());
		
		MetadataBlockData mbd = new MetadataBlockDataPadding( mbh.getDataLength() );
		metadataBlockPadding.add(new MetadataBlock(mbh, mbd));
	}
	
	private void handleApplication(MetadataBlockHeader mbh) {
		byte[] b = new byte[mbh.getDataLength()];
		mbb.get(b);
			
		MetadataBlockData mbd = new MetadataBlockDataApplication(b);
		metadataBlockApplication.add(new MetadataBlock(mbh, mbd));
	}
	
	private void handleSeekTable(MetadataBlockHeader mbh) {
		byte[] b = new byte[mbh.getDataLength()];
		mbb.get(b);
			
		MetadataBlockData mbd = new MetadataBlockDataSeekTable(b);
		metadataBlockSeekTable.add(new MetadataBlock(mbh, mbd));
	}
	
	private void handleCueSheet(MetadataBlockHeader mbh) {
		byte[] b = new byte[mbh.getDataLength()];
		mbb.get(b);
			
		MetadataBlockData mbd = new MetadataBlockDataCueSheet(b);
		metadataBlockCueSheet.add(new MetadataBlock(mbh, mbd));
	}

}

