/*
 *  ********************************************************************   **
 *  Copyright notice                                                       **
 *  **																	   **
 *  (c) 2003 Entagged Developpement Team				                   **
 *  http://www.sourceforge.net/projects/entagged                           **
 *  **																	   **
 *  All rights reserved                                                    **
 *  **																	   **
 *  This script is part of the Entagged project. The Entagged 			   **
 *  project is free software; you can redistribute it and/or modify        **
 *  it under the terms of the GNU General Public License as published by   **
 *  the Free Software Foundation; either version 2 of the License, or      **
 *  (at your option) any later version.                                    **
 *  **																	   **
 *  The GNU General Public License can be found at                         **
 *  http://www.gnu.org/copyleft/gpl.html.                                  **
 *  **																	   **
 *  This copyright notice MUST APPEAR in all copies of the file!           **
 *  ********************************************************************
 */
package net.sf.ext.entagged.plugins.audioformats.flac.inforeader;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;

import net.sf.ext.entagged.libraries.audioformats.EncodingInfo;
import net.sf.ext.entagged.libraries.audioformats.interfaces.EncodingInfoReader;
import net.sf.ext.entagged.libraries.exceptions.CannotReadException;


public class FLACEncodingInfoReader implements EncodingInfoReader {

	private EncodingInfo encodingInfo;

	/**  MappedByte Buffer for this reader (reusable) */
	private MappedByteBuffer mbb;


	public EncodingInfo read( File f ) throws CannotReadException{
		try {
			RandomAccessFile raf = new RandomAccessFile( f, "r" );

			mbb = raf.getChannel().map( FileChannel.MapMode.READ_ONLY, 0, raf.getChannel().size() );
			raf.close();
		} catch ( IOException e ) {
			//e.printStackTrace();
			throw new CannotReadException("Error: Could not access the file -"+f+"- :\n"+e);
		}
		encodingInfo = new EncodingInfo();
		readFileHeader( f );

		mbb = null;
		
		if(!encodingInfo.isValid( ))
			throw new CannotReadException("Error: Could not extract valid basic infos over the file -"+f+"-");

		return encodingInfo;
	}



	private int computeBitrate( int length, long size ) {
		return (int) ( ( size / 1000 ) * 8 / length );
	}


	private void readFileHeader( File f ) {
		if ( f.length()==0 ) {
			//Empty File
			encodingInfo.setValid( false );
			return;
		}
		mbb.position( 0 );

		//FLAC Header string
		byte[] b = new byte[4];
		mbb.get(b);
		String flac = new String(b);
		System.out.println(flac);
		if(!flac.equals("fLaC")) {
			encodingInfo.setValid( false );
			return;
		}
		
		MetadataBlockDataStreamInfo mbdsi = null;
		boolean isLastBlock = false;
		while(!isLastBlock) {
			b = new byte[4];
			mbb.get(b);
			MetadataBlockHeader mbh = new MetadataBlockHeader(b);
		
			if(mbh.getBlockType() == MetadataBlockHeader.STREAMINFO) {
				b = new byte[mbh.getDataLength()];
				mbb.get(b);
			
				//System.out.println("StreamInfo found");
				mbdsi = new MetadataBlockDataStreamInfo(b);
				if(!mbdsi.isValid()) {
					encodingInfo.setValid( false );
					return;
				}
				break;
			}
			mbb.position(mbb.position() + mbh.getDataLength());
			//System.out.println("Skipped block type: " + mbh.getBlockTypeString());
			
			isLastBlock = mbh.isLastBlock();
		}
		
		encodingInfo.setTimelength( mbdsi.getLength() );
		encodingInfo.setChannelMode( mbdsi.getChannelMode() );
		encodingInfo.setSamplingRate( mbdsi.getSamplingRate() );
		encodingInfo.setEncodingType( mbdsi.getEncodingType() );
		encodingInfo.setExtraEncodingInfos( "" );
		encodingInfo.setBitrate( computeBitrate( mbdsi.getLength(), f.length() ) );
		encodingInfo.setValid( true );
	}
}


