/*
 *  ********************************************************************   **
 *  Copyright notice                                                       **
 *  **																	   **
 *  (c) 2003 Entagged Developpement Team				                   **
 *  http://www.sourceforge.net/projects/entagged                           **
 *  **																	   **
 *  All rights reserved                                                    **
 *  **																	   **
 *  This script is part of the Entagged project. The Entagged 			   **
 *  project is free software; you can redistribute it and/or modify        **
 *  it under the terms of the GNU General Public License as published by   **
 *  the Free Software Foundation; either version 2 of the License, or      **
 *  (at your option) any later version.                                    **
 *  **																	   **
 *  The GNU General Public License can be found at                         **
 *  http://www.gnu.org/copyleft/gpl.html.                                  **
 *  **																	   **
 *  This copyright notice MUST APPEAR in all copies of the file!           **
 *  ********************************************************************
 */
package net.sf.ext.entagged.plugins.audioformats.ogg;

import java.io.File;

import net.sf.ext.entagged.libraries.audioformats.interfaces.AudioFormatInterface;
import net.sf.ext.entagged.libraries.audioformats.interfaces.EncodingInfoReader;
import net.sf.ext.entagged.libraries.audioformats.interfaces.TagReaderInterface;
import net.sf.ext.entagged.libraries.audioformats.interfaces.TagTypeInterface;
import net.sf.ext.entagged.plugins.audioformats.ogg.inforeader.OGGEncodingInfoReader;

public class AudioFormat implements AudioFormatInterface{

	private static AudioFormatInterface ogg = new AudioFormat();
	private static File rootFile;
		
	public TagTypeInterface[] getSupportedTagTypes() {
		return new TagTypeInterface[] {OGGTagType.getInstance()};
	}
	
	public TagReaderInterface[] getTagReaders() {
		return new TagReaderInterface[]{OGGTagType.getInstance().getTagReader()};
	}
	
	public TagTypeInterface getTagType(String tagType) {
		if(tagType.equals(OGGTag.TAGTYPE))
			return OGGTagType.getInstance();
		else return null;
	}
	
	public String getName() {
		return "OGG Tags";
	}
	
	public String[] getSupportedExtensions() {
		return new String[]{"ogg"};
	}
	
	public EncodingInfoReader getEncodingInfoReader() {
		return new OGGEncodingInfoReader();
	}

	public TagTypeInterface getDefaultTagType() {
		return OGGTagType.getInstance();
	}
	
	public void setRootFile(File f) {
		rootFile = f;
	}
	
	public File getRootFile() {
		return rootFile;
	}
	
	
	public static AudioFormatInterface getInstance() {
		return ogg;
	}
	
	public String toString() {
		return getName();
	}
	
	public String getAuthor() {
		return "Raphael Slinckx (KiKiDonK)";
	}
	
	public String getDescription() {
		return "This plugin handles the Open Source OGG/vorbis files tags.\n"+
		"These files save the tag in an intricated way, so reading is ok, but\nWriting tags may take some time, as no padding system\nis used with this type of files.\nThe Entire files has to be rewritten...\n\n"+
		"Please report any bugs...";
	}
}
