/*
 ******************************************************************* **
 Copyright notice                                                       **
 **																	   **
 (c) 2003 Entagged Developpement Team				                   **
 http://www.sourceforge.net/projects/entagged                           **
 **																	   **
 All rights reserved                                                    **
 **																	   **
 This script is part of the Entagged project. The Entagged 			   **
 project is free software; you can redistribute it and/or modify        **
 it under the terms of the GNU General Public License as published by   **
 the Free Software Foundation; either version 2 of the License, or      **
 (at your option) any later version.                                    **
 **																	   **
 The GNU General Public License can be found at                         **
 http://www.gnu.org/copyleft/gpl.html.                                  **
 **																	   **
 This copyright notice MUST APPEAR in all copies of the file!           **
 ********************************************************************
 */
package net.sf.ext.entagged.plugins.audioformats.ogg;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.io.UnsupportedEncodingException;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import java.util.Hashtable;

import net.sf.ext.entagged.libraries.audioformats.interfaces.Tag;
import net.sf.ext.entagged.libraries.audioformats.interfaces.TagReaderInterface;
import net.sf.ext.entagged.libraries.exceptions.CannotReadException;
import net.sf.ext.entagged.plugins.audioformats.ogg.inforeader.OGGPageHeader;


/**
 $Id: OGGReader.java,v 1.1 2004/09/17 07:15:34 rwaals Exp $
 *
 * @author     Raphael Slinckx (KiKiDonK)
 * @version    16 d?cembre 2003
 */
public class OGGReader implements TagReaderInterface {

	private MappedByteBuffer mbb;


	public Tag read( File f ) throws CannotReadException{
		try {
			RandomAccessFile raf = new RandomAccessFile( f, "r" );

			mbb = raf.getChannel().map( FileChannel.MapMode.READ_ONLY, 0, raf.getChannel().size() );
			raf.close();
		} catch ( IOException e ) {
			//e.printStackTrace();
			throw new CannotReadException("Error: Could not access the file -"+f+"- :\n"+e);
		}

		OGGTag tag = null;
		
		if ( isOGGFile( f ) )
			tag = readOGG( f );

		return tag;
	}


	public String toString() {
		return "OGG Tag Reader";
	}

	private boolean isOGGFile(File f) {
		mbb.position( 0 );
		byte[] b = new byte[4];
		mbb.get(b);
		
		String ogg = new String(b);
		return ogg.equals("OggS");
	}
	
	private OGGTag readOGG( File f ) throws CannotReadException {
		mbb.position( 0 );
		//System.out.println("BEGIN OGG TAG READING");

		//Supposing 1st page = codec infos
		//			2nd page = comment+decode info
		//...Extracting 2nd page
		
		//1st page to get the length
		byte[] b = new byte[4];
		int pageSegments = u( mbb.get( 26 ) );

		b = new byte[27 + pageSegments];
		mbb.get( b );

		OGGPageHeader pageHeader = new OGGPageHeader( b );

		mbb.position( mbb.position() + pageHeader.getPageLength() );

		//2nd page extraction
		pageSegments = u( mbb.get( mbb.position() + 26 ) );
		b = new byte[27 + pageSegments];
		mbb.get( b );
		pageHeader = new OGGPageHeader( b );

		b = new byte[7];
		mbb.get( b );

		int packetType = b[0];
		String vorbisString = new String( b, 1, 6 );

		b = new byte[4];
		mbb.get( b );

		int vendorStringLength = u( b[0] ) + ( u( b[1] ) << 8 ) + ( u( b[2] ) << 16 ) + ( u( b[3] ) << 24 );

		b = new byte[vendorStringLength];
		mbb.get( b );

		String vendorString = new String( b );

		b = new byte[4];
		mbb.get( b );

		int userComments = u( b[0] ) + ( u( b[1] ) << 8 ) + ( u( b[2] ) << 16 ) + ( u( b[3] ) << 24 );

		Hashtable ht = new Hashtable(10);
		//String[] fields = new String[userComments];
		//String[] contents = new String[userComments];

		for ( int i = 0; i < userComments; i++ ) {
			b = new byte[4];
			mbb.get( b );

			int commentLength = u( b[0] ) + ( u( b[1] ) << 8 ) + ( u( b[2] ) << 16 ) + ( u( b[3] ) << 24 );

			b = new byte[commentLength];
			mbb.get( b );

			// Attempt to convert the comment bytes to UTF-8 - if that fails,
			// just use the platforms default charset.
			String comment = null;

			try {
				comment = new String( b, "UTF-8" );
			} catch ( UnsupportedEncodingException e ) {
				comment = new String( b );
			}

			String[] splitComment = comment.split( "=" );
			if(splitComment.length>1)
				ht.put(splitComment[0],splitComment[1]);
		}

		byte isValid = mbb.get();

		if ( isValid != 0 ) {
			//return new OGGTag( fields, contents, vendorString );
			OGGTag tag = new OGGTag();
		
			tag.setTitle(  (ht.get("TITLE")==null) ? "" : (String)ht.get("TITLE") );
			tag.setArtist((ht.get("ARTIST")==null) ? "" : (String)ht.get("ARTIST"));
			tag.setAlbum((ht.get("ALBUM")==null) ? "" : (String)ht.get("ALBUM"));
			tag.setYear((ht.get("DATE")==null) ? "" : (String)ht.get("DATE"));
			
			tag.setComment(
				(ht.get("COMMENT")==null) ? 
					(
					(ht.get("DESCRIPTION")==null) ? "" : (String)ht.get("DESCRIPTION")
					)
					: (String)ht.get("COMMENT"));
			
			tag.setTrackNumber(
				(ht.get("TRACK")==null) ? 
					(
					(ht.get("TRACKNUMBER")==null) ? "" : (String)ht.get("TRACKNUMBER")
					)
					: (String)ht.get("TRACK"));
			
			tag.setGenre((ht.get("GENRE")==null) ? "" : (String)ht.get("GENRE"));
			
			tag.setVendor(vendorString);
		
			mbb.rewind();
			//System.out.println(tag);
			return tag;
		}
		else
			throw new CannotReadException("Error: The OGG Stream isn't valid, could not extract the tag");
	}
	
	private int u(int i) {
		return i & 0xFF;
	}
}

