/*
 *  ********************************************************************   **
 *  Copyright notice                                                       **
 *  **																	   **
 *  (c) 2003 Entagged Developpement Team				                   **
 *  http://www.sourceforge.net/projects/entagged                           **
 *  **																	   **
 *  All rights reserved                                                    **
 *  **																	   **
 *  This script is part of the Entagged project. The Entagged 			   **
 *  project is free software; you can redistribute it and/or modify        **
 *  it under the terms of the GNU General Public License as published by   **
 *  the Free Software Foundation; either version 2 of the License, or      **
 *  (at your option) any later version.                                    **
 *  **																	   **
 *  The GNU General Public License can be found at                         **
 *  http://www.gnu.org/copyleft/gpl.html.                                  **
 *  **																	   **
 *  This copyright notice MUST APPEAR in all copies of the file!           **
 *  ********************************************************************
 */
package net.sf.ext.entagged.plugins.audioformats.ogg;

import java.nio.ByteBuffer;
import java.util.Enumeration;
import java.util.Hashtable;

import net.sf.ext.entagged.libraries.audioformats.interfaces.Tag;


/**
 *  $Id: OGGTag.java,v 1.1 2004/09/17 07:15:34 rwaals Exp $
 *
 * @author     Raphael Slinckx (KiKiDonK)
 * @version    16 d?cembre 2003
 */
public class OGGTag implements Tag {
	private Hashtable ht;

	private String vendorString;
	
	public final static String TAGTYPE = "ogg";


	/*
	public OGGTag( String[] fields, String[] contents, String vendorString ) {
		ht = new Hashht();

		this.vendorString = vendorString;

		for ( int i = 0; i < fields.length; i++ )
			ht.put( fields[i].toUpperCase(), contents[i] );
		//System.out.println("Field: "+fields[i]+" | " + contents[i]);

		this.content = contents;
		this.field = fields;
	}
	*/


	public OGGTag() {
		ht = new Hashtable();
		this.vendorString = "Entagged - The Musical Box";
	}


	public String getAlbum() {
		return (String) ht.get( "ALBUM" );
	}


	public String getArtist() {
		return (String) ht.get( "ARTIST" );
	}


	public ByteBuffer getByteBufferRepresentation( int paddingSize ) {
		ByteBuffer buf = ByteBuffer.allocate( getTagLength() );
		
		//[packet type=comment0x03]['vorbis']
		buf.put( new byte[]{(byte) 0x03, (byte) 0x76, (byte) 0x6f, (byte) 0x72, (byte) 0x62, (byte) 0x69, (byte) 0x73} );
		
		int vendorLength = getVendor().length();
		buf.put( new byte[]{(byte)(vendorLength&0xFF), (byte)((vendorLength & 0xFF00) >>> 8) , (byte)((vendorLength & 0xFF0000) >>> 16), (byte)((vendorLength & 0xFF000000) >>> 24)  } );
		buf.put( getVendor().getBytes() );

		//[user comment list length]
		int listLength = getFieldNumber();
		byte[] b = new byte[4];
		b[3] = (byte) ( ( listLength & 0xFF000000 ) >>> 24 );
		b[2] = (byte) ( ( listLength & 0x00FF0000 ) >>> 16 );
		b[1] = (byte) ( ( listLength & 0x0000FF00 ) >>> 8 );
		b[0] = (byte) ( listLength & 0x000000FF );
		buf.put( b );

		Enumeration fields = ht.keys();
		while (fields.hasMoreElements()) {
			String field = (String)fields.nextElement();
			String content = (String)ht.get(field);
			
			if(!content.trim().equals("")) {
				//[field length]
				int length = field.length() + 1 + content.length();

				b = new byte[4];
				b[3] = (byte) ( ( length & 0xFF000000 ) >>> 24 );
				b[2] = (byte) ( ( length & 0x00FF0000 ) >>> 16 );
				b[1] = (byte) ( ( length & 0x0000FF00 ) >>> 8 );
				b[0] = (byte) ( length & 0x000000FF );
				buf.put( b );

				//[Comment: field=content ]
				buf.put( field.getBytes() );
				buf.put( (byte) 0x3d ); // "="
				buf.put( content.getBytes() );
			}
		}

		//Framing bit = 1
		buf.put( (byte) 0x01 );

		buf.rewind();
		
		return buf;
	}
	
	private int getFieldNumber() {
		int count = 0;
		Enumeration fields = ht.keys();
		while (fields.hasMoreElements()) {
			String field = (String)fields.nextElement();
			String content = (String)ht.get(field);
			if(!content.trim().equals(""))
				count++;
		}
		return count;
	}

	public String getComment() {
		return (String) ht.get( "COMMENT" );
	}


	public String getGenre() {
		return (String) ht.get( "GENRE" );
	}


	public int getTagLength() {
		int length = 0;

		length += 7 + 4 + 26 + 4 + 1;
		
		Enumeration fields = ht.keys();
		while (fields.hasMoreElements()) {
			String currentField = (String)fields.nextElement();
			String currentContent = (String)ht.get(currentField);
			if(!currentContent.trim().equals("")) {
				length += 4;
				length += currentField.length();
				length += currentContent.length() + 1;
			}
		}
		return length;
	}


	public String getTagType() {
		return TAGTYPE;
	}


	public String getTitle() {
		return (String) ht.get( "TITLE" );
	}


	public String getTrackNumber() {
		return (String) ht.get( "TRACKNUMBER" );
	}


	public String getYear() {
		return (String) ht.get( "DATE" );
	}
	
	public String getVendor() {
		return vendorString;
	}
	
	public boolean isEmpty() {
		Enumeration e = ht.keys();
		while (e.hasMoreElements()) {
			String key = (String) e.nextElement();
			String content = (String) ht.get(key);
			if(!content.trim().equals(""))
				return false;
		}
		return true;
	}
	

	public void setTitle(String title) {
		ht.put("TITLE",title);
	}


	public void setAlbum(String album) {
		ht.put("ALBUM",album);
	}


	public void setArtist(String artist) {
		ht.put("ARTIST",artist);
	}


	public void setGenre(String genre) {
		ht.put("GENRE",genre);
	}
	
	public void setTrackNumber(String trackNumber) {
		ht.put("TRACKNUMBER",trackNumber);
	}

	public void setYear(String year) {
		ht.put("DATE",year);
	}


	public void setComment(String comment) {
		ht.put("COMMENT",comment);
	}
	
	public void setVendor(String vendor) {
		this.vendorString = vendor;
	}


	/**
	 * @return    the string representation of this tag
	 */
	public String toString() {
		String output = "OGGTag-(l:" + getTagLength() + ")-----------------------------------\n";

		output += "Artiste: " + getArtist() + "\tAlbum: " + getAlbum() + "\n";
		output += "Track n?" + getTrackNumber() + " " + getTitle() + "\n";
		output += "Ann?e: " + getYear() + "\tGenre: " + getGenre() + "\n";
		output += "Comments: -" + getComment() + "-\n";
		output += "-----------------------------------------------------\n";
		return output;
	}
}

