/*
 *  ********************************************************************   **
 *  Copyright notice                                                       **
 *  **																	   **
 *  (c) 2003 Entagged Developpement Team				                   **
 *  http://www.sourceforge.net/projects/entagged                           **
 *  **																	   **
 *  All rights reserved                                                    **
 *  **																	   **
 *  This script is part of the Entagged project. The Entagged 			   **
 *  project is free software; you can redistribute it and/or modify        **
 *  it under the terms of the GNU General Public License as published by   **
 *  the Free Software Foundation; either version 2 of the License, or      **
 *  (at your option) any later version.                                    **
 *  **																	   **
 *  The GNU General Public License can be found at                         **
 *  http://www.gnu.org/copyleft/gpl.html.                                  **
 *  **																	   **
 *  This copyright notice MUST APPEAR in all copies of the file!           **
 *  ********************************************************************
 */
package net.sf.ext.entagged.plugins.audioformats.ogg;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;

import net.sf.ext.entagged.libraries.audioformats.interfaces.Tag;
import net.sf.ext.entagged.libraries.audioformats.interfaces.TagWriterInterface;
import net.sf.ext.entagged.libraries.exceptions.CannotWriteException;
import net.sf.ext.entagged.libraries.exceptions.InvalidFormatException;
import net.sf.ext.entagged.plugins.audioformats.ogg.inforeader.OGGCRCFactory;
import net.sf.ext.entagged.plugins.audioformats.ogg.inforeader.OGGPageHeader;


/**
 *  $Id: OGGWriter.java,v 1.1 2004/09/17 07:15:34 rwaals Exp $
 *
 * @author     Raphael Slinckx (KiKiDonK)
 * @version    16 d?cembre 2003
 */
public class OGGWriter implements TagWriterInterface {

	private FileChannel fc;

	public void delete( File oggFile ) throws CannotWriteException{}


	public void write( File mp3File, Tag tag ) throws CannotWriteException{
		if ( mp3File.canWrite() )
			writeOGG( mp3File, tag );
		else
			throw new CannotWriteException("Error: Could not write in the file -"+mp3File+"- .check that the file is not read-only.");
	}


	private void writeOGG( File f, Tag tag ) throws CannotWriteException{
		//System.out.println( "Will write tag::::::\n"+tag );
		try {
			
			File tempOGG = File.createTempFile( "entagged", "temp.ogg", f.getParentFile() );
			RandomAccessFile rafTemp = new RandomAccessFile( tempOGG, "rw" );
			FileChannel fcTemp = rafTemp.getChannel();

			RandomAccessFile raf = new RandomAccessFile( f, "r" );
			FileChannel fc = raf.getChannel();
			MappedByteBuffer mbb = raf.getChannel().map( FileChannel.MapMode.READ_ONLY, 0, raf.getChannel().size() );
							
			
			//Read firstPage----------------------------------------------------
			mbb.position( 0 );
			byte[] b = new byte[4];
			int pageSegments = u( mbb.get( 26 ) );

			b = new byte[27 + pageSegments];
			mbb.get( b );

			OGGPageHeader firstPage = new OGGPageHeader( b );
			//System.out.println(firstPage.isValid());
			//------------------------------------------------------------------
			
			//write 1st page (unchanged)----------------------------------------
			rafTemp.getChannel().transferFrom( raf.getChannel(), 0, firstPage.getPageLength()+mbb.position() );
			rafTemp.skipBytes( firstPage.getPageLength() + mbb.position() );
			//------------------------------------------------------------------
			
			
			//Read 2nd page-----------------------------------------------------
			mbb.position( mbb.position() + firstPage.getPageLength() );
			
			pageSegments = u( mbb.get( mbb.position() + 26 ) );
			b = new byte[27 + pageSegments];
			mbb.get( b );
			OGGPageHeader secondPage = new OGGPageHeader( b );
			//System.out.println(secondPage.isValid());
			raf.skipBytes(27 + pageSegments);
			//------------------------------------------------------------------
			
			
			//Compute old comment length----------------------------------------
			int oldCommentLength = 7; // [.vorbis]
			mbb.position(mbb.position()+7);

			b = new byte[4];
			mbb.get( b );
			int vendorStringLength = u( b[0] ) + ( u( b[1] ) << 8 ) + ( u( b[2] ) << 16 ) + ( u( b[3] ) << 24 );
			oldCommentLength += 4+vendorStringLength;

			mbb.position(mbb.position()+vendorStringLength);

			b = new byte[4];
			mbb.get( b );
			int userComments = u( b[0] ) + ( u( b[1] ) << 8 ) + ( u( b[2] ) << 16 ) + ( u( b[3] ) << 24 );
			oldCommentLength += 4;

			for ( int i = 0; i < userComments; i++ ) {
				b = new byte[4];
				mbb.get( b );
				int commentLength = u( b[0] ) + ( u( b[1] ) << 8 ) + ( u( b[2] ) << 16 ) + ( u( b[3] ) << 24 );
				oldCommentLength += 4+commentLength;
				
				mbb.position(mbb.position()+commentLength);
			}

			byte isValid = mbb.get();
			oldCommentLength += 1;
			if(isValid !=1 )
				throw new CannotWriteException("Unable to retreive old tag informations");
			//System.out.println("oldCommentLength: "+oldCommentLength);
			//------------------------------------------------------------------
			
			
			//Get the new comment and create the container bytebuffer for 2nd page---
			ByteBuffer newComment = null;
			try{
				newComment = tag.getByteBufferRepresentation(0);
			}catch(InvalidFormatException e) {
				//e.printStackTrace();
				throw new CannotWriteException(e.getMessage());
			}
			
			int newCommentLength = newComment.capacity();
			int newSecondPageLength = secondPage.getPageLength() - oldCommentLength + newCommentLength;
			
			byte pageSegmentsNb = (byte) (newSecondPageLength / 255 + 1);
			int newSecondPageHeaderLength = 27+pageSegmentsNb;
			
			ByteBuffer secondPageBuffer = ByteBuffer.allocate(newSecondPageLength+newSecondPageHeaderLength);
			//System.out.println("Page size: "+secondPage.getPageLength());
			//System.out.println("Old comment: "+oldCommentLength);
			//System.out.println("New comment: "+newCommentLength);
			//System.out.println("New page length: "+(secondPage.getPageLength()-oldCommentLength+newCommentLength));
			//------------------------------------------------------------------
			
			
			//Build the new second page header----------------------------------
			//OggS capture
			b = new String("OggS").getBytes();
			secondPageBuffer.put(b);
			//Stream struct revision
			secondPageBuffer.put((byte)0);
			//header_type_flag
			secondPageBuffer.put((byte)0);
			//absolute granule position
			secondPageBuffer.put(new byte[]{(byte)0,(byte)0,(byte)0,(byte)0,(byte)0,(byte)0,(byte)0,(byte)0});
			
			//stream serial number
			int serialNb = secondPage.getSerialNumber();
			//System.out.println(serialNb);
			b = new byte[4];
			b[3] = (byte) ( ( serialNb & 0xFF000000 ) >>> 24 );
			b[2] = (byte) ( ( serialNb & 0x00FF0000 ) >>> 16 );
			b[1] = (byte) ( ( serialNb & 0x0000FF00 ) >>> 8 );
			b[0] = (byte) ( serialNb & 0x000000FF );
			secondPageBuffer.put(b);
			
			
			//page sequence no
			int seqNb = secondPage.getPageSequence();
			//System.out.println(seqNb);
			b = new byte[4];
			b[3] = (byte) ( ( seqNb & 0xFF000000 ) >>> 24 );
			b[2] = (byte) ( ( seqNb & 0x00FF0000 ) >>> 16 );
			b[1] = (byte) ( ( seqNb & 0x0000FF00 ) >>> 8 );
			b[0] = (byte) ( seqNb & 0x000000FF );
			secondPageBuffer.put(b);
			
			//CRC (to be computed later)
			secondPageBuffer.put(new byte[]{(byte) 0,( byte) 0,( byte) 0, (byte) 0});
			int crcOffset = 22;
			
			//page_segments nb.
			secondPageBuffer.put(pageSegmentsNb);

			//page segment table
			secondPageBuffer.put((byte) (newSecondPageLength % 255));
			for(int i = 0; i<pageSegmentsNb-1; i++)
				secondPageBuffer.put((byte)0xFF);
			//------------------------------------------------------------------
			
			
			//Add to the new second page the new comment------------------------
			secondPageBuffer.put( newComment );
			//------------------------------------------------------------------
			

			//Add the remaining old second page (encoding infos, etc)-----------
			raf.skipBytes( oldCommentLength );
			raf.getChannel().read(secondPageBuffer);
			//------------------------------------------------------------------
			
			
			//Compute CRC over the new second page------------------------------
			byte[] crc = OGGCRCFactory.computeCRC(secondPageBuffer.array());
			for(int i = 0; i<crc.length; i++) {
				//System.out.println("Offset: "+(crcOffset+i) +"|"+crc[i]);
				secondPageBuffer.put(crcOffset+i,crc[i]);
				//System.out.println(secondPageBuffer.get(crcOffset+i));
			}
			//------------------------------------------------------------------
			
			
			//Transfer the second page bytebuffer content-----------------------
			secondPageBuffer.rewind();
			rafTemp.getChannel().write(secondPageBuffer);
			//rafTemp.skipBytes( newSecondPageLength+newSecondPageHeaderLength );
			//------------------------------------------------------------------
			
			
			//Write the rest of the original file-------------------------------
			rafTemp.getChannel().transferFrom(raf.getChannel(), rafTemp.getFilePointer(), raf.length()-raf.getFilePointer());
			//------------------------------------------------------------------
			
			raf.close();
			rafTemp.close();
			
			f.delete();
			tempOGG.renameTo( f );
		} catch ( IOException e ) {
			//e.printStackTrace();
			throw new CannotWriteException(e.getMessage());
		}
	}
	
	private int u(int i) {
		return i & 0xFF;
	}
}

