/*
 *  ********************************************************************   **
 *  Copyright notice                                                       **
 *  **																	   **
 *  (c) 2003 Entagged Developpement Team				                   **
 *  http://www.sourceforge.net/projects/entagged                           **
 *  **																	   **
 *  All rights reserved                                                    **
 *  **																	   **
 *  This script is part of the Entagged project. The Entagged 			   **
 *  project is free software; you can redistribute it and/or modify        **
 *  it under the terms of the GNU General Public License as published by   **
 *  the Free Software Foundation; either version 2 of the License, or      **
 *  (at your option) any later version.                                    **
 *  **																	   **
 *  The GNU General Public License can be found at                         **
 *  http://www.gnu.org/copyleft/gpl.html.                                  **
 *  **																	   **
 *  This copyright notice MUST APPEAR in all copies of the file!           **
 *  ********************************************************************
 */
package net.sf.ext.entagged.plugins.audioformats.ogg.inforeader;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;

import net.sf.ext.entagged.libraries.audioformats.EncodingInfo;
import net.sf.ext.entagged.libraries.audioformats.interfaces.EncodingInfoReader;
import net.sf.ext.entagged.libraries.exceptions.CannotReadException;


/**
 *  $Id: OGGEncodingInfoReader.java,v 1.1 2004/09/17 07:15:36 rwaals Exp $
 *
 * @author     Raphael Slinckx (KiKiDonK)
 * @version    16 d?cembre 2003
 */
public class OGGEncodingInfoReader implements EncodingInfoReader {
	private EncodingInfo encodingInfo;

	private MappedByteBuffer mbb;


	public EncodingInfo read( File f )  throws CannotReadException{
		try {
			RandomAccessFile raf = new RandomAccessFile( f, "r" );

			mbb = raf.getChannel().map( FileChannel.MapMode.READ_ONLY, 0, raf.getChannel().size() );
			raf.close();
		} catch ( IOException e ) {
			//e.printStackTrace();
			throw new CannotReadException("Error: Could not access the file -"+f+"- :\n"+e);
		}

		encodingInfo = new EncodingInfo();
		readFileHeader( f );

		mbb = null;
		
		if(!encodingInfo.isValid( ))
			throw new CannotReadException("Error: Could not extract valid basic infos over the file -"+f+"- :\n");

		return encodingInfo;
	}


	public String toString() {
		return "Ogg encoding info reader";
	}


	private int computeBitrate( int length, long size ) {
		return (int) ( ( size / 1000 ) * 8 / length );
	}


	private double computeSampleNumber() throws CannotReadException{
		mbb.position(mbb.capacity()-2);
		while(mbb.position() >= 4) {
			if(mbb.get()==0x53) {
				mbb.position(mbb.position() - 4);
				byte[] ogg = new byte[3];
				mbb.get(ogg);
				if(ogg[0]==0x4F && ogg[1]==0x67 && ogg[2]==0x67) {
					mbb.position(mbb.position() - 3);
					
					int pageSegments = u( mbb.get( mbb.position() + 26 ) );
					
					byte[] b = new byte[27 + pageSegments];
					mbb.get( b );

					OGGPageHeader pageHeader = new OGGPageHeader( b );
					mbb.rewind();
					return pageHeader.getAbsoluteGranulePosition();
				}
			}	
			mbb.position(mbb.position() - 2);
		}
		throw new CannotReadException("Error: Could not find the Ogg Setup block");
	}


	private void readFileHeader( File mp3File ) throws CannotReadException{
		mbb.position( 0 );

		double PCMSamplesNumber = computeSampleNumber();

		//Supposing 1st page = codec infos
		//			2nd page = comment+decode info
		//...Extracting 1st page
		byte[] b = new byte[4];
		int pageSegments = u( mbb.get( 26 ) );

		b = new byte[27 + pageSegments];
		mbb.get( b );

		OGGPageHeader pageHeader = new OGGPageHeader( b );

		byte[] vorbisData = new byte[pageHeader.getPageLength()];

		mbb.get( vorbisData );

		VorbisCodecHeader vorbisCodecHeader = new VorbisCodecHeader( vorbisData );

		//Populates encodingInfo----------------------------------------------------
		encodingInfo.setTimelength( (int) ( PCMSamplesNumber / vorbisCodecHeader.getSamplingRate() ) );
		encodingInfo.setChannelMode( vorbisCodecHeader.getChannelMode() );
		encodingInfo.setSamplingRate( vorbisCodecHeader.getSamplingRate() );
		encodingInfo.setEncodingType( vorbisCodecHeader.getEncodingType() );
		encodingInfo.setExtraEncodingInfos( "" );
		encodingInfo.setBitrate( computeBitrate( encodingInfo.getTimeLength(), mbb.capacity() ) );
		encodingInfo.setValid( true );

		mbb.rewind();
	}
	
	private int u(int i) {
		return i & 0xFF;
	}
}

