/*
 *  ********************************************************************   **
 *  Copyright notice                                                       **
 *  **																	   **
 *  (c) 2003 Entagged Developpement Team				                   **
 *  http://www.sourceforge.net/projects/entagged                           **
 *  **																	   **
 *  All rights reserved                                                    **
 *  **																	   **
 *  This script is part of the Entagged project. The Entagged 			   **
 *  project is free software; you can redistribute it and/or modify        **
 *  it under the terms of the GNU General Public License as published by   **
 *  the Free Software Foundation; either version 2 of the License, or      **
 *  (at your option) any later version.                                    **
 *  **																	   **
 *  The GNU General Public License can be found at                         **
 *  http://www.gnu.org/copyleft/gpl.html.                                  **
 *  **																	   **
 *  This copyright notice MUST APPEAR in all copies of the file!           **
 *  ********************************************************************
 */
package net.sf.ext.entagged.plugins.audioformats.ogg.inforeader;



/**
 *  $Id: OGGPageHeader.java,v 1.1 2004/09/17 07:15:36 rwaals Exp $
 *
 * @author     Raphael Slinckx (KiKiDonK)
 * @version    16 d�cembre 2003
 */
public class OGGPageHeader {
	private double absoluteGranulePosition;
	private byte[] checksum;
	private byte headerTypeFlag;

	private boolean isValid = false;
	private int pageLength = 0;
	private int pageSequenceNumber,streamSerialNumber;

	public OGGPageHeader( byte[] b ) {
		//System.out.println(new String(b, 0 , 4));
		int streamStructureRevision = b[4];
		//System.out.println("streamStructureRevision: " + streamStructureRevision);
		headerTypeFlag = b[5];
		//System.out.println("headerTypeFlag: " + headerTypeFlag);
		if ( streamStructureRevision == 0 ) {
			this.absoluteGranulePosition = 0;  //b[6] + (b[7]<<8) + (b[8]<<16) + (b[9]<<24) + (b[10]<<32) + (b[11]<<40) + (b[12]<<48) + (b[13]<<56);
			for ( int i = 0; i < 8; i++ )
				this.absoluteGranulePosition += u( b[i + 6] ) * Math.pow( 2, 8 * i );

			streamSerialNumber = u(b[14]) + ( u(b[15]) << 8 ) + ( u(b[16]) << 16 ) + ( u(b[17]) << 24 );
			//System.out.println("streamSerialNumber: " + streamSerialNumber);
			pageSequenceNumber = u(b[18]) + (u(b[19]) << 8 ) + ( u(b[20]) << 16 ) + ( u(b[21]) << 24 );
			//System.out.println("pageSequenceNumber: " + pageSequenceNumber);
			checksum = new byte[]{b[22], b[23], b[24], b[25]};


			int pageSegments = u( b[26] );
			//System.out.println("pageSegments: " + pageSegments);

			byte[] segmentTable = new byte[b.length - 27];
			//System.out.println("pagesegment length; "+ (b.length-27));
			for ( int i = 0; i < segmentTable.length; i++ ) {
				segmentTable[i] = b[27 + i];
				this.pageLength += u( b[27 + i] );
				//System.out.println("acc page length: "+this.pageLength);
				//System.out.println(segmentTable[i]);
			}

			isValid = true;
		}
	}
	
	private int u(int i) {
		return i & 0xFF;
	}


	public double getAbsoluteGranulePosition() {
		//System.out.println("Number Of Samples: "+absoluteGranulePosition);
		return this.absoluteGranulePosition;
	}


	public byte[] getCheckSum() {
		return checksum;
	}


	public byte getHeaderType() {
		return headerTypeFlag;
	}


	public int getPageLength() {
		//System.out.println("This page length: "+pageLength);
		return this.pageLength;
	}
	
	public int getPageSequence() {
		return pageSequenceNumber;
	}
	
	public int getSerialNumber() {
		return streamSerialNumber;
	}


	public boolean isValid() {
		return isValid;
	}

	public String toString() {
		String out = "Ogg Page Header:\n";

		out += "Is valid?: " + isValid + " | page length: " + pageLength + "\n";
		out += "Header type: " + headerTypeFlag;
		return out;
	}
}

