/*
 *  ********************************************************************   **
 *  Copyright notice                                                       **
 *  **																	   **
 *  (c) 2003 Entagged Developpement Team				                   **
 *  http://www.sourceforge.net/projects/entagged                           **
 *  **																	   **
 *  All rights reserved                                                    **
 *  **																	   **
 *  This script is part of the Entagged project. The Entagged 			   **
 *  project is free software; you can redistribute it and/or modify        **
 *  it under the terms of the GNU General Public License as published by   **
 *  the Free Software Foundation; either version 2 of the License, or      **
 *  (at your option) any later version.                                    **
 *  **																	   **
 *  The GNU General Public License can be found at                         **
 *  http://www.gnu.org/copyleft/gpl.html.                                  **
 *  **																	   **
 *  This copyright notice MUST APPEAR in all copies of the file!           **
 *  ********************************************************************
 */
package net.sf.ext.entagged.plugins.audioformats.ogg.inforeader;


/**
 *  $Id: VorbisCodecHeader.java,v 1.1 2004/09/17 07:15:36 rwaals Exp $
 *
 * @author     Raphael Slinckx (KiKiDonK)
 * @version    16 d�cembre 2003
 */
public class VorbisCodecHeader {
	private int audioChannels;
	private boolean isValid = false;

	private byte[] vorbisData;
	private int vorbisVersion, audioSampleRate;


	public VorbisCodecHeader( byte[] vorbisData ) {
		generateCodecHeader( vorbisData );
	}


	public String getChannelMode() {
		switch ( audioChannels ) {
					case 0:
						return "No Audio Channels";
					case 1:
						return "Mono";
					case 2:
						return "Stereo";
					default:
						return "More than stereo";
		}
	}


	public String getEncodingType() {
		return "Ogg Vorbis Version " + vorbisVersion;
	}


	public int getSamplingRate() {
		return audioSampleRate;
	}


	public boolean isValid() {
		return isValid;
	}


	public void generateCodecHeader( byte[] b ) {
		int packetType = b[0];
		//System.out.println("packetType" + packetType);
		String vorbis = new String( b, 1, 6 );
		//System.out.println("vorbis" + vorbis);
		if ( packetType == 1 && vorbis.equals( "vorbis" ) ) {
			this.vorbisVersion = b[7] + ( b[8] << 8 ) + ( b[9] << 16 ) + ( b[10] << 24 );
			//System.out.println("vorbisVersion" +vorbisVersion );
			this.audioChannels = u( b[11] );
			//System.out.println("audioChannels" +audioChannels );
			this.audioSampleRate = u( b[12] ) + ( u( b[13] ) << 8 ) + ( u( b[14] ) << 16 ) + ( u( b[15] ) << 24 );
			//System.out.println("audioSampleRate" + audioSampleRate);
			//System.out.println("audioSampleRate" + b[12] + " "+b[13] +" "+ b[14]);
			int bitrateMinimal = b[16] + ( b[17] << 8 ) + ( b[18] << 16 ) + ( b[19] << 24 );
			int bitrateNominal = b[20] + ( b[21] << 8 ) + ( b[22] << 16 ) + ( b[23] << 24 );
			int bitrateMaximal = b[24] + ( b[25] << 8 ) + ( b[26] << 16 ) + ( b[27] << 24 );
			byte blockSize0 = (byte) ( b[28] & 240 );
			byte blockSize1 = (byte) ( b[28] & 15 );
			int framingFlag = b[29];
			//System.out.println("framingFlag" +framingFlag );
			if ( framingFlag != 0 )
				isValid = true;

		}
	}
	
	private int u(int i) {
		return i & 0xFF;
	}
}

