/*
 *  ********************************************************************   **
 *  Copyright notice                                                       **
 *  **																	   **
 *  (c) 2003 Entagged Developpement Team				                   **
 *  http://www.sourceforge.net/projects/entagged                           **
 *  **																	   **
 *  All rights reserved                                                    **
 *  **																	   **
 *  This script is part of the Entagged project. The Entagged 			   **
 *  project is free software; you can redistribute it and/or modify        **
 *  it under the terms of the GNU General Public License as published by   **
 *  the Free Software Foundation; either version 2 of the License, or      **
 *  (at your option) any later version.                                    **
 *  **																	   **
 *  The GNU General Public License can be found at                         **
 *  http://www.gnu.org/copyleft/gpl.html.                                  **
 *  **																	   **
 *  This copyright notice MUST APPEAR in all copies of the file!           **
 *  ********************************************************************
 */
package net.sf.ext.entagged.plugins.audioformats.wav;

import java.io.File;

import net.sf.ext.entagged.libraries.audioformats.interfaces.AudioFormatInterface;
import net.sf.ext.entagged.libraries.audioformats.interfaces.EncodingInfoReader;
import net.sf.ext.entagged.libraries.audioformats.interfaces.TagReaderInterface;
import net.sf.ext.entagged.libraries.audioformats.interfaces.TagTypeInterface;
import net.sf.ext.entagged.plugins.audioformats.wav.inforeader.WAVEncodingInfoReader;

public class AudioFormat implements AudioFormatInterface{

	private static AudioFormatInterface wav = new AudioFormat();
	private static File rootFile;
		
	public TagTypeInterface[] getSupportedTagTypes() {
		return new TagTypeInterface[] {WAVTagType.getInstance()};
	}
	
	public TagReaderInterface[] getTagReaders() {
		return new TagReaderInterface[]{};
	}
	
	public TagTypeInterface getTagType(String tagType) {
		if(tagType.equals(WAVTag.TAGTYPE))
			return WAVTagType.getInstance();
		else return null;
	}
	
	public String getName() {
		return "WAV pseudo-Tags";
	}
	
	public String[] getSupportedExtensions() {
		return new String[]{"wav"};
	}
	
	public EncodingInfoReader getEncodingInfoReader() {
		return new WAVEncodingInfoReader();
	}

	public TagTypeInterface getDefaultTagType() {
		return WAVTagType.getInstance();
	}
	
	public void setRootFile(File f) {
		rootFile = f;
	}
	
	public File getRootFile() {
		return rootFile;
	}
	
	
	public static AudioFormatInterface getInstance() {
		return wav;
	}
	
	public String toString() {
		return getName();
	}
	
	public String getAuthor() {
		return "Raphael Slinckx (KiKiDonK)";
	}
	
	public String getDescription() {
		return "This is not a real WAV tagging plugin, but rather a simulation, so\nyou can search freedb against these wav files\nand then rename them or whatever, just like normal files.\nWhen you re-read the files, tag information will get lost, be warned !\n\nThis plugin don't do any file modification...";
	}
}
