/*
 *  ********************************************************************   **
 *  Copyright notice                                                       **
 *  **																	   **
 *  (c) 2003 Entagged Developpement Team				                   **
 *  http://www.sourceforge.net/projects/entagged                           **
 *  **																	   **
 *  All rights reserved                                                    **
 *  **																	   **
 *  This script is part of the Entagged project. The Entagged 			   **
 *  project is free software; you can redistribute it and/or modify        **
 *  it under the terms of the GNU General Public License as published by   **
 *  the Free Software Foundation; either version 2 of the License, or      **
 *  (at your option) any later version.                                    **
 *  **																	   **
 *  The GNU General Public License can be found at                         **
 *  http://www.gnu.org/copyleft/gpl.html.                                  **
 *  **																	   **
 *  This copyright notice MUST APPEAR in all copies of the file!           **
 *  ********************************************************************
 */
package net.sf.ext.entagged.plugins.audioformats.wav;

import java.nio.ByteBuffer;
import java.util.Enumeration;
import java.util.Hashtable;

import net.sf.ext.entagged.libraries.audioformats.interfaces.Tag;


public class WAVTag implements Tag {
	private Hashtable ht;

	public final static String TAGTYPE = "wav";


	public WAVTag() {
		ht = new Hashtable();
	}


	public String getAlbum() {
		return (String) ht.get( "ALBUM" );
	}


	public String getArtist() {
		return (String) ht.get( "ARTIST" );
	}


	public ByteBuffer getByteBufferRepresentation( int paddingSize ) {
		return null;
	}


	public String getComment() {
		return (String) ht.get( "COMMENT" );
	}


	public String getGenre() {
		return (String) ht.get( "GENRE" );
	}


	public int getTagLength() {
		return -1;
	}


	public String getTagType() {
		return TAGTYPE;
	}


	public String getTitle() {
		return (String) ht.get( "TITLE" );
	}


	public String getTrackNumber() {
		return (String) ht.get( "TRACKNUMBER" );
	}


	public String getYear() {
		return (String) ht.get( "DATE" );
	}
	
	public boolean isEmpty() {
		Enumeration e = ht.keys();
		while (e.hasMoreElements()) {
			String key = (String) e.nextElement();
			String content = (String) ht.get(key);
			if(!content.equals(""))
				return false;
		}
		return true;
	}
	

	public void setTitle(String title) {
		ht.put("TITLE",title);
	}


	public void setAlbum(String album) {
		ht.put("ALBUM",album);
	}


	public void setArtist(String artist) {
		ht.put("ARTIST",artist);
	}


	public void setGenre(String genre) {
		ht.put("GENRE",genre);
	}
	
	public void setTrackNumber(String trackNumber) {
		ht.put("TRACKNUMBER",trackNumber);
	}

	public void setYear(String year) {
		ht.put("DATE",year);
	}


	public void setComment(String comment) {
		ht.put("COMMENT",comment);
	}


	/**
	 * @return    the string representation of this tag
	 */
	public String toString() {
		String output = "WAVTag------------------------------------\n";
		output += "Artiste: " + getArtist() + "\tAlbum: " + getAlbum() + "\n";
		output += "Track n?" + getTrackNumber() + " " + getTitle() + "\n";
		output += "Ann?e: " + getYear() + "\tGenre: " + getGenre() + "\n";
		output += "Comments: -" + getComment() + "-\n";
		output += "-----------------------------------------------------\n";
		return output;
	}
}

