/*
 *  ********************************************************************   **
 *  Copyright notice                                                       **
 *  **																	   **
 *  (c) 2003 Entagged Developpement Team				                   **
 *  http://www.sourceforge.net/projects/entagged                           **
 *  **																	   **
 *  All rights reserved                                                    **
 *  **																	   **
 *  This script is part of the Entagged project. The Entagged 			   **
 *  project is free software; you can redistribute it and/or modify        **
 *  it under the terms of the GNU General Public License as published by   **
 *  the Free Software Foundation; either version 2 of the License, or      **
 *  (at your option) any later version.                                    **
 *  **																	   **
 *  The GNU General Public License can be found at                         **
 *  http://www.gnu.org/copyleft/gpl.html.                                  **
 *  **																	   **
 *  This copyright notice MUST APPEAR in all copies of the file!           **
 *  ********************************************************************
 */
package net.sf.ext.entagged.plugins.audioformats.wav.inforeader;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;

import net.sf.ext.entagged.libraries.audioformats.EncodingInfo;
import net.sf.ext.entagged.libraries.audioformats.interfaces.EncodingInfoReader;
import net.sf.ext.entagged.libraries.exceptions.CannotReadException;


public class WAVEncodingInfoReader implements EncodingInfoReader {
	private EncodingInfo encodingInfo;

	private MappedByteBuffer mbb;


	public EncodingInfo read( File f )  throws CannotReadException{
		try {
			RandomAccessFile raf = new RandomAccessFile( f, "r" );

			mbb = raf.getChannel().map( FileChannel.MapMode.READ_ONLY, 0, raf.getChannel().size() );
			raf.close();
		} catch ( IOException e ) {
			//e.printStackTrace();
			throw new CannotReadException("Error: Could not access the file -"+f+"- :\n"+e);
		}

		encodingInfo = new EncodingInfo();
		readFileHeader( f );

		mbb = null;
		
		if(!encodingInfo.isValid( ))
			throw new CannotReadException("Error: Could not extract valid basic infos over the file -"+f+"- :\n");

		return encodingInfo;
	}


	public String toString() {
		return "WAV encoding info reader";
	}

	
	private void readFileHeader( File f ) {
		mbb.position( 0 );
		
		byte[] b = new byte[12];
		mbb.get(b);
		
		WAVRIFFHeader wh = new WAVRIFFHeader(b);
		if(wh.isValid()) {
			b = new byte[24];
			mbb.get(b);
			
			WAVFormatHeader wfh = new WAVFormatHeader(b);
			if(wfh.isValid()){
				//Populates encodingInfo----------------------------------------------------
				//System.out.println(f.length());
				encodingInfo.setTimelength( (int) (f.length() / wfh.getBytesPerSecond()));
				//System.out.println((int) (f.length() / wfh.getBytesPerSecond()));
				encodingInfo.setChannelMode( wfh.getChannelMode() );
				encodingInfo.setSamplingRate( wfh.getSamplingRate() );
				encodingInfo.setEncodingType( "WAV-RIFF " + wfh.getBitrate() +" bits" );
				encodingInfo.setExtraEncodingInfos( "" );
				encodingInfo.setBitrate( wfh.getBytesPerSecond()*8/1000 );
				encodingInfo.setValid( true );
				//System.out.println(encodingInfo);
			}
		}
		mbb.rewind();
	}
}

