/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   * 
 *                           sub-project: JFlexSett                           *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/


package net.sf.ext.jflexsett.settings.ui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTree;
import javax.swing.border.EtchedBorder;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeSelectionModel;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.Layout;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.resources.DCLabels;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.jflexsett.settings.SettingsFile;
import net.sf.ext.jflexsett.settings.SettingsGroup;
import net.sf.ext.util.Resources;

/**
 * Tree view with panels. Loads and saves settings
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class SettingsView extends JFrame {

    protected JTree tree;
    private InformationPanel panelInfo; 
    private JPanel panelActions;
    
    private JButton buttonSaveCurrent;
    private JButton buttonSaveAll;
    private JButton buttonClose;

    public SettingsView(String sTitle) {
        buildView();   
        this.setTitle(sTitle);
    }

    private void buildView() {
        try {
            setResizable(false);
            addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    closeView();
                }
            });
            Image image = IconLibrary._icoSettings.getImage();
            setIconImage(image);
            image.flush();
        } catch (Error err) {
        }

        panelInfo = new InformationPanel();
        getContentPane().setLayout(Layout.getGBL());
        tree = new JTree(buildTreeModel());
        tree.setFont(ComponentFactory.getBoldFont());
        tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.addTreeSelectionListener(new NodeSelectedAction());
        tree.setBorder(new EtchedBorder());
        tree.setPreferredSize(new Dimension(200, 400));
        
        getContentPane().add(tree,  Layout.getGBC( 0, 0, 1, 2, 40.0, 40.0
                            ,GridBagConstraints.WEST, GridBagConstraints.BOTH,
                             new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(getActionPanel(),  Layout.getGBC( 1, 1, 1, 1, 40.0, 40.0
                            ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                             new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(panelInfo,  Layout.getGBC( 1, 1, 1, 1, 40.0, 40.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(5, 5, 5, 5), 0, 0));

        initializeSettings();
        this.setResizable(true);
        this.pack();
        
        setLocation(100, 100);
    }
    
    protected void closeView() {
        Settings.setValue(Repository.settings.stSettingsViewSize, this.getSize());
        SettingsFile.save();  
        this.dispose();
    }

    /**
     * Creates the action panel (save buttons and such)
     */    
    private JPanel getActionPanel() {
        panelActions = new JPanel(false);
        
        buttonSaveCurrent = ComponentFactory.getButton(Resources.getText(DCLabels.lblSaveCurrent));
        buttonSaveAll = ComponentFactory.getButton(Resources.getText(DCLabels.lblSaveAll));
        buttonClose = ComponentFactory.getButton(Resources.getText(DCLabels.lblClose));
        
        panelActions.add(buttonSaveCurrent,  Layout.getGBC( 0, 0, 1, 1, 40.0, 40.0
                                      ,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                                       new Insets(5, 5, 5, 5), 0, 0));       
        panelActions.add(buttonSaveAll,Layout.getGBC( 1, 0, 1, 1, 40.0, 40.0
                                      ,GridBagConstraints.EAST, GridBagConstraints.BOTH,
                                       new Insets(5, 5, 5, 5), 0, 0));       
        panelActions.add(buttonClose,  Layout.getGBC( 1, 0, 1, 1, 80.0, 80.0
                                      ,GridBagConstraints.WEST, GridBagConstraints.BOTH,
                                       new Insets(5, 5, 5, 5), 0, 0));  
        panelActions.setVisible(false);
                                       
        buttonSaveAll.addActionListener(new SaveAllSettingsAction());
        buttonSaveCurrent.addActionListener(new SaveCurrentVisibleSettingsAction());
        buttonClose.addActionListener(new CloseAction());
        
        return panelActions;
    }

    public void setBackgroundColor(Color color) {
        this.getContentPane().setBackground(color);
        panelActions.setBackground(color);
        panelInfo.setBackground(color);
        Vector panels = getAllPanels();
        for (Enumeration enum = panels.elements(); enum.hasMoreElements(); ) {
            JPanel panel = (JPanel) enum.nextElement();
            panel.setBackground(color);
            
            Component[] comps = panel.getComponents();
            for (int i = 0; i < comps.length; i++) {
                comps[i].setBackground(color);
            }
        }
    }

    public void setDisclaimer(String sText) {
        panelInfo.setText(sText);
    }

    /**
     * Information panel, holder of the disclaimer information
     */    
    public class InformationPanel extends JPanel {
        JTextArea textInfo = new JTextArea();
        JScrollPane scrollInfo = null;
        
        public InformationPanel() {
            textInfo.setFont(new Font("Courier New", Font.PLAIN, 12));
            textInfo.setEditable(false);
            textInfo.setWrapStyleWord(true);
            
            textInfo.setBackground(new Color(204,204,204));
    
            scrollInfo = new JScrollPane(textInfo);
            scrollInfo.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
            scrollInfo.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
    
            // Build the panel
            this.setLayout(Layout.getGBL());
            this.add(scrollInfo, Layout.getGBC( 0, 0, 1, 1, 10.0, 10.0
                                    , GridBagConstraints.CENTER, GridBagConstraints.BOTH,
                                      new Insets( 5, 5, 5, 5), 0, 0));
        }
        
        
        public void setText(String sText) {
            textInfo.setText(sText);
        }
        
        public String toString() {
            return "Settings";
        }
    }

    /**
     * Puts the current values of the settings in the panel
     */
    private void initializeSettings() {
        Vector vPanels = getAllPanels();
        for (Enumeration enum = vPanels.elements(); enum.hasMoreElements(); ) {
            SettingsPanel panel = (SettingsPanel) enum.nextElement();
            panel.initializeSettings();
        }
    }

    /**
     * Builds the tree model by reading the settings groups.
     * Each settingsgroup can contain several childeren, which are
     * added as leafs of the parent in the tree.
     * The settings panels are instantiated here as well.
     * Each leaf contains a specific panel with the settings components.
     */
    private DefaultMutableTreeNode buildTreeModel() {
        DefaultMutableTreeNode topTreeNode =
            new DefaultMutableTreeNode("Settings");
        topTreeNode.setUserObject(panelInfo);

        Hashtable htGroups = Settings.getSettingsGroups();
        Vector vChildGroups = new Vector();
        for (Enumeration enum = htGroups.elements(); enum.hasMoreElements(); ) {
            SettingsGroup group = (SettingsGroup) enum.nextElement();
            JPanel panel = new SettingsPanel(group);
            DefaultMutableTreeNode treeNode = new DefaultMutableTreeNode(panel);

            // Add the treenode to the top note
            topTreeNode.add(treeNode);
            // Add the childs to child vector
            vChildGroups.addAll(group.getChildren().values());

            getContentPane().add(panel, Layout.getGBC( 1, 0, 1, 1, 80.0, 80.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                 new Insets(5, 5, 5, 5), 0, 0));
            
            Hashtable htChildren = group.getChildren();
            for (Enumeration enumChilds = htChildren.elements(); enumChilds.hasMoreElements(); ) {
                SettingsGroup childGroup = (SettingsGroup) enumChilds.nextElement();   
                JPanel childPanel = new SettingsPanel(childGroup);
                DefaultMutableTreeNode childTreeNode = new DefaultMutableTreeNode(childPanel);
                treeNode.add(childTreeNode);

                getContentPane().add(childPanel, Layout.getGBC( 1, 0, 1, 1, 80.0, 80.0
                                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                     new Insets(5, 5, 5, 5), 0, 0));
            }
        }
        return topTreeNode;
    }

    /**
     * Saves all settings as they are defined in the panels
     */
    protected void saveAllSettings() {
        Vector vPanels = getAllPanels();
        for (Enumeration enum = vPanels.elements(); enum.hasMoreElements(); ) {
            SettingsPanel panel = (SettingsPanel) enum.nextElement();
            panel.saveSettings();
        }
        DataCrow.mainFrame.setRequiredFields();
    }    

    /**
     * Saves the settings of the current visible panel
     */
    protected void saveCurrentVisibleSettings() {
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                                       tree.getLastSelectedPathComponent();

        if (node.getUserObject() instanceof SettingsPanel) {
            SettingsPanel panel = (SettingsPanel) node.getUserObject();
            panel.saveSettings();
        }
        DataCrow.mainFrame.setRequiredFields();
    }
    
    private Vector getAllPanels() {
        Vector vPanels = new Vector();
        DefaultMutableTreeNode oRoot = (DefaultMutableTreeNode) tree.getModel().getRoot();
        for (Enumeration enum = oRoot.children(); enum.hasMoreElements(); ) {
            DefaultMutableTreeNode oCurrent = (DefaultMutableTreeNode) enum.nextElement();
            SettingsPanel nodePanel = (SettingsPanel) oCurrent.getUserObject();
            vPanels.addElement(nodePanel);

            for (Enumeration enumChilds = oCurrent.children(); enumChilds.hasMoreElements(); ) {
                DefaultMutableTreeNode oChild = (DefaultMutableTreeNode) enumChilds.nextElement();
                SettingsPanel nodePanelChild = (SettingsPanel) oChild.getUserObject();
                vPanels.addElement(nodePanelChild);
            }
        }
        return vPanels;
    }

    protected void setVisible(JPanel panel) {
        Vector vPanels = getAllPanels();
        for (Enumeration enum = vPanels.elements(); enum.hasMoreElements(); ) {
            SettingsPanel nodePanel = (SettingsPanel) enum.nextElement();
            if (nodePanel.equals(panel)) {
                nodePanel.setVisible(true);
            } else {
                nodePanel.setVisible(false);
            }
        }
    }    

    protected void setPanelsVisible(boolean visible) {
        Vector vPanels = getAllPanels();
        for (Enumeration enum = vPanels.elements(); enum.hasMoreElements(); ) {
            SettingsPanel nodePanel = (SettingsPanel) enum.nextElement();
            nodePanel.setVisible(visible);
        }
    }

    private class NodeSelectedAction implements TreeSelectionListener {
        public void valueChanged(TreeSelectionEvent e) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                                           tree.getLastSelectedPathComponent();
            if (node == null) return;
            Object nodeInfo = node.getUserObject();

            if (nodeInfo.equals(panelInfo)) {
                JPanel panel = (JPanel) nodeInfo;
                panel.setVisible(true);
                setPanelsVisible(false);
                panelActions.setVisible(false);
            } else if (nodeInfo instanceof SettingsPanel) {
                SettingsPanel panel = (SettingsPanel) nodeInfo;
                setVisible(panel);
                panelInfo.setVisible(false);
                panelActions.setVisible(true);
            } 
        }
    }    

    public class SaveCurrentVisibleSettingsAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            saveCurrentVisibleSettings();
        }
    }    
    
    public class SaveAllSettingsAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            saveAllSettings();
        }
    }    

    public class CloseAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            closeView();
        }
    }  
}
