/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.ext.musicorganizer;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import net.sf.dc.console.MessageBox;
import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.FileField;
import net.sf.dc.console.components.Layout;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.ext.helliker.id3.MP3File;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.DirectoryReader;
import net.sf.ext.util.SwingUtil;
import net.sf.ext.util.Utilities;

/**
 * UI
 *
 * @author Robert-Jan van der Waals
 * @since 1.4
 * @version 0.1
 * 
 * TODO: Change everything ;)
 */
public class OrganizeMp3UI extends JFrame {

    private final String[] sortBy = {" ", "genre", "album", "artist"};

    private JPanel  mainPanel = new JPanel(false);
    private JPanel  actionPanel = new JPanel(false);
    private JPanel  logPanel = new JPanel(false);
    private JPanel  progressPanel = new JPanel(false);  

    private boolean running = false;

    Thread  organize;

    private JButton buttonStop;
    private JButton buttonGo;
    private JLabel labelLevelOne;
    private JComboBox comboLevelOne;

    private JLabel labelOutput;
    private JLabel labelInput;
    private FileField fieldOutput;
    private FileField fieldInput;

    private JTextArea textLog;

    private JProgressBar progress;

    /**
     * Builds and initializes the UI
     */
    public OrganizeMp3UI() {
        SwingUtil.setRootFrame(this);

        Color backgroundColor = (Color) Settings.getValue(Repository.settings.stColor);
        setBackgroundColor(backgroundColor);

        setTitle("MP3 Organizer");
        setResizable(false);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                Settings.setValue(Repository.settings.stMusicOrganizerViewSize,
                                  getSize());
                stop();
            }
        });

        Image image = IconLibrary._icoMp3Organizer.getImage();
        this.setIconImage(image);
        image.flush();

        Color color = Settings.getValueAsColor(Repository.settings.stColor);

        mainPanel = createMainPanel();
        mainPanel.setBackground(color);

        actionPanel = createActionPanel();
        actionPanel.setBackground(color);

        logPanel = createLogPanel();
        logPanel.setBackground(color);

        progressPanel = createProgressPanel();
        progressPanel.setBackground(color);

        this.getContentPane().setLayout(Layout.getGBL());
        this.getContentPane().setBackground(color);

        getContentPane().add(mainPanel,     Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(actionPanel,   Layout.getGBC( 0, 1, 1, 1, 0.0, 0.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                             new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(logPanel,      Layout.getGBC( 0, 2, 1, 1, 40.0, 40.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(progressPanel, Layout.getGBC( 0, 3, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets(5, 5, 5, 5), 0, 0));


        this.setResizable(true);
        pack();
        this.setSize(Settings.getValueAsDimension(
                     Repository.settings.stMusicOrganizerViewSize));
                     
        Dimension location = Utilities.getCenteredWindowLocation(getSize());
        setLocation(location.width, location.height);
    }

    public void stop() {
        running = false;
        dispose();
    }

    /**
     * Starts the <url>OrganizeThread</url>.
     * The output and input locations are verified and a check is done
     * whether a sort by criteria has been specified.
     * Messages are displayed to the user by using a messagebox.
     */
    private void startMoveAction() {
        try {
            String[] sSortBy = null;

            if (comboLevelOne.getSelectedIndex() < 1) {
                new MessageBox( "Please select ordering type",
                                MessageBox._INFORMATION);
                return;
            } else {
                sSortBy = new String[1];
                sSortBy[0] = comboLevelOne.getSelectedItem().toString();
            }
            String sInputDirectory = fieldInput.getFileAsString() + File.separator;
            String sOutputDirectory = fieldOutput.getFileAsString() + File.separator;
            File fileIn = new File(sInputDirectory);

            if (!fileIn.exists() || !fileIn.isDirectory()) {
                new MessageBox("Please select a valid input / output directory",
                               MessageBox._INFORMATION);
                return;
            }

            organize = new Thread(new OrganizeThread(sSortBy,
                                                     sInputDirectory,
                                                     sOutputDirectory));
            organize.setPriority(1);
            organize.setDaemon(true);
            organize.start();

        } catch (Exception exp) {
            setLogText("Error occured;" + '\n' + exp.toString());
        }
    }

    /**
     * Writes a message to the top of the log field
     *
     * @param text  event Strings
     */
    public void setLogText(String text) {
        textLog.setText(text + '\n' + textLog.getText());
    }

    /**
     * Creates the main panel
     */
    public JPanel createMainPanel() {
        // Initialize Components
        labelLevelOne       = ComponentFactory.getLabel("Ordering type");
        comboLevelOne       = ComponentFactory.getComboBox();

        for (int i = 0; i < sortBy.length; i++) {
            comboLevelOne.addItem(sortBy[i]);
        }

        labelOutput = ComponentFactory.getLabel("Output Directory");
        labelInput = ComponentFactory.getLabel("Input Directory");
        fieldOutput = new FileField(false, true);
        fieldInput = new FileField(false, true);

        labelOutput.setToolTipText("If the output directory is not selected, the input directory will also be used as the output directory");

        mainPanel.setLayout(Layout.getGBL());
        mainPanel.add(labelLevelOne, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                      new Insets(0, 0, 0, 0), 0, 0));
        mainPanel.add(comboLevelOne, Layout.getGBC( 1, 0, 2, 1, 1.0, 1.0
                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                      new Insets(0, 0, 0, 0), 0, 0));
        mainPanel.add(labelInput,    Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                      new Insets(0, 0, 0, 0), 0, 0));
        mainPanel.add(fieldInput,    Layout.getGBC( 1, 1, 1, 1, 10.0, 10.0
                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                      new Insets(0, 0, 0, 0), 0, 0));
        mainPanel.add(labelOutput,   Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                      new Insets(0, 0, 0, 0), 0, 0));
        mainPanel.add(fieldOutput,   Layout.getGBC( 1, 2, 1, 1, 10.0, 10.0
                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                      new Insets(0, 0, 0, 0), 0, 0));

        return mainPanel;
    }

    public JPanel createProgressPanel() {
        progress = new JProgressBar();
        progressPanel.setLayout(Layout.getGBL());
        progressPanel.add(progress, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                          new Insets(5, 5, 5, 5), 0, 0));

        return progressPanel;
    }

    /**
     * Create the action panel
     */
    public JPanel createActionPanel() {
        buttonGo = ComponentFactory.getButton("Go");
        buttonStop = ComponentFactory.getButton("Stop");
        
        buttonGo.setMnemonic(KeyEvent.VK_G);
        buttonStop.setMnemonic(KeyEvent.VK_S);
        
        buttonGo.addActionListener(new StartMoveAction());
        buttonStop.addActionListener(new StopAction());

        actionPanel.add(buttonGo);
        actionPanel.add(buttonStop);

        buttonStop.setEnabled(false);

        return actionPanel;
    }

    /**
     * Creates the logging panels
     */
    public JPanel createLogPanel() {
        textLog = ComponentFactory.getTextArea();
        JScrollPane scroll = new JScrollPane(textLog);

        scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        scroll.setPreferredSize(new Dimension(550, 300));

        textLog.setEditable(false);

        logPanel.setLayout(Layout.getGBL());

        logPanel.add(scroll, Layout.getGBC( 0, 0, 7, 1, 40.0, 40.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                     new Insets(0, 0, 0, 0), 0, 0));

        return logPanel;
    }

    private class StartMoveAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startMoveAction();
        }
    }

    private class StopAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            running = false;
            buttonGo.setEnabled(true);
            buttonStop.setEnabled(false);
        }
    }

    /**
     * Applies the background color to the sub panels
     *
     * @param color     background color
     */
    public void setBackgroundColor(Color color) {
        this.setBackground(color);

        try {
            setCompBackColors(this.getComponents(), color);
        } catch (java.lang.Exception exp) {
        }
    }

    /**
     * Applies the background color to all the components
     *
     * @param comps     the components on which the color is applied
     * @param color     background color
     */
    private void setCompBackColors(Component[] comps, Color color) {
        try {
            for (int i = 0; i < comps.length; i++) {
                comps[i].setBackground(color);
            }
        } catch (java.lang.Exception exp) {
        }
    }

    private class OrganizeThread extends Thread {

        private String sSourceDir;
        private String sTargetDir;

        private boolean  bSameTarget;
        private String[] sortBy;

        /**
         * Initializes the process
         *
         * @param sortby        the sort criteria
         * @param sSourceDir    source directory (input)
         * @param sTargetDir    target directory (output)
         */
        public OrganizeThread(String[] sortBy, String sSourceDir, String sTargetDir) {
            if (sortBy == null || sortBy.length < 1) {
                return;
            }

            this.sortBy     = sortBy;
            this.sSourceDir = sSourceDir;
            if (sTargetDir == null || sTargetDir.trim().equals("")) {
                bSameTarget = true;
            } else {
                this.sTargetDir = sTargetDir;
                bSameTarget = false;
            }
        }

        /**
         * Starts the process
         */
        public void run() {
            if (sortBy == null) {
                return;
            }

            for (int i = 0; i < sortBy.length; i++) {
                setLogText("> Starting process");
                boolean byArtist   = sortBy[i].toLowerCase().equals("artist");
                boolean byAlbum    = sortBy[i].toLowerCase().equals("album");
                boolean byGenre    = sortBy[i].toLowerCase().equals("genre");
                start(byArtist, byAlbum, byGenre);
            }
            setLogText("> Ending process");
        }

        /**
         * Searches for Mp3 files in the source location and moves them to the target
         * location
         *
         * @param byArtist  sort by artist
         * @param byAlbum   sort by album
         * @param byTitle   sort by title
         */
        private void start(boolean byArtist, boolean byAlbum, boolean byGenre) {
            try {
                Vector  vFiles  = DirectoryReader.read(sSourceDir, true, true, "mp3");
                if (vFiles == null || vFiles.size() < 1) {
                    setLogText("No files found to organize!");
                } else {
                    moveFiles(vFiles, byArtist, byAlbum, byGenre);
                }
            } catch (Exception exp) {
                exp.printStackTrace();
                System.exit(0);
            }
        }

        /**
         * Moves the Mp3 files to the target directory + sort criteria value
         *
         * @param vFiles    mp3 file locations
         * @param byArtist  sort by artist
         * @param byAlbum   sort by album
         * @param byTitle   sort by title
         */
        private void moveFiles(Vector vFiles, boolean byArtist, boolean byAlbum, boolean byGenre) {
            File             fCurrentFile = null;
            String           sCurrentFile = null;

    		progress.setMinimum(0);
            progress.setMaximum(vFiles.size());
            progress.setValue(0);
            buttonGo.setEnabled(false);
            buttonStop.setEnabled(true);
            int i = 1;
            running = true;
            for (Enumeration enum = vFiles.elements(); enum.hasMoreElements(); ) {
                try {
                    if (!running) {
                        return;
                    }
                    sCurrentFile    = (String) enum.nextElement();
                    fCurrentFile    = new File(sCurrentFile);
                    sSourceDir      = sCurrentFile.substring(0, ((sCurrentFile.length()-1) - fCurrentFile.getName().length()));

                    boolean bMoved       = false;
                    String  sDirectory   = null;

                    if (fCurrentFile.isFile() && !fCurrentFile.isDirectory()) {
                        MP3File mp3File     = new MP3File(sCurrentFile);

                        String sAlbum       = mp3File.getAlbum();
                        String sArtist      = mp3File.getArtist();
                        String sGenre       = mp3File.getGenre();

                        String  sNewFile     = null;

                        if (byArtist && (sArtist != null && !sArtist.trim().equals(""))) {
                            sDirectory  = createDir(sArtist);
                            sNewFile    = sDirectory + File.separator + fCurrentFile.getName();
                            bMoved      = moveFile(sCurrentFile, sNewFile);
                        } else if (byAlbum && (sAlbum != null && !sAlbum.trim().equals(""))) {
                            sDirectory  = createDir(sAlbum);
                            sNewFile    = sDirectory + File.separator + fCurrentFile.getName();
                            bMoved      = moveFile(sCurrentFile, sNewFile);
                        } else if (byGenre && (sGenre != null && !sGenre.trim().equals(""))) {
                            sDirectory  = createDir(sGenre);
                            sNewFile    = sDirectory + File.separator + fCurrentFile.getName();
                            bMoved      = moveFile(sCurrentFile, sNewFile);
                        }
                    }

                    if (bMoved) {
                        setLogText("Moved file <" +
                                                 sCurrentFile +
                                                 "> To <" +
                                                 sDirectory +
                                                 ">");
                        fCurrentFile.delete();
                    } else {
                        setLogText("Failed to move file <" +
                                                  sCurrentFile +
                                                  ">");
                    }
                } catch (Exception exp) {
                    setLogText("Error moving file <" +
                                              sCurrentFile +
                                              ">, reason:");
                                              exp.printStackTrace();
                } finally {
                    progress.setValue(i++);
                }
            }
            buttonGo.setEnabled(true);
            buttonStop.setEnabled(false);
        }

        /**
         * Moves one mp3 file to the target directory
         *
         * @param sOldFile  the old filename (absolute path)
         * @param sNewFile  new filename (absolute path)
         */
        private boolean moveFile(String sOldFile, String sNewFile) {
            try {
                sleep(1000l);

                final FileInputStream  fis = new FileInputStream(sOldFile);
                final FileOutputStream fos = new FileOutputStream(sNewFile);
                byte[]  b = new byte[fis.available()];
                int     k = 0;
                do {
                    k = fis.read(b);
                    if (k != -1) {
                        fos.write(b);
                    }
                } while (k > 0);

                sleep(1000l);

                fos.close();
                fis.close();
            } catch (Exception exp) {
                return false;
            }
            return true;
        }

        /**
         * Creates a directory
         *
         * @param sDirectory    new directory name
         */
        private String createDir(String sDirectory) {
            File    fDirectory  = null;
            try {
                if (bSameTarget) {
                    fDirectory = new File(sSourceDir);
                    fDirectory.mkdir();

                    fDirectory  = new File(sSourceDir + File.separator + sDirectory.trim());
                    fDirectory.mkdir();
                } else {
                    fDirectory = new File(sTargetDir);
                    fDirectory.mkdir();

                    fDirectory  = new File(sTargetDir + File.separator + sDirectory.trim());
                    fDirectory.mkdir();
                }
            } catch (Exception exp) {
                exp.printStackTrace();
            }
            return fDirectory.getAbsolutePath();
        }
    }
}
