/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.ext.util;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.RandomAccessFile;
import java.net.URL;
import java.util.Calendar;
import java.util.Random;

import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DCMessages;

/**
 * General utilities
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class Utilities {

    private static Random random = new Random();
    private static String cacheDirectory;
    
    static {
        try {
        	File fl = new File (".");
        	cacheDirectory = fl.getCanonicalFile() + 
			                 File.separator + 
							 "cache" + 
							 File.separator;
        } catch (Exception exp) {
        	Messages.setMessage(exp);
        }
    }

    public static Dimension getCenteredWindowLocation(Dimension windowSize) {
        Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
        dim.height = (dim.height-windowSize.height)/2;
        dim.width = (dim.width-windowSize.width) /2;
        return dim;
    }


    public static String getUniqueID() {
        Calendar calendar = Calendar.getInstance();
        
        long temp = random.nextLong();
        if (temp < 0l) {
            temp = temp * -1;
        }
        
        String month = "" + (calendar.get(Calendar.MONTH) + 1 );
        if (month.length() == 1) {
            month = "0" + month;
        }
        
        String dayOfMonth = "" + calendar.get(Calendar.DAY_OF_MONTH);
        if (dayOfMonth.length() == 1) {
            dayOfMonth = "0" + dayOfMonth;
        }
        
        return "" + calendar.get(Calendar.YEAR) + month + dayOfMonth + temp;
    }
    
    public static String getExtension(File f) {
        String name = f.getName().toLowerCase();
        int i = name.lastIndexOf( "." );
        if (i == -1) {
            return "";
        }
        return name.substring( i + 1 );
    }    
    
    public static String getCompareStringForFile(URL url) throws Exception{
        RandomAccessFile access = new RandomAccessFile(url.getFile(), "rw");
        byte[] byties = new byte[100];
        access.readFully(byties);
        access.close();
        String s = String.valueOf(Base64.encode(byties));
        return s.substring(0, s.length() - 2);
    }
    
    public static byte[] readFile(File file) throws Exception {
        RandomAccessFile access = new RandomAccessFile(file, "rw");
        byte[] byties = new byte[new Long(access.length()).intValue()];
        access.readFully(byties);
        access.close();
        return byties;        
    }

    public static URL createCachedImage(String s) {
        try {
        	if (cacheDirectory == null) {
        		throw new RuntimeException("Cache directory does not exist! Severe error !");
            }
        	String sFileName = cacheDirectory + Utilities.getUniqueID() + ".jpg";
        	return createCachedImage(s, sFileName);
        } catch (Exception exp) {
            Messages.setMessage(Resources.getText(DCMessages.msgCouldNotCreateCacheForPicture),
                                Messages._ERROR);
        	Messages.setMessage(exp);
            return null;
        }
    }
    
    public static URL createCachedImage(String s, String fileName) {
        byte[] b = null;
        try {
        	b = Base64.decode(s.toCharArray());
        } catch (Exception exp) {
            /**Messages.setMessage(Resources.getText(DCMessages.msgCouldNotCreateCacheForPicture),
            		            Messages._ERROR);
            		            Messages.setMessage(exp); */
        	return null;
        }
        
        URL url = null;
        try {
            File flCache = new File("." + File.separator + "cache");
            if (!flCache.exists()) {
            	flCache.mkdirs();
                flCache = null;
            }
            File fl = new File(fileName);
            FileOutputStream out = new FileOutputStream(fl.toString());
            BufferedOutputStream bfrOut = new BufferedOutputStream(out);
            bfrOut.write(b);
            bfrOut.flush();
            out.close();
            bfrOut.close();
            b = null;
            url = fl.toURL();
        } catch (Exception exp) {
            Messages.setMessage(Resources.getText(DCMessages.msgCouldNotCreateCacheForPicture),
            		            Messages._ERROR);
            Messages.setMessage(exp);
        }
        return url;
    }

    public static String getCacheDirectory() {
    	return cacheDirectory;
    }
    
    public static String fileToBase64String(URL url) {
        try {
            if (url != null) {
                File file = new File(url.getFile());
                byte[] b = Utilities.readFile(file);
                file = null;
                return String.valueOf(Base64.encode(b));
            }
        } catch (Exception exp) {
            Messages.setMessage(Resources.getText(DCMessages.msgCouldNotLoadPicture),
            		            Messages._ERROR);
            Messages.setMessage(exp);
        }
        return "";
    }
}
